# MonArch - Groundwork Monitor Architect
# MonarchDoc.pm
#
###############################################################################
# Release 0.94c
# 16-May-2005
###############################################################################
#

use strict;

package Doc;

sub properties_doc(@) {
	my $obj = $_[1];
	my $props = $_[2];
	my @props = @{$props};
	my %docs = ();
	foreach my $prop (@props) {

# host template
		if ($obj eq 'host_templates' && $prop eq 'process_perf_data') {
			$docs{$obj eq 'host_templates' && $prop} = q(Process perf data: Optional. This directive is used to determine whether or not the processing of performance data is enabled for this host. Values:  unchecked = disable performance data processing, checked = enable performance data processing.);
		} elsif ($obj eq 'host_templates' && $prop eq 'retain_status_information') {
			$docs{$prop} = q(Retain status information: Optional. This directive is used to determine whether or not status-related information about the host is retained across program restarts. This is only useful if you have enabled state retention using the retain_state_information directive. Value:  unchecked = disable status information retention, checked = enable status information retention.);
		} elsif ($obj eq 'host_templates' && $prop eq 'flap_detection_enabled') {
			$docs{$obj eq 'host_templates' && $prop} = q(Flap detection enabled: Optional. This directive is used to determine whether or not flap detection is enabled for this host. Values:  unchecked = disable host flap detection, checked = enable host flap detection.);
		} elsif ($obj eq 'host_templates' && $prop eq 'low_flap_threshhold') {
			$docs{$prop} = q(Low flap threshhold: Optional. This directive is used to specify the low state change threshold used in flap detection for this host. If you set this directive to a value of 0, the program-wide value specified by the low_host_flap_threshold directive will be used.);
		} elsif ($obj eq 'host_templates' && $prop eq 'high_flap_threshhold') {
			$docs{$prop} = q(High flap threshhold: Optional. This directive is used to specify the high state change threshold used in flap detection for this host. If you set this directive to a value of 0, the program-wide value specified by the high_host_flap_threshold directive will be used.);
		} elsif ($obj eq 'host_templates' && $prop eq 'retain_nonstatus_information') {
			$docs{$prop} = q(Retain nonstatus information: Optional. This directive is used to determine whether or not non-status information about the host is retained across program restarts. This is only useful if you have enabled state retention using the retain_state_information directive. Value:  unchecked = disable non-status information retention, checked = enable non-status information retention.);
		} elsif ($prop eq 'checks_enabled') {
			$docs{$prop} = q(Checks enabled: Optional. This directive is used to determine whether or not checks of this host are enabled. Values:  unchecked = disable host checks, checked = enable host checks.);
		} elsif ($obj eq 'host_templates' && $prop eq 'check_command') {
			$docs{$prop} = q(Host templates: Optional. This directive is used to specify the short name of the command that should be used to check if the host is up or down. Typically, this command would try and ping the host to see if it is 'alive'. The command must return a status of OK (0) or Nagios will assume the host is down. If you leave this argument blank, the host will not be checked - Nagios will always assume the host is up. This is useful if you are monitoring printers or other devices that are frequently turned off. The maximum amount of time that the notification command can run is controlled by the host_check_timeout option.);
		} elsif ($obj eq 'host_templates' && $prop eq 'max_check_attempts') {
			$docs{$prop} = q(Max check attempts: Required. This directive is used to define the number of times that Nagios will retry the host check command if it returns any state other than an OK state. Setting this value to 1 will cause Nagios to generate an alert without retrying the host check again. Note:  If you do not want to check the status of the host, you must still set this to a minimum value of 1. To bypass the host check, just leave the check_command option blank.);
		} elsif ($obj eq 'host_templates' && $prop eq 'event_handler_enabled') {
			$docs{$prop} = q(Event handler enabled: Optional. This directive is used to determine whether or not the event handler for this host is enabled. Values:  unchecked = disable host event handler, checked = enable host event handler.);
		} elsif ($obj eq 'host_templates' && $prop eq 'event_handler') {
			$docs{$prop} = q(Event handler: Optional. This directive is used to specify the short name of the command that should be run whenever a change in the state of the host is detected (i.e. whenever it goes down or recovers). The maximum amount of time that the event handler command can run is controlled by the event_handler_timeout option.);
		} elsif ($obj eq 'host_templates' && $prop eq 'notifications_enabled') {
			$docs{$prop} = q(Notifications enabled: Optional. This directive is used to determine whether or not notifications for this host are enabled. Values:  unchecked = disable host notifications, checked = enable host notifications.);
		} elsif ($obj eq 'host_templates' && $prop eq 'notification_interval') {
			$docs{$prop} = q(Notification interval: Required. This directive is used to define the number of 'time units' to wait before re-notifying a contact that this server is still down or unreachable. Unless you've changed the interval_length directive from the default value of 60, this number will mean minutes. If you set this value to 0, Nagios will not re-notify contacts about problems for this host - only one problem notification will be sent out.);
		} elsif ($obj eq 'host_templates' && $prop eq 'notification_period') {
			$docs{$prop} = q(Notification period: Required. This directive is used to specify the short name of the time period during which notifications of events for this host can be sent out to contacts. If a host goes down, becomes unreachable, or recoveries during a time which is not covered by the time period, no notifications will be sent out.);
		} elsif ($obj eq 'host_templates' && $prop eq 'notification_options') {
			$docs{$prop} = q(Notification options: Required. This directive is used to determine when notifications for the host should be sent out. Valid options are a combination of one or more of the following:  Down checked = send notifications on a DOWN state, Unreachable checked = send notifications on an UNREACHABLE state, and Recovery checked = send notifications on recoveries (OK state). If you specify n (none) as an option, no host notifications will be sent out. Example:  If you specify Down and Recovery, notifications will only be sent out when the host goes DOWN and when it recovers from a DOWN state.);
		} elsif ($obj eq 'host_templates' && $prop eq 'stalking_options') {
			$docs{$prop} = q(Stalking options: Optional. This directive determines which host states 'stalking' is enabled for. Valid options are a combination of one or more of the following:  Up checked = stalk on UP states, Down checked = stalk on DOWN states, and Unreachable checked = stalk on UNREACHABLE states.);

# host dependencies
		} elsif ($prop eq 'dependent_host') {
			$docs{$prop} = q(Dependent_host: Required. This directive is used to identify the short name of the dependent host.);
		} elsif ($prop eq 'parent_host') {
			$docs{$prop} = q(Parent_host: Required. This directive is used to identify the short name of the host that is being depended upon.);
		} elsif ($prop eq 'notification_failure_criteria') {
			$docs{$prop} = q(Notification_failure_criteria: Required. This directive is used to define the criteria that determine when notifications for the dependent host should not be sent out. If the host that is being depended upon is in one of the failure states we specify, notifications for the dependent host will not be sent to contacts. Valid options are a combination of one or more of the following:  o = fail on an UP state, d = fail on a DOWN state, and u = fail on an UNREACHABLE state. If you specify n (none) as an option, the notification dependency will never fail and notifications for the dependent host will always be sent out. Example:  If you specify d in this field, the notifications for the dependent host will not be sent out if the host that is being depended upon is in a DOWN state.);

# host / service ext info
		} elsif ($prop eq 'notes_url') {
			$docs{$prop} = q(notes_url: Optional. This variable is used to define an optional URL that can be used to provide more information about the host. If you specify an URL, you will see a link that says 'Notes About This Host' in the extended information CGI (when you are viewing information about the specified host). Any valid URL can be used. If you plan on using relative paths, the base path will the the same as what is used to access the CGIs (i.e. /cgi-bin/nagios/). This can be very useful if you want to make detailed information on the host, emergency contact methods, etc. available to other support staff. );
		} elsif ($prop eq 'script') {
			$docs{$prop} = q(Script: Optional custom script. Enter full path, file name and command line arguments. Use Control -> Run Extended Info Scripts to execute custom scripts for objects assigned this template. Arguements can include $HOSTNAME$ and $SERVICENAME$ macros.);
		} elsif ($prop eq 'icon_image') {
			$docs{$prop} = q(icon_image: Optional. This variable is used to define the name of a GIF, PNG, or JPG image that should be associated with this host. This image will be displayed in the status and extended information CGIs. The image will look best if it is 40x40 pixels in size. Images for hosts are assumed to be in the logos/ subdirectory in your HTML images directory (i.e. /usr/local/nagios/share/images/logos).);
		} elsif ($prop eq 'icon_image_alt') {
			$docs{$prop} = q(icon_image_alt: Optional. This variable is used to define an optional string that is used in the ALT tag of the image specified by the <icon_image> argument. The ALT tag is used in the status, extended information and statusmap CGIs.);
		} elsif ($prop eq 'vrml_image') {
			$docs{$prop} = q(vrml_image: Optional. This variable is used to define the name of a GIF, PNG, or JPG image that should be associated with this host. This image will be used as the texture map for the specified host in the statuswrl CGI. Unlike the image you use for the <icon_image> variable, this one should probably not have any transparency. If it does, the host object will look a bit wierd. Images for hosts are assumed to be in the logos/ subdirectory in your HTML images directory (i.e. /usr/local/nagios/share/images/logos).);
		} elsif ($prop eq 'statusmap_image') {
			$docs{$prop} = q(statusmap_image: Optional. This variable is used to define the name of an image that should be associated with this host in the statusmap CGI. You can specify a JPEG, PNG, and GIF image if you want, although I would strongly suggest using a GD2 format image, as other image formats will result in a lot of wasted CPU time when the statusmap image is generated. GD2 images can be created from PNG images by using the pngtogd2 utility supplied with Thomas Boutell's gd library. The GD2 images should be created in uncompressed format in order to minimize CPU load when the statusmap CGI is generating the network map image. The image will look best if it is 40x40 pixels in size. You can leave these option blank if you are not using the statusmap CGI. Images for hosts are assumed to be in the logos/ subdirectory in your HTML images directory (i.e. /usr/local/nagios/share/images/logos).);
		} elsif ($prop eq '2d_coords') {
			$docs{$prop} = q(2d_coords: Optional. This variable is used to define coordinates to use when drawing the host in the statusmap CGI. Coordinates should be given in positive integers, as the correspond to physical pixels in the generated image. The origin for drawing (0,0) is in the upper left hand corner of the image and extends in the positive x direction (to the right) along the top of the image and in the positive y direction (down) along the left hand side of the image. For reference, the size of the icons drawn is usually about 40x40 pixels (text takes a little extra space). The coordinates you specify here are for the upper left hand corner of the host icon that is drawn. Note:  Don't worry about what the maximum x and y coordinates that you can use are. The CGI will automatically calculate the maximum dimensions of the image it creates based on the largest x and y coordinates you specify.);
		} elsif ($prop eq '3d_coords') {
			$docs{$prop} = q(3d_coords: Optional. This variable is used to set coordinates to use when drawing the host in the statuswrl CGI. Coordinates can be positive or negative real numbers. The origin for drawing is (0.0,0.0,0.0). For reference, the size of the host cubes drawn is 0.5 units on each side (text takes a little more space). The coordinates you specify here are used as the center of the host cube.);

# hostgroup
		} elsif ($obj eq 'hostgroups' && $prop eq 'alias') {
			$docs{$prop} = q(Alias: Required. This directive is used to define a longer name or description used to identify the host group. It is provided in order to allow you to more easily identify a particular host group.);
		} elsif ($obj eq 'hostgroups' && $prop eq 'members') {
			$docs{$prop} = q(members: Required. This is a list of the short names of hosts that should be included in this group.);
		} elsif ($obj eq 'hostgroups' && $prop eq 'contactgroup') {
			$docs{$prop} = q(contactgroup: Required. This is a list the contact groups that should be notified whenever there are problems (or recoveries) with any of the hosts in this host group.);
		} elsif ($obj eq 'hostgroups' && $prop eq 'hostgroup_escalation_id') {
			$docs{$prop} = q(hostgroup_escalation_id: Optional. Select the hostgrooup escalation tree applicable to this hostgroup (see Design/Manage -> Escalation Trees). (Note Nagios 2.x users may use this directive or the Host escalation id directive to achieve the same result.));
		} elsif ($obj eq 'hostgroups' && $prop eq 'host_escalation_id') {
			$docs{$prop} = q(host_escalation_id: Optional. Select the host escalation tree applicable to this hostgroup in (see Design/Manage -> Escalation Trees). All hosts in the hostgroup receive the same host escalation.);
		} elsif ($obj eq 'hostgroups' && $prop eq 'service_escalation_id') {
			$docs{$prop} = q(service_escalation_id: Optional. Select the service escalation tree applicable to this hostgroup in (see Design/Manage -> Escalation Trees). All services on each host in the hostgroup receive the same host escalation.);

# service template
		} elsif ($obj eq 'service_templates' && $prop eq 'template') {
			$docs{$prop} = q(Use: Optional. Derive values from a parent service template);
		} elsif ($obj eq 'service_templates' && $prop eq 'check_period') {
			$docs{$prop} = q(Check period: Required. This directive is used to specify the short name of the time period during which active checks of this service can be made.);
		} elsif ($obj eq 'service_templates' && $prop eq 'max_check_attempts') {
			$docs{$prop} = q(Max check attempts: Required. This directive is used to define the number of times that Nagios will retry the service check command if it returns any state other than an OK state. Setting this value to 1 will cause Nagios to generate an alert without retrying the service check again.);
		} elsif ($obj eq 'service_templates' && $prop eq 'normal_check_interval') {
			$docs{$prop} = q(Normal check interval: Required. This directive is used to define the number of 'time units' to wait before scheduling the next 'regular' check of the service. 'Regular' checks are those that occur when the service is in an OK state or when the service is in a non-OK state, but has already been rechecked max_attempts number of times. Unless you've changed the interval_length directive from the default value of 60, this number will mean minutes.);
		} elsif ($obj eq 'service_templates' && $prop eq 'retry_check_interval') {
			$docs{$prop} = q(Retry check interval: Required. This directive is used to define the number of 'time units' to wait before scheduling a re-check of the service. Services are rescheduled at the retry interval when they have changed to a non-OK state. Once the service has been retried max_attempts times without a change in its status, it will revert to being scheduled at its 'normal' rate as defined by the check_interval value. Unless you've changed the interval_length directive from the default value of 60, this number will mean minutes.);
		} elsif ($obj eq 'service_templates' && $prop eq 'active_checks_enabled') {
			$docs{$prop} = q(Active checks enabled: Optional. This directive is used to determine whether or not active checks of this service are enabled. Values:  unchecked = disable active service checks, checked = enable active service checks.);
		} elsif ($obj eq 'service_templates' && $prop eq 'passive_checks_enabled') {
			$docs{$prop} = q(Passive checks enabled: Optional. This directive is used to determine whether or not passive checks of this service are enabled. Values:  unchecked = disable passive service checks, checked = enable passive service checks.);
		} elsif ($obj eq 'service_templates' && $prop eq 'parallelize_check') {
			$docs{$prop} = q(Parallelize check: Optional. This directive is used to determine whether or not the service check can be parallelized. By default, all service checks are parallelized. Disabling parallel checks of services can result in serious performance problems. Values:  unchecked = service check cannot be parallelized (use with caution!), checked = service check can be parallelized.);
		} elsif ($obj eq 'service_templates' && $prop eq 'obsess_over_service') {
			$docs{$prop} = q(Obsess over service: );
		} elsif ($obj eq 'service_templates' && $prop eq 'check_freshness') {
			$docs{$prop} = q(Check freshness: Optional. This directive is used to determine whether or not freshness checks are enabled for this service. Values:  unchecked = disable freshness checks, checked = enable freshness checks.);
		} elsif ($obj eq 'service_templates' && $prop eq 'freshness_threshold') {
			$docs{$prop} = q(Freshness threshold: Optional. This directive is used to specify the freshness threshold (in seconds) for this service. If you set this directive to a value of 0, Nagios will determine a freshness threshold to use automatically.);
		} elsif ($obj eq 'service_templates' && $prop eq 'notifications_enabled') {
			$docs{$prop} = q(Notifications enabled: Optional. This directive is used to determine whether or not notifications for this service are enabled. Values:  unchecked = disable service notifications, checked = enable service notifications.);
		} elsif ($obj eq 'service_templates' && $prop eq 'notification_interval') {
			$docs{$prop} = q(Notification interval: Required. This directive is used to define the number of 'time units' to wait before re-notifying a contact that this service is still in a non-OK state. Unless you've changed the interval_length directive from the default value of 60, this number will mean minutes. If you set this value to 0, Nagios will not re-notify contacts about problems for this service - only one problem notification will be sent out, unless there has been a state change.);
		} elsif ($obj eq 'service_templates' && $prop eq 'notification_period') {
			$docs{$prop} = q(Notification period: Required. This directive is used to specify the short name of the time period during which notifications of events for this service can be sent out to contacts. No service notifications will be sent out during times which is not covered by the time period.);
		} elsif ($obj eq 'service_templates' && $prop eq 'notification_options') {
			$docs{$prop} = q(Notification options: Required. This directive is used to determine when notifications for the service should be sent out. Valid options are a combination of one or more of the following: Wawning checked = send notifications on a WARNING state, Unknown checked = send notifications on an UNKNOWN state, Critical checked = send notifications on a CRITICAL state, and Recovery checked = send notifications on recoveries (OK state). If you specify none as an option, no service notifications will be sent out.); 
		} elsif ($obj eq 'service_templates' && $prop eq 'event_handler_enabled') {
			$docs{$prop} = q(Event handler enabled: Optional. This directive is used to determine whether or not the event handler for this service is enabled. Values:  unchecked = disable service event handler, checked = enable service event handler.);
		} elsif ($obj eq 'service_templates' && $prop eq 'event_handler') {
			$docs{$prop} = q(Event handler: Optional. This directive is used to specify the short name of the command that should be run whenever a change in the state of the host is detected (i.e. whenever it goes down or recovers). The maximum amount of time that the event handler command can run is controlled by the event_handler_timeout option. );
		} elsif ($obj eq 'service_templates' && $prop eq 'flap_detection_enabled') {
			$docs{$prop} = q(Flap detection enabled: Optional. This directive is used to determine whether or not flap detection is enabled for this service. Values:  unchecked = disable service flap detection, checked = enable service flap detection.);
		} elsif ($obj eq 'service_templates' && $prop eq 'low_flap_threshhold') {
			$docs{$prop} = q(Low flap threshhold: Optional. This directive is used to specify the low state change threshold used in flap detection for this service. If you set this directive to a value of 0, the program-wide value specified by the low_service_flap_threshold directive will be used.);
		} elsif ($obj eq 'service_templates' && $prop eq 'high_flap_threshhold') {
			$docs{$prop} = q(High flap threshhold: Optional. This directive is used to specify the high state change threshold used in flap detection for this service. If you set this directive to a value of 0, the program-wide value specified by the high_service_flap_threshold directive will be used.);
		} elsif ($obj eq 'service_templates' && $prop eq 'retain_status_information') {
			$docs{$prop} = q(Retain status information: Optional. This directive is used to determine whether or not status-related information about the service is retained across program restarts. This is only useful if you have enabled state retention using the retain_state_information directive. Value:  unchecked = disable status information retention, checked = enable status information retention.);
		} elsif ($obj eq 'service_templates' && $prop eq 'retain_nonstatus_information') {
			$docs{$prop} = q(Retain nonstatus information: Optional. This directive is used to determine whether or not non-status information about the service is retained across program restarts. This is only useful if you have enabled state retention using the retain_state_information directive. Value:  unchecked = disable non-status information retention, checked = enable non-status information retention.);
		} elsif ($obj eq 'service_templates' && $prop eq 'check_command') {
			$docs{$prop} = q(Check command: Optional, sets the default value. Select the command that Nagios will run in order to check the status of the service. );
		} elsif ($obj eq 'service_templates' && $prop eq 'command_line') {
			$docs{$prop} = q(Command line: Optional, sets the default value. If check command requires arguments, enter the check command with command arguments separated by a ! character. Example: check-disk!/dev/sda1); 
		} elsif ($obj eq 'service_templates' && $prop eq 'process_perf_data') {
			$docs{$obj eq 'host_templates' && $prop} = q(Process perf data: Optional. This directive is used to determine whether or not the processing of performance data is enabled for this service. Values:  unchecked = disable performance data processing, checked = enable performance data processing.);

# service_dependency_templates
		} elsif ($obj eq 'service_dependency_templates' && $prop eq 'service_name') {
			$docs{$prop} = q(Service name: Required. Specify the parent service name.);
		} elsif ($obj eq 'service_dependency_templates' && $prop eq 'execution_failure_criteria') {
			$docs{$prop} = q(Execution failure criteria: Optional. This directive is used to specify the criteria that determine when the dependent service should not be executed. If the service that is being depended upon is in one of the failure states we specify, the dependent service will not be executed. Valid options are a combination of one or more of the following:  Okay checked = fail on an OK state, Warning checked = fail on a WARNING state, Unknown checked = fail on an UNKNOWN state, and Critical checked = fail on a CRITICAL state. If you specify n (none) as an option, the execution dependency will never fail and checks of the dependent service will always be executed. Example:  If you specify o,c,u in this field, the dependent service will not be executed if the service that's being depended upon is in either an OK, a CRITICAL, or an UNKNOWN state.);
		} elsif ($obj eq 'service_dependency_templates' && $prop eq 'notification_failure_criteria') {
			$docs{$prop} = q(Notification failure criteria: Optional. This directive is used to define the criteria that determine when notifications for the dependent service should not be sent out. If the service that is being depended upon is in one of the failure states we specify, notifications for the dependent service will not be sent to contacts. Valid options are a combination of one or more of the following:  Okay checked = fail on an OK state, Warning checked = fail on a WARNING state, Unknown checked = fail on an UNKNOWN state, and Critical checked = fail on a CRITICAL state. If you specify n (none) as an option, the notification dependency will never fail and notifications for the dependent service will always be sent out. Example:  If you specify w in this field, the notifications for the dependent service will not be sent out if the service that is being depended upon is in a WARNING state.);

# contact template
		} elsif ($obj eq 'contact_templates' && $prop eq 'host_notification_period') {
			$docs{$prop} = q(Host notification period: Required. This directive is used to specify the short name of the time period during which the contact can be notified about host problems or recoveries. You can think of this as an 'on call' time for host notifications for the contact. Read the documentation on time periods for more information on how this works and potential problems that may result from improper use.);
		} elsif ($obj eq 'contact_templates' && $prop eq 'service_notification_period') {
			$docs{$obj eq 'contact_templates' && $prop} = q(Service notification period: Required. This directive is used to specify the short name of the time period during which the contact can be notified about service problems or recoveries. You can think of this as an 'on call' time for service notifications for the contact. Read the documentation on time periods for more information on how this works and potential problems that may result from improper use.);
		} elsif ($obj eq 'contact_templates' && $prop eq 'host_notification_options') {
			$docs{$prop} = q(Host notification options: Required. This directive is used to define the host states for which notifications can be sent out to this contact. Valid options are a combination of one or more of the following:  Down checked = notify on DOWN host states, Unreachable checked = notify on UNREACHABLE host states, and Recovery checked = notify on host recoveries (UP states). If you specify None as an option, the contact will not receive any type of host notifications.);
		} elsif ($obj eq 'contact_templates' && $prop eq 'service_notification_options') {
			$docs{$prop} = q(Service notification options: Required. This directive is used to define the service states for which notifications can be sent out to this contact. Valid options are a combination of one or more of the following:  Warning checked = notify on WARNING service states, Unknown checked = notify on UNKNOWN service states, Critical checked = notify on CRITICAL service states, and Recovery checked = notify on service recoveries (OK states). If you specify None as an option, the contact will not receive any type of service notifications.);
		} elsif ($obj eq 'contact_templates' && $prop eq 'host_notification_commands') {
			$docs{$prop} = q(Host notification commands: Optional. This directive is used to define a list of the short names of the commands used to notify the contact of a host problem or recovery. All notification commands are executed when the contact needs to be notified. The maximum amount of time that a notification command can run is controlled by the notification_timeout option.);
		} elsif ($obj eq 'contact_templates' && $prop eq 'service_notification_commands') {
			$docs{$prop} = q(Service notification commands: Optional. This directive is used to define a list of the short names of the commands used to notify the contact of a service problem or recovery. All notification commands are executed when the contact needs to be notified. The maximum amount of time that a notification command can run is controlled by the notification_timeout option.);

# contacts 
		} elsif ($obj eq 'contacts' && $prop eq 'template') {
			$docs{$prop} = q(Use: Required, Select template.);
		} elsif ($obj eq 'contacts' && $prop eq 'alias') {
			$docs{$prop} = q(Alias: Required. This directive is used to define a longer name or description for the contact. Under the rights circumstances, the $CONTACTALIAS$ macro will contain this value.);
		} elsif ($obj eq 'contacts' && $prop eq 'email') {
			$docs{$prop} = q(Email: Optional. This directive is used to define an email address for the contact. Depending on how you configure your notification commands, it can be used to send out an alert email to the contact. Under the right circumstances, the $CONTACTEMAIL$ macro will contain this value.);
		} elsif ($obj eq 'contacts' && $prop eq 'pager') {
			$docs{$prop} = q(Pager: Optional. This directive is used to define a pager number for the contact. It can also be an email address to a pager gateway (i.e. pagejoe@pagenet.com). Depending on how you configure your notification commands, it can be used to send out an alert page to the contact. Under the right circumstances, the $CONTACTPAGER$ macro will contact this value.);

# contactgroups
		} elsif ($obj eq 'contactgroups' && $prop eq 'alias') {
			$docs{$prop} = q(Alias: Required. This directive is used to define a longer name or description used to identify the contact group.);
		} elsif ($obj eq 'contactgroups' && $prop eq 'contact') {
			$docs{$prop} = q(Contacts: Required. Assign contacts to the group.);

# timeperiods
		} elsif ($obj eq 'time_periods' && $prop eq 'alias') {
			$docs{$prop} = q(Alias: Required. This directive is a longer name or description used to identify the time period.);
		} elsif ($obj eq 'time_periods' && $prop eq 'sunday') {
			$docs{$prop} = q(Sunday-Saturday: Optional. The sunday through saturday directives are comma-delimited lists of time ranges that are 'valid' times for a particular day of the week. Notice that there are seven different days for which you can define time ranges (Sunday through Saturday). Each time range is in the form of HH: MM-HH: MM, where hours are specified on a 24 hour clock. For example, 00: 15-24: 00 means 12: 15am in the morning for this day until 12: 20am midnight (a 23 hour, 45 minute total time range). If you wish to exclude an entire day from the timeperiod, simply do not include it in the timeperiod definition.);

# commands
		} elsif ($obj eq 'commands' && $prop eq 'type') {
			$docs{$prop} = q(Type: Required. Specify command type: check = service/host check commands, handlers; notify = notification commands; other = other commands (i.e. performance data collectors).);
		} elsif ($obj eq 'commands' && $prop eq 'command_line') {
			$docs{$prop} = q(Command line: Required. This directive is used to define what is actually executed by Nagios when the command is used for service or host checks, notifications, or event handlers. Before the command line is executed, all valid macros are replaced with their respective values. See the documentation on macros for determining when you can use different macros. Note that the command line is not surrounded in quotes. Also, if you want to pass a dollar sign ($) on the command line, you have to escape it with another dollar sign.);

# escalation_templates
		} elsif ($obj =~ /escalation_templates/ && $prop eq 'contactgroup') {
			$docs{$prop} = q(Contactgroup: Required. This directive is used to identify the short name of the contact group that should be notified when the host or service notification is escalated.);
		} elsif ($obj =~ /escalation_templates/ && $prop eq 'first_notification') {
			$docs{$prop} = q(First notification: Required. This directive is a number that identifies the first notification for which this escalation is effective. For instance, if you set this value to 3, this escalation will only be used if the host is down or unreachable long enough for a third notification to go out or if the service is in a non-OK state long enough for a third notification to go out.);
		} elsif ($obj =~ /escalation_templates/ && $prop eq 'last_notification') {
			$docs{$prop} = q(Last notification: Required. This directive is a number that identifies the last notification for which this escalation is effective. For instance, if you set this value to 5, this escalation will not be used if more than five notifications are sent out for the host or service. Setting this value to 0 means to keep using this escalation entry forever (no matter how many notifications go out).);
		} elsif ($obj =~ /escalation_templates/ && $prop eq 'notification_interval') {
			$docs{$prop} = q(Notification interval: Required. This directive is used to determine the interval at which notifications should be made while this escalation is valid. If you specify a value of 0 for the interval, Nagios will send the first notification when this escalation definition is valid, but will then prevent any more problem notifications from being sent out for the host. Notifications are sent out again until the host recovers. This is useful if you want to stop having notifications sent out after a certain amount of time. Note:  If multiple escalation entries for a host overlap for one or more notification ranges, the smallest notification interval from all escalation entries is used.);
		} elsif ($obj =~ /escalation_templates/ && $prop eq 'service_description') {
			$docs{$prop} = q(Service description: Required. This directive is used to identify the description of the service the escalation should apply to.);
		}	
	}
	return %docs;
}

sub service_names() {
	my %docs = ();
	$docs{'description'} = q(Description: Optional. Store comments or instructions here.);
	$docs{'service_template'} = q(Service template: Required.  This option sets the default template for this service name definitions in 'Design-> Host Wizard' and 'Manage -> Host -> Services -> Service Detail'.);
	$docs{'use_template_command'} = q(Use template command: If checked, the check command and command line options are derived from the service template. Be careful to ensure the service template has a check command defined before checking this option.);
	$docs{'check_command'} = q(Check command: If 'Use template command' is unchecked, select the command that Nagios will run in order to check the status of the service.);
	$docs{'command_line'} = q(Command line: If 'Use template command' is unchecked and if check command requires arguments, enter the check command with command arguments separated by a ! character. Example: check-disk!/dev/sda1);
	$docs{'dependency'} = q(Service dependency template: Optional. Select the template that defines a parent service relationship on a given host. Monarch will create a service dependency relationship on a host assigned this service name. To define a dependency for a service running on a different host use 'Manage -> Hosts -> Services -> Service Dependency' and select the parent host. Be careful to ensure that the parent service is assigned to the parent host definition and is included in the relevant service profiles.);
	$docs{'ext_info'} = q(Extended info template: Optional. Defines extended service information for this service name. The template controls the icon and url link as seen in Nagios for hosts assigned this service definition.);
	$docs{'escalation'} = q(Service escalation tree: Optional. Select an escalation tree appropriate for this service name. To avoid amplified notifications (i.e. multiple notifications for the same event), a service escalation assigned to a service name should not also be assigned to the host or the host group.);
	$docs{'externals'} = q(Externals: Custom client specific option.);

	return %docs;
}

sub service_profiles() {
	my %docs = ();
	$docs{'description'} = q(Description: Optional. Store comments or instructions here.);
	$docs{'services'} = q(Services: Required. Select services from the right hand list to be included in this profile. Where service dependencies have been assigned to service names, be careful to include the parent service names as defined in the dependency templates.);
	$docs{'file'} = q(File: Required. Select the default file for services in the profile. To add files go to 'Control -> Files -> Add a host/service file'.);

	return %docs;
}

sub host_profile_profile() {
	my %docs = ();
	$docs{'description'} = q(Description: Optional. Store comments or instructions here.);
	$docs{'extended_host_info_template'} = q(Extended host info template: Optional. Defines extended host information for this host profile. The template controls the icon and url link as seen in Nagios for hosts assigned this profile.);
	$docs{'file'} = q(File: Required. Select the default file for hosts assigned this profile. To add files go to 'Control -> Files -> Add a host/service file'.);

	return %docs;
}

sub host_profile_template() {
	my %docs = ();
	$docs{'host_template'} = q(Host template: This option sets the default template in 'Design -> Host Wizard'.);

	return %docs;
}

sub host_profile_parents() {
	my %docs = ();
	$docs{'parents'} = q(Parents: Optional. This option sets the default parent or parents in 'Design -> Host Wizard'.);

	return %docs;
}

sub host_profile_hostgroups() {
	my %docs = ();
	$docs{'hostgroups'} = q(Hostgroups: Optional. This option sets the default hostgroup or hostgroups in 'Design -> Host Wizard'.);

	return %docs;
}

sub host_profile_escalations() {
	my %docs = ();
	$docs{'host_escalation'} = q(Host escalation tree: Optional. Select an escalation tree appropriate for hosts assigned this profile. To avoid amplified notifications (i.e. multiple notifications for the same event), a host escalation assigned to a host using this profile should not also be assigned to a hostgroup in which the host is a member.);
	$docs{'service_escalation'} = q(Service escalation tree: Optional. Select an escalation tree appropriate services using this profile. When a service escalation is assigned to a host, all services on that host use the same escalation. To avoid amplified notifications (i.e. multiple notifications for the same event), a service escalation assigned to a service name should not also be assigned to the host or the host group.);

	return %docs;
}

sub host_profile_externals() {
	my %docs = ();
	$docs{'externals'} = q(Services:);

	return %docs;
}

sub host_profile_services() {
	my %docs = ();
	$docs{'service_profile'} = q(Service profile: Required. Select the default service profile for this host profile.);

	return %docs;
}

sub host_wizard_vitals() {
	my %docs = ();
	$docs{'alias'} = q(Alias: Required. This directive is used to define a longer name or description used to identify the host. It is provided in order to allow you to more easily identify a particular host.);
	$docs{'address'} = q(Address: Required. This directive is used to define the address of the host. Normally, this is an IP address, although it could really be anything you want (so long as it can be used to check the status of the host). You can use a FQDN to identify the host instead of an IP address, but if DNS services are not availble this could cause problems. When used properly, the $HOSTADDRESS$ macro will contain this address. Note: If you do not specify an address directive in a host definition, the name of the host will be used as its address. A word of caution about doing this, however - if DNS fails, most of your service checks will fail because the plugins will be unable to resolve the host name.);
	$docs{'host_profile'} = q(Host profile: Optional. Host Profiles can be used to aid the design and managgement of hosts.);

	return %docs;
}

sub host_wizard_attribs_1() {
	my %docs = ();
	$docs{'host_template'} = q(Host template: Required. Select the host template most suitable for this host.);
	$docs{'parents'} = q(Parents: Optional. Assign this host one or more parents. Parent child relationships may also be managed via 'Manage -> Parent -> Child'. To establish a new parent use 'Design-> Parent Child'. );
	$docs{'file'} = q(File: Required. Select the file for this host. To add files go to 'Control -> Files -> Add a host/service file'.);
	return %docs;
}

sub host_wizard_attribs_2() {
	my %docs = ();
	$docs{'hostgroups'} = q(Hostgroups: Optional. Assign this host one or more hostgroups. Hostgroups  may also be managed via 'Manage -> Hostgroups'. To create a new hostgroup use 'Design-> Hostgroups'.);
	$docs{'extinfo'} = q(Extended host info template: Optional. Defines extended host information for this host profile. The template controls the icon and url link as seen in Nagios.);
	$docs{'coords2d'} = q(2d status map coords: Optional. Defines the host's position in the Nagios 2d status map. Coordinates should be given in positive integers, as they correspond to physical pixels in the generated image. The origin for drawing (0,0) is in the upper left hand corner of the image and extends in the positive x direction (to the right) along the top of the image and in the positive y direction (down) along the left hand side of the image. For reference, the size of the icons drawn is usually about 40x40 pixels (text takes a little extra space). The coordinates you specify here are for the upper left hand corner of the host icon that is drawn. Note: Don't worry about what the maximum x and y coordinates that you can use are. The CGI will automatically calculate the maximum dimensions of the image it creates based on the largest x and y coordinates you specify.);
	$docs{'coords3d'} = q(3d status map coords: Optional. Defines the host's position in the Nagios 3d status map.  Coordinates can be positive or negative real numbers. The origin for drawing is (0.0,0.0,0.0). For reference, the size of the host cubes drawn is 0.5 units on each side (text takes a little more space). The coordinates you specify here are used as the center of the host cube.);
	$docs{'host_escalation_tree'} = q(Host escalation tree: Optional. Select an escalation tree appropriate for this host. To avoid amplified notifications (i.e. multiple notifications for the same event), a host escalation assigned to this host should not also be assigned to a hostgroup in which the host is a member.);
	$docs{'service_escalation_tree'} = q(Service escalation tree: Optional. Select an escalation tree appropriate for services on this host. When a service escalation is assigned to a host, all services on that host use the same escalation tree. To avoid amplified notifications (i.e. multiple notifications for the same event), a service escalation assigned to a service name should not also be assigned to the host or the hostgroup.);
	$docs{'service_profile'} = q(Service profile: Optional. Select a service profile for this host. This sets up a default set of services);
	return %docs;
}

sub host_wizard_select_services() {
	my %docs = ();
	$docs{'services'} = q(Service list: By default the list of services is derived from the service profile selected in the last step. Each service has an option to Include/Modify/discard.<br><ul><li>Include: Accept this service as is with its default settings.</li><li>Modify: Accept this service but prompt for changes.</li><li>Discard: Do not include this service on this host.</li><li>Add to list: Add a service not included in the profile.</li></ul>);
	$docs{'file'} = q(File: Required. Select the default file for services assigned this host. To add files go to 'Control -> Files -> Add a host/service file'.);
	return %docs;
}

sub host_wizard_service_detail() {
	my %docs = ();
	$docs{'template'} = q(Service template: Required.  Select the appropriate service template for this service.);
	$docs{'use_template_command'} = q(Use template command: If checked, the check command and command line options are derived from the service template. Be careful to ensure the service template has a check command defined before checking this option.);
	$docs{'check_command'} = q(Check command: If 'Use template command' is unchecked, select the command that Nagios will run in order to check the status of the service.);
	$docs{'command_line'} = q(Command line: If 'Use template command' is unchecked and if check command requires arguments, enter the check command with command arguments separated by a ! character. Example: check-disk!/dev/sda1);
	$docs{'dependency'} = q(Service dependency template: Optional. Select the template that defines a parent service relationship on this host. Monarch will create one service dependency relationship on this host setting this service name as the dependent service. To define a dependency for a service running on a different host use 'Manage -> Hosts -> Services -> Service Dependency' and select the parent host. Be careful to ensure that the parent service is included in the list of services for this host.);
	$docs{'extinfo'} = q(Extended info template: Optional. Defines extended service information for this service name. The template controls the icon and url link as seen in Nagios.);
	$docs{'escalation'} = q(Service escalation tree: Optional. Select an escalation tree appropriate for this service name. To avoid amplified notifications (i.e. multiple notifications for the same event), a service escalation assigned to a service name should not also be assigned to the host or one of the host's hostgroups.);
	return %docs;
}


sub escalation_select_type() {
	my %docs = ();
	$docs{'escalation type'} = q(Select escalation type: 1. Hostgroup - Nagios v. 1.x defines a hostgroup escalation; Nagios v. 2.x defines a host escalation. 2. Host: Defines a host escalation. 3. Service: Defines a service escalation.);
	return %docs;
}

sub escalation_tree() {
	my %docs = ();
	$docs{'escalation_tree'} = q(Add escalation: Add an escalation template to this escalation tree.<br>Remove escalation: Remove an escalation template from this escalation tree.<br>Modify groups: Modify the list of contact groups associated with the escalation template.<br>);
	return %docs;
}

sub escalation_contact_groups() {
	my %docs = ();
	$docs{'escalation'} = q(Escalation: An escalation template.);
	$docs{'global_groups'} = q(Global groups: These are contact groups assigned to the escalation template. (Use 'Modify -> Escalations -> Escalation Templates' to update list).);
	$docs{'contact_groups'} = q(Contact groups: These are contact groups associated with the escalation template but assigned to the escalation tree.);
	return %docs;
}

sub manage_hosts_vitals() {
	my %docs = ();
	$docs{'alias'} = q(Alias: Required. This directive is used to define a longer name or description used to identify the host. It is provided in order to allow you to more easily identify a particular host.);
	$docs{'address'} = q(Address: Required. This directive is used to define the address of the host. Normally, this is an IP address, although it could really be anything you want (so long as it can be used to check the status of the host). You can use a FQDN to identify the host instead of an IP address, but if DNS services are not availble this could cause problems. When used properly, the $HOSTADDRESS$ macro will contain this address. Note: If you do not specify an address directive in a host definition, the name of the host will be used as its address. A word of caution about doing this, however - if DNS fails, most of your service checks will fail because the plugins will be unable to resolve the host name.);
	$docs{'host_template'} = q(Host template: Required. Select the host template most suitable for this host.);
	$docs{'checks_enabled'} = q(This directive is used to determine whether or not checks of this host are enabled. Values: unchecked = disable host checks, checked = enable host checks.);
	$docs{'extinfo'} = q(Extended host info template: Optional. Defines extended host information for this host profile. The template controls the icon and url link as seen in Nagios.);
	$docs{'coords2d'} = q(2d status map coords: Optional. Defines the host's position in the Nagios 2d status map. Coordinates should be given in positive integers, as they correspond to physical pixels in the generated image. The origin for drawing (0,0) is in the upper left hand corner of the image and extends in the positive x direction (to the right) along the top of the image and in the positive y direction (down) along the left hand side of the image. For reference, the size of the icons drawn is usually about 40x40 pixels (text takes a little extra space). The coordinates you specify here are for the upper left hand corner of the host icon that is drawn. Note: Don't worry about what the maximum x and y coordinates that you can use are. The CGI will automatically calculate the maximum dimensions of the image it creates based on the largest x and y coordinates you specify.);
	$docs{'coords3d'} = q(3d status map coords: Optional. Defines the host's position in the Nagios 3d status map.  Coordinates can be positive or negative real numbers. The origin for drawing is (0.0,0.0,0.0). For reference, the size of the host cubes drawn is 0.5 units on each side (text takes a little more space). The coordinates you specify here are used as the center of the host cube.);
	$docs{'file'} = q(File: Required. Select the file for this host. To add files go to 'Control -> Files -> Add a host/service file'.);
	return %docs;
}

sub manage_hosts_profile() {
	my %docs = ();
	$docs{'host_profile'} = q(Host profile: Optional. Host Profiles can be used to aid the design and managgement of hosts.);
	$docs{'service_profile'} = q(Service profile: Optional. Select a service profile for this host. This sets up a default set of services);

	return %docs;
}

sub manage_hosts_apply_profile() {
	my %docs = ();
	$docs{'host_profile'} = q(Host profile: Optional. Host Profiles can be used to aid the design and managgement of hosts.);
	$docs{'service_profile'} = q(Service profile: Optional. Select a service profile for this host. This sets up a default set of services);

	return %docs;
}

sub manage_hosts_parents() {
	my %docs = ();
	$docs{'parents'} = q(Parents: Optional. Assign this host one or more parents. Parent child relationships may also be managed via 'Manage -> Parent -> Child'. To establish a new parent use 'Design-> Parent Child'. );
	return %docs;
}

sub manage_hosts_hostgroups() {
	my %docs = ();
	$docs{'hostgroups'} = q(Hostgroups: Optional. Assign this host one or more hostgroups. Hostgroups  may also be managed via 'Manage -> Hostgroups'. To create a new hostgroup use 'Design-> Hostgroups'.);
	return %docs;
}

sub manage_hosts_escalations() {
	my %docs = ();
	$docs{'host_escalation_tree'} = q(Host escalation tree: Optional. Select an escalation tree appropriate for this host. To avoid amplified notifications (i.e. multiple notifications for the same event), a host escalation assigned to this host should not also be assigned to a hostgroup in which the host is a member.);
	$docs{'service_escalation_tree'} = q(Service escalation tree: Optional. Select an escalation tree appropriate for services on this host. When a service escalation is assigned to a host, all services on that host use the same escalation tree. To avoid amplified notifications (i.e. multiple notifications for the same event), a service escalation assigned to a service name should not also be assigned to the host or the hostgroup.);
	return %docs;
}

sub manage_hosts_services() {
	my %docs = ();

	return %docs;
}

sub manage_hosts_service_detail() {
	my %docs = ();
	my %docs = ();
	$docs{'template'} = q(Service template: Required.  Select the appropriate service template for this service.);
	$docs{'use_template_command'} = q(Use template command: If checked, the check command and command line options are derived from the service template. Be careful to ensure the service template has a check command defined before checking this option.);
	$docs{'check_command'} = q(Check command: If 'Use template command' is unchecked, select the command that Nagios will run in order to check the status of the service.);
	$docs{'command_line'} = q(Command line: If 'Use template command' is unchecked and if check command requires arguments, enter the check command with command arguments separated by a ! character. Example: check-disk!/dev/sda1);
	$docs{'extinfo'} = q(Extended info template: Optional. Defines extended service information for this service name. The template controls the icon and url link as seen in Nagios.);
	$docs{'escalation'} = q(Service escalation tree: Optional. Select an escalation tree appropriate for this service name. To avoid amplified notifications (i.e. multiple notifications for the same event), a service escalation assigned to a service name should not also be assigned to the host or one of the host's hostgroups.);
	return %docs;

	return %docs;
}

sub manage_hosts_service_dependencies() {
	my %docs = ();
	$docs{'dependency'} = q(Service dependency template: Optional. Select the template that defines a parent service relationship on this host. Monarch will create one service dependency relationship on this host setting this service name as the dependent service. To define a dependency for a service running on a different host use 'Manage -> Hosts -> Services -> Service Dependency' and select the parent host. Be careful to ensure that the parent service is included in the list of services for this host.);

	return %docs;
}

sub setup() {
	my %docs = ();
	$docs{'login_authentication'} = q(Login authentication: Select none, active or passive. none = no authentication - all users have full control; active = users prompted with login with access checks enabled; passive = no login if user account is passed in URL but access checks are enabled (single sign-on method).);
	$docs{'session_timeout'} = q(Session timeout: With login authentication active, this is the number of seconds of inactivity before a user is prompted to login.);
	$docs{'nagios_home'} = q(Nagios nagios.cfg dir: Path to the folder containing nagios.cfg.);
	$docs{'cgi_home'} = q(Nagios cgi.cfg dir: Path to the folder containing cgi.cfg.);
	$docs{'workspace_home'} = q(Workspace dir: This folder is used to stage and test Nagios files. Your web server must have read write access.);
	$docs{'backup_home'} = q(Backup dir: This folder is used to backup your Nagios files when a commit is engauged. Your web server must have read write access.);

	return %docs;
}

sub nagios_cfg() {
	my %docs = ();
	$docs{'log_file'} = q(Log file: This is the main log file where service and host events are logged for historical purposes.  This should be the first option specified in the config file!);
	$docs{'resource_file'} = q(Resource file: This is an optional resource file that contains $USERx$ macro definitions. Multiple resource files can be specified by using multiple resource_file definitions.  The CGIs will not attempt to read the contents of resource files, so information that is considered to be sensitive (usernames, passwords, etc) can be defined as macros in this file and restrictive permissions (600) can be placed on this file.);
	$docs{'status_file'} = q(Status file: This is where the current status of all monitored services and hosts is stored.  Its contents are read and processed by the CGIs. The contentsof the status file are deleted every time Nagios restarts.);
	$docs{'nagios_user'} = q(Nagios user: This determines the effective user that Nagios should run as. You can either supply a username or a UID.);
	$docs{'nagios_group'} = q(Nagios group: This determines the effective group that Nagios should run as. You can either supply a group name or a GID.);
	$docs{'check_external_commands'} = q(Check external commands: This option allows you to specify whether or not Nagios should check for external commands (in the command file defined below).  By default Nagios will *not* check for external commands, just to be on the cautious side.  If you want to be able to use the CGI command interface you will have to enable this.  Setting this value to unchecked disables command checking (the default), other values enable it.);
	$docs{'command_check_interval'} = q(Command check interval: This is the interval at which Nagios should check for external commands. This value works of the interval_length you specify later.  If you leave that at its default value of 60 (seconds), a value of 1 here will cause Nagios to check for external commands every minute.  If you specify a number followed by an 's' (i.e. 15s), this will be interpreted to mean actual seconds rather than a multiple of the interval_length variable. Note: In addition to reading the external command file at regularly scheduled intervals, Nagios will also check for external commands after event handlers are executed. NOTE: Setting this value to -1 causes Nagios to check the external command file as often as possible.);
	$docs{'command_file'} = q(This is the file that Nagios checks for external command requests. It is also where the command CGI will write commands that are submitted by users, so it must be writeable by the user that the web server is running as (usually 'nobody').  Permissions should be set at the directory level instead of on the file, as the file is deleted every time its contents are processed.);
	$docs{'comment_file'} = q(This is the file that Nagios will use for storing host and service comments.);
	$docs{'downtime_file'} = q(Downtime file: This is the file that Nagios will use for storing host and service downtime data.);
	$docs{'lock_file'} = q(Lock file: This is the lockfile that Nagios will use to store its PID number in when it is running in daemon mode.);
	$docs{'temp_file'} = q(Temp file: This is a temporary file that is used as scratch space when Nagios updates the status log, cleans the comment file, etc.  This file is created, used, and deleted throughout the time that Nagios is running.);
	$docs{'log_rotation_method'} = q(Log rotation method: This is the log rotation method that Nagios should use to rotate the main log file. Values are as follows. None: checked = None - don't rotate the log. Hourly: checked = Hourly rotation (top of the hour). Daily: checked = Daily rotation (midnight every day). Weekly: checked = Weekly rotation (midnight on Saturday evening). Monthly: checked = Monthly rotation (midnight last day of month));
	$docs{'log_archive_path'} = q(Log archive path: This is the directory where archived (rotated) log files should be placed (assuming you've chosen to do log rotation).);
	$docs{'use_syslog'} = q(Use syslog: If you want messages logged to the syslog facility, as well as the NetAlarm log file set this option to checked.  If not, set it to unchecked.);
	$docs{'log_notifications'} = q(Log notifications: If you don't want notifications to be logged, set this value to unchecked. If notifications should be logged, set the value to checked.);
	$docs{'log_service_retries'} = q(Log service retries: If you don't want service check retries to be logged, set this value to unchecked.  If retries should be logged, set the value to checked.);
	$docs{'log_host_retries'} = q(Log host retries: If you don't want host check retries to be logged, set this value to unchecked.  If retries should be logged, set the value to checked.);
	$docs{'log_event_handlers'} = q(Log event handlers: If you don't want host and service event handlers to be logged, set this value to unchecked.  If event handlers should be logged, set the value to checked.);
	$docs{'log_initial_states'} = q(Log initial states: If you want Nagios to log all initial host and service states to the main log file (the first time the service or host is checked) you can enable this option by setting this value to checked.  If you are not using an external application that does long term state statistics reporting, you do not need to enable this option.  In this case, set the value to unchecked.);
	$docs{'log_external_commands'} = q(Log external commands: If you don't want Nagios to log external commands, set this value to unchecked.  If external commands should be logged, set this value to checked. Note: This option does not include logging of passive service checks - see the option below for controlling whether or not passive checks are logged.);
	$docs{'log_passive_service_checks'} = q(Log passive service checks: If you don't want Nagios to log passive service checks, set this value to unchecked.  If passive service checks should be logged, set this value to checked.);
	$docs{'global_host_event_handler'} = q(Global host event handler: This option allows you to specify a host event handler command that is to be run for every host state change. The global event handler is executed immediately prior to the event handler that you have optionally specified in each host definition. The command argument is the short name of a command definition that you define in your host configuration file. Read the Nagios HTML docs for more information.);
	$docs{'global_service_event_handler'} = q(Global service event handler: This option allows you to specify a service event handler command that is to be run for every host state change. The global event handler is executed immediately prior to the event handler that you have optionally specified in each service definition. The command argument is the short name of a command definition that you define in your host configuration file. Read the Nagios HTML docs for more information.);
	$docs{'inter_check_delay_method'} = q(Inter check delay method: Inter check delay method: This is the method that Nagios should use when initially 'spreading out' service checks when it starts monitoring. The default is to use smart delay calculation, which will try to space all service checks out evenly to minimize CPU load. Using the dumb setting will cause all checks to be scheduled at the same time (with no delay between them)!  This is not a good thing for production, but is useful when testing the parallelization functionality. None: selected = None - don't use any delay between checks. Dumb: selected = Use a 'dumb' delay of 1 second between checks. Smart: selected = Use 'smart' inter-check delay calculation. Delay: selected = Enter an inter-check delay of x.xx seconds.);
	$docs{'service_interleave_factor'} = q(Service interleave factor: This variable determines how service checks are interleaved. Interleaving the service checks allows for a more even distribution of service checks and reduced load on remote hosts.  Setting this value to 1 is equivalent to how versions of Nagios previous to 0.0.5 did service checks.  Set this value to s (smart) for automatic calculation of the interleave factor unless you have a specific reason to change it. Smart: selected = Use 'smart' interleave factor calculation. Value: selected = Enter an interleave factor of x, where x is a number greater than or equal to 1.);
	$docs{'max_concurrent_checks'} = q(Max concurrent checks: This option allows you to specify the maximum number of service checks that can be run in parallel at any given time. Specifying a value of 1 for this variable essentially prevents any service checks from being parallelized.  A value of 0 will not restrict the number of concurrent checks that are being executed.);
	$docs{'service_reaper_frequency'} = q(Service reaper frequency: This is the frequency (in seconds!) that Nagios will process the results of services that have been checked.);
	$docs{'sleep_time'} = q(Sleep time: This is the number of seconds to sleep between checking for system events and service checks that need to be run.  I would recommend *not* changing this from its default value of 1 second.);
	$docs{'timeout_values'} = q(Timeout values: These options control how much time Nagios will allow various types of commands to execute before killing them off.  Options are available for controlling maximum time allotted for service checks, host checks, event handlers, notifications, the ocsp command, and performance data commands.  All values are in seconds.);
	$docs{'retain_state_information'} = q(Retain state information: This setting determines whether or not Nagios will save state information for services and hosts before it shuts down. Upon startup Nagios will reload all saved service and host state information before starting to monitor.  This is useful for maintaining long-term data on state statistics, etc, but will slow Nagios down a bit when it (re)starts.  Since its only a one-time penalty, I think its well worth the additional startup delay. Checked = enabled.);
	$docs{'state_retention_file'} = q(State retention file: This is the file that Nagios should use to store host and service state information before it shuts down. The state information in this file is also read immediately prior to starting to monitor the network when Nagios is restarted. This file is used only if the preserve_state_information variable is checked.);
	$docs{'retention_update_interval'} = q(Retention update interval: This setting determines how often (in minutes) that Nagios will automatically save retention data during normal operation. If you set this value to 0, Nagios will not save retention data at regular interval, but it will still save retention data before shutting down or restarting.  If you have disabled state retention, this option has no effect.);
	$docs{'use_retained_program_state'} = q(Use retained program state: This setting determines whether or not Nagios will set program status variables based on the values saved in the retention file.  If you want to use retained program status information, set this value to 1.  If not, set this value to 0.);
	$docs{'interval_length'} = q(Interval length: This is the seconds per unit interval as used in the host/contact/service configuration files.  Setting this to 60 means that each interval is one minute long (60 seconds).  Other settings have not been tested much, so your mileage is likely to vary...);
	$docs{'use_agressive_host_checking'} = q(Use agressive host checking: If you don't want to turn on agressive host checking features, set this value to unchecked (the default).  Otherwise set this value to checked to enable the agressive check option.  Read the docs for more info on what agressive host check is or check out the source code in base/checks.c);
	$docs{'execute_service_checks'} = q(Execute service checks: This determines whether or not Nagios will actively execute service checks when it initially starts.  If this option is disabled, checks are not actively made, but Nagios can still receive and process passive check results that come in.  Unless you're implementing redundant hosts or have a special need for disabling the execution of service checks, leave this enabled! Values: checked = enable checks, unchecked = disable checks>);
	$docs{'accept_passive_service_checks'} = q(Accept passive service checks: This determines whether or not Nagios will accept passive service checks results when it initially (re)starts. Values: checked = accept passive checks, unchecked = reject passive checks.);
	$docs{'enable_notifications'} = q(Enable notifications: This determines whether or not Nagios will sent out any host or service notifications when it is initially (re)started. Values: checked = enable notifications, unchecked = disable notifications);
	$docs{'enable_event_handlers'} = q(Enable event handlers: This determines whether or not Nagios will run any host or service event handlers when it is initially (re)started.  Unless you're implementing redundant hosts, leave this option enabled. Values: checked = enable event handlers, unchecked = disable event handlers);
	$docs{'process_performance_data'} = q(Process performance data: This determines whether or not Nagios will process performance data returned from service and host checks.  If this option is enabled, host performance data will be processed using the host perfdata command (defined below) and service performance data will be processed using the service perfdata command (also defined below).  Read the HTML docs for more information on performance data. Values: checked = process performance data, unchecked = do not process performance data);
	$docs{'use_agressive_host_checking'} = q(Use agressive host checking: This command is run after every host check is performed.  These commands are executed only if the enable_performance_data option (above) is set to 1.  The command argument is the short name of a command definition that you define in your host configuration file.  Read the Nagios HTML docs for more information on performance data.);
	$docs{'use_agressive_service_checking'} = q(Use agressive service checking: This command is run after every service check is performed.  These commands are executed only if the enable_performance_data option (above) is checked.  The command argument is the short name of a command definition that you define in your host configuration file.  Read the Nagios HTML docs for more information on performance data.);
	$docs{'obsess_over_services'} = q(Obsess over services: This determines whether or not Nagios will obsess over service checks and run the ocsp_command defined below.  Unless you're planning on implementing distributed monitoring, do not enable this option.  Read the HTML docs for more information on implementing distributed monitoring. Values: checked = obsess over services, unchecked = do not obsess (default));
	$docs{'ocsp_command'} = q(Ocsp command: This is the command that is run for every service check that is processed by Nagios.  This command is executed only if the obsess_over_service option (above) is set to 1.  The command argument is the short name of a command definition that you define in your host configuration file. Read the HTML docs for more information on implementing distributed monitoring.);
	$docs{'check_for_orphaned_services'} = q(Check for orphaned services: This determines whether or not Nagios will periodically check for orphaned services.  Since service checks are not rescheduled until the results of their previous execution instance are processed, there exists a possibility that some checks may never get rescheduled.  This seems to be a rare problem and should not happen under normal circumstances. If you have problems with service checks never getting rescheduled, you might want to try enabling this option. Values: checked = enable checks, unchecked = disable checks);
	$docs{'check_service_freshness'} = q(Check service freshness: This option determines whether or not Nagios will periodically check the 'freshness' of service results.  Enabling this option is useful for ensuring passive checks are received in a timely manner. Values: checked = enabled freshness checking, unchecked = disable freshness checking.);
	$docs{'freshness_check_interval'} = q(Freshness check interval: This setting determines how often (in seconds) Nagios will check the 'freshness' of service check results.  If you have disabled service freshness checking, this option has no effect.);
	$docs{'aggregate_status_updates'} = q(Aggregate status updates: This option determines whether or not Nagios will aggregate updates of host, service, and program status data. Normally, status data is updated immediately when a change occurs.  This can result in high CPU loads if you are monitoring a lot of services.  If you want Nagios to only refresh status data every few seconds, disable this option. Values: checked = enable aggregate updates, unchecked = disable aggregate updates.);
	$docs{'status_update_interval'} = q(Status update interval: Combined with the aggregate_status_updates option, this option determines the frequency (in seconds!) that Nagios will periodically dump program, host, and service status data.  If you are not using aggregated status data updates, this option has no effect.);
	$docs{'enable_flap_detection'} = q(Enable flap detection: This option determines whether or not Nagios will try and detect hosts and services that are 'flapping'. Flapping occurs when a host or service changes between states too frequently.  When Nagios detects that a host or service is flapping, it will temporarily supress notifications for that host/service until it stops flapping.  Flap detection is very experimental, so read the HTML documentation before enabling this feature! Values: checked = enable flap detection unchecked = disable flap detection (default));
	$docs{'host_flap_detection_threshholds'} = q(Host flap detection threshholds: Read the Nagios HTML documentation on flap detection for an explanation of what this option does.  This option has no effect if flap detection is disabled.);
	$docs{'service_flap_detection_threshholds'} = q(Service flap detection threshholds: Read the Nagios HTML documentation on flap detection for an explanation of what this option does.  This option has no effect if flap detection is disabled.);
	$docs{'date_format'} = q(Date format: This option determines how short dates are displayed. US: MM-DD-YYYY HH:MM:SS, Euro DD-MM-YYYY HH:MM:SS, ISO8601 YYYY-MM-DD HH:MM:SS,Strict-ISO8601 YYYY-MM-DDTHH:MM:SS);
	$docs{'illegal_object_name_chars'} = q(Illegal object name chars: This options allows you to specify illegal characters that cannot be used in host names, service descriptions, or names of other object types.);
	$docs{'illegal_macro_output_chars'} = q(Illegal macro output chars: This options allows you to specify illegal characters that are stripped from macros before being used in notifications, event handlers, etc.  This DOES NOT affect macros used in service or host check commands. The following macros are stripped of the characters you specify: $OUTPUT$, $PERFDATA$.);
	$docs{'admin_email'} = q(Admin email: The email address of the administrator of *this* machine (the one doing the monitoring).  Nagios never uses this value itself, but you can access this value by using the $ADMINEMAIL$ macro in your notification commands.);
	$docs{'admin_pager'} = q(Admin pager: The pager number/address for the administrator of *this* machine. Nagios never uses this value itself, but you can access this value by using the $ADMINPAGER$ macro in your notification commands.);
	return %docs;
}

1;

  

