# MonArch - Groundwork Monitor Architect
# MonarchStorProc.pm
#
###############################################################################
# Release 0.94c
# 16-May-2005
###############################################################################
#


use strict;
use XML::LibXML;
use DBI;
use Time::Local;
use MonarchConf;

package StorProc;

my ($dbh, $err, $rows);
my $empty_data = qq(<?xml version="1.0" ?>
<data>
</data>);

sub dbconnect() {
	my ($database,$user,$passwd) = Conf->get_dbauth();
	my $dsn = "DBI:mysql:$database:localhost";
	$dbh = DBI->connect($dsn, $user, $passwd, {'RaiseError' => 1});
	my $sqlstmt = "select value from setup where name = 'login_authentication'";
	my ($login_type) = $dbh->selectrow_array($sqlstmt);
	if ($login_type eq 'none') {
		return 1;
	} elsif ($login_type eq 'active')  {
		return 2;
	} else {
		return 0;
	}
}

sub check_status() {
	my $sqlstmt = "select value from setup where name = 'load_status'";
	my $status = $dbh->selectrow_array($sqlstmt);
	if ($status == 0) {
		return 0;	
	} else {
		return 1;	
	}
}

sub check_user(@) {
	my $user_acct = $_[1];
	my $password = $_[2];
	my $error = undef;
	my $now = time;
	my $sqlstmt = "select user_id, session from users where user_acct = '$user_acct'";
	my ($userid,$session) = $dbh->selectrow_array($sqlstmt);
	if ($userid) {
		$sqlstmt = "select value from setup where name = 'login_authentication'";
		my ($login_type) = $dbh->selectrow_array($sqlstmt);
		if ($login_type eq 'active') {
			if ($password) {
				$sqlstmt = "select password from users where user_id = '$userid'";
				my ($ck_password) = $dbh->selectrow_array($sqlstmt);
				if ($ck_password eq $password) {
				my $sth = $dbh->prepare ("update users set session = $now where user_id = '$userid'");
					$sth->execute;
					$sth->finish;
				} else {
					$error = "Invalid password";
				}
			} else {
				$sqlstmt = "select value from setup where name = 'session_timeout'";
				my ($timeout) = $dbh->selectrow_array($sqlstmt);
				if (($now - $session) > $timeout) {
					$error = "Session timed out. Please login.";
				} else {
					my $sth = $dbh->prepare ("update users set session = $now where user_id = '$userid'");
					$sth->execute;
					$sth->finish;
				}
			}
		} else {
			my $sth = $dbh->prepare ("update users set session = $now where user_id = '$userid'");
			$sth->execute;
			$sth->finish;
		}
	} else {
		$error = "Invalid user id";
	}
	if ($error) {
		return $error;
	} else {
		return $userid;
	}
}

sub auth_matrix(@) {
	my $userid = $_[1];
	my %auth_add = {};
	my %auth_modify = {};
	my %auth_delete = {};
	my $sqlstmt = "select usergroup_id from user_group where user_id = '$userid'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my $groupid = undef;
	eval {$sth->bind_columns(undef, \$groupid)};
	while($sth->fetch()) {
		my ($object,$type,$access_values) = undef;
		$sqlstmt = "select object, type, access_values from access_list where usergroup_id = '$groupid'";
		my $sth2 = $dbh->prepare($sqlstmt);
		$sth2->execute;
		eval {$sth2->bind_columns(undef,\$object,\$type,\$access_values)};
		while($sth2->fetch()) {
			if ($type eq 'design_manage') {
				if ($access_values =~ /add/) { $auth_add{$object} = 1; $auth_add{'design'} = 1 }
				if ($access_values =~ /modify/) { $auth_modify{$object} = 1; $auth_add{'modify'} = 1 }
				if ($access_values =~ /delete/) { $auth_delete{$object} = 1 }
			} else {
				if ($type eq 'control') {
					$auth_add{'control'} = 1;
				} else {
					$auth_add{'discover'} = 1;
				}
				$auth_add{$object} = 1;
			} 
		}
		$sth2->finish;
	}
	$sth->finish;
	return (\%auth_add, \%auth_modify, \%auth_delete);
}

sub logout(@) {
	my $user_acct = $_[1];
	my $sth = $dbh->prepare ("update users set session = '0' where user_acct = '$user_acct'");
	$sth->execute;
	$sth->finish;
}

sub dbdisconnect() {
	if ($dbh) {
		$dbh and $dbh->disconnect();
	}
}

sub get_hosts() {
	my %hosts = ();
	my $sqlstmt = "select name, host_id from hosts";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) {
		$hosts{$values[0]} = $values[1];
	}
	$sth->finish;
	return %hosts;
}


sub parse_xml($) {
	my $data = shift;
	my %properties;
	if ($data) {
		my $parser = XML::LibXML->new();
		my $doc = $parser->parse_string($data);
		my @nodes = $doc->findnodes( "//prop" );
		foreach my $node (@nodes) {
			if ($node->hasAttributes()) {
				my $property = $node->getAttribute('name');
				my $value = $node->textContent;
				$value =~ s/\s+$|\n//g;
				if ($property =~ /command$/) {
					my $command_line = '';
					if ($value) {
						my @command = split(/!/, $value);
						$properties{$property} = $command[0];
						if ($command[1]) {
							foreach my $c (@command) {
								$command_line .= "$c!";
							}
						}
					}
					$command_line =~ s/!$//;
					$properties{'command_line'} = $command_line;
				} elsif ($property =~ /last_notification$/) {
					my $value = $node->textContent;
					$value =~ s/\s+$|\n//g;
					if ($value == 0) {
						$properties{$property} = '-zero-';
					} else {					
						$properties{$property} = $value;
					}
				} else {
					$properties{$property} = $value;
				}
			}
		}
		return %properties;
	} else {
		$properties{'error'} = "Empty String (parse_xml)";
	}
}

sub fetch_last(@) {
	my $table = $_[1];
	my $sqlstmt = "select max(id) as lastrow from $table";
	my $id = $dbh->selectrow_array($sqlstmt);
	return $id;
}

sub fetch_one(@) {
	my $table = $_[1];
	my $name = $_[2];
	my $value = $_[3];
	my $values = undef;
	my %properties = {};
	$value = $dbh->quote($value);
	my $sqlstmt = "select * from $table where $name = $value";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	$values = $sth->fetchrow_hashref();
	foreach my $key (keys %{$values}) {
		if ($key eq 'data' && $table !~ /external/) {
			my %data = parse_xml($values->{$key});
			foreach my $k (keys %data) {
				$properties{$k} = $data{$k};
			}
		} else {
			$properties{$key} = $values->{$key};
		}
	}
	$sth->finish;
	return %properties;
}

sub fetch_one_where(@) {
	my $table = $_[1];
	my $where = $_[2];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "select * from $table where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my $values = $sth->fetchrow_hashref();
	my %properties = {};
	foreach my $key (keys %{$values}) {
		if ($key eq 'data' && $table !~ /external/) {
			my %data = parse_xml($values->{$key});
			foreach my $k (keys %data) {
				$properties{$k} = $data{$k};
			}
		} else {
			$properties{$key} = $values->{$key};
		}
	}
	$sth->finish;
	return %properties;
}


sub fetch_list(@) {
	my $table = $_[1];
	my $list = $_[2];
	my $orderby = $_[3];
	my @values = ();
	my @elements = ();
	if ($orderby) { $orderby = " order by $orderby" }
	my $sqlstmt = "select $list from $table$orderby";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_list_where(@) {
	my $table = $_[1];
	my $list = $_[2];
	my $where = $_[3];
	my $orderby = $_[4];
	if ($orderby) { $orderby = " order by $orderby" }
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "select $list from $table where $where_clause$orderby";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @elements = ();
	my @values = ();
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_list_like(@) {
	my $table = $_[1];
	my $list = $_[2];
	my $where = $_[3];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			my $like = '%'.$where{$name}.'%';
			$where_clause .= " $name like '$like' and"; 
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "select $list from $table where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @elements = ();
	my @values = ();
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_list_start(@) {
	my $table = $_[1];
	my $list = $_[2];
	my $name = $_[3];
	my $value = $_[4];
	$value .= "%";
	my $sqlstmt = "select $list from $table where $name like '$value'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @elements = ();
	while(my @values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_distinct_list(@) {
	my $table = $_[1];
	my $list = $_[2];
	my @values = ();
	my @elements = ();
	my $sqlstmt = "select distinct $list from $table";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}


sub fetch_file_hash(@) {
	my %files = {};
	my $sqlstmt = "select name, value from setup where type = 'file'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) {
		$files{$values[0]} = $values[1];
	}
	$sth->finish;
	return %files;
}

sub fetch_unique(@) {
	my $table = $_[1];
	my $list = $_[2];
	my $object = $_[3];
	my $value = $_[4];
	my @values = ();
	my @elements = ();
	my $sqlstmt = "select $list from $table where $object = '$value'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_list_hash_array(@) {
	my $table = $_[1];
	my $where = $_[2];
	my %where = %{$where};
	my %elements;
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH|NONE/) {
			$where_clause .= " $name = '$where{$name}' and"; 
		}
	}
	$where_clause =~ s/ and$//;
	if ($where_clause) { $where_clause = " where $where_clause" }
	my $sqlstmt = "select * from $table$where_clause";
	my $sth = $dbh->prepare($sqlstmt);
	if ($sth->execute) {
		while(my @values = $sth->fetchrow_array()) {
			$elements{$values[0]} = [ @values ];
		}
		return %elements;
	} else {
		$elements{'error'} = "Error: $@";
		return %elements;
	}
	$sth->finish;
}


sub fetch_all($) {
	my $table = $_[1];
	my %elements = {};
	my @values = undef;
	my $sqlstmt = "select * from $table";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		while(@values = $sth->fetchrow_array()) {
			$elements{$values[0]} = [ @values ];
		}
		return %elements;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}

sub insert_obj(@) {
	my $table = $_[1];
	my $values =  $_[2];
	my @values = @{$values};
	my $valstr = undef;
	foreach my $val (@values) {
		$val = $dbh->quote($val);
		$valstr .= "$val,";
	}
	chop $valstr;
	my $sqlstmt = "insert into $table values($valstr)";
	my $sth = undef;
	eval{$sth = $dbh->prepare ($sqlstmt);
	$sth->execute()};
	if ($@) {
		return "Error: $sqlstmt $@";
	} else {
		return 1;
	}
	$sth->finish;
}

sub insert_obj_id(@) {
	my $table = $_[1];
	my $values =  $_[2];
	my $id =  $_[3];
	my @values = @{$values};
	my $valstr = undef;
	foreach my $val (@values) {
		$val = $dbh->quote($val);
		$valstr .= "$val,";
	}
	chop $valstr;
	my $sqlstmt = "insert into $table values($valstr)";
	my $sth = undef;
	eval{$sth = $dbh->prepare ($sqlstmt);
	$sth->execute()};
	if ($@) {
		return "Error: $sqlstmt $@";
	} else {
		my $sqlstmt = "select max($id) as lastrow from $table";
		my $id = $dbh->selectrow_array($sqlstmt);
		return $id;
	}
	$sth->finish;
}

sub update_obj(@) {
	my $table = $_[1];
	my $name = $_[2];
	my $obj = $_[3];
	my $values =  $_[4];
	my %values = %{$values};
	my $valstr = undef;
	$obj = $dbh->quote($obj);
	foreach my $key (keys %values) {
		unless ($key =~ /HASH/) {
			my $val = $dbh->quote($values{$key});
			$valstr .= "$key = $val,";
		}
	}
	chop $valstr;
	my $sqlstmt = "update $table set $valstr where $name = $obj";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		return 1;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}

sub update_obj_where(@) {
	my $table = $_[1];
	my $values =  $_[2];
	my $where = $_[3];
	my %values = %{$values};
	my %where = %{$where};
	my $valstr = undef;
	foreach my $key (keys %values) {
		unless ($key =~ /HASH/) {
			my $val = $dbh->quote($values{$key});
			$valstr .= "$key = $val,";
		}
	}
	chop $valstr;
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			my $val = $dbh->quote($where{$name});
			$where_clause .= " $name = $val and"; 
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "update $table set $valstr where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		return 1;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}

sub delete_all(@) {
	my $table = $_[1];
	my $name = $_[2];
	my $obj = $_[3];
	$obj = $dbh->quote($obj);
	my $sqlstmt = "delete from $table where $name = $obj";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		return 1;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}

sub delete_one_where(@) {
	my $table = $_[1];
	my $where = $_[2];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				my $val = $dbh->quote($where{$name});
				$where_clause .= " $name = $val and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "delete from $table where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		return 1;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}


sub fetch_service(@) {
	my $sid = $_[1];
	my @errors = ();
	my $values = undef;
	my %properties = {};
	my $sqlstmt = "select * from services where service_id = '$sid'";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		$values = $sth->fetchrow_hashref();
		foreach my $key (keys %{$values}) {
			$properties{$key} = $values->{$key};
		}
	} else {
		push @errors, "$sqlstmt $@";
	}
	$sth->finish;

	$sqlstmt = "select name from service_names where servicename_id = '$properties{'servicename_id'}'";
	$properties{'service_name'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select name from service_templates where servicetemplate_id = '$properties{'servicetemplate_id'}'";
	$properties{'template'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select name from extended_service_info_templates where serviceextinfo_id = '$properties{'serviceextinfo_id'}'";
	$properties{'ext_info'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select name from escalation_trees where tree_id = '$properties{'escalation_id'}'";
	$properties{'escalation'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select name from commands where command_id = '$properties{'check_command'}'";
	$properties{'check_command'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select name from service_dependency_templates where id in ".
		"(select template from service_dependency where service_id = '$sid')";
	$properties{'dependency'} = $dbh->selectrow_array($sqlstmt);

	if (@errors) {
		$properties{'errors'} = @errors;
	}
	return %properties;
}



sub fetch_host(@) {
	my $name = $_[1];
	my @errors = ();
	my $values = undef;
	my %properties = {};
	my $sqlstmt = "select * from hosts where name = '$name'";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		$values = $sth->fetchrow_hashref();
		foreach my $key (keys %{$values}) {
			$properties{$key} = $values->{$key};
		}
	} else {
		push @errors, "$sqlstmt $@";
	}
	$sth->finish;

	$sqlstmt = "select name from host_templates where hosttemplate_id = '$properties{'hosttemplate_id'}'";
	$properties{'template'} = $dbh->selectrow_array($sqlstmt);

	# Host parent
	$sqlstmt = "select hosts.name from hosts left join host_parent on hosts.host_id = host_parent.parent_id ".
		"where host_parent.host_id = '$properties{'host_id'}'";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0] }
	$sth->finish;	
	$properties{'parents'} = [ @names ];

	# Hostgroups
	$sqlstmt = "select hostgroups.name from hostgroups left join ".
		"hostgroup_host on hostgroups.hostgroup_id = hostgroup_host.hostgroup_id where host_id = '$properties{'host_id'}'";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0] }
	$sth->finish;	
	$properties{'hostgroups'} = [ @names ];

	$sqlstmt = "select name from extended_host_info_templates where hostextinfo_id = '$properties{'hostextinfo_id'}'";
	$properties{'ext_info'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select name from extended_host_info_templates where hostextinfo_id = '$properties{'hostextinfo_id'}'";
	$properties{'coords2d'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select data from extended_info_coords where host_id = '$properties{'host_id'}'";
	my $data = $dbh->selectrow_array($sqlstmt);
	my %data = parse_xml($data);
	$properties{'coords2d'} = $data{'2d_coords'};
	$properties{'coords3d'} = $data{'3d_coords'};

	$sqlstmt = "select name from escalation_trees where tree_id = '$properties{'host_escalation_id'}'";
	$properties{'host_escalation'} = $dbh->selectrow_array($sqlstmt);

	$sqlstmt = "select name from escalation_trees where tree_id = '$properties{'service_escalation_id'}'";
	$properties{'service_escalation'} = $dbh->selectrow_array($sqlstmt);
	
	my %overrides = fetch_one('','host_overrides','host_id',$properties{'host_id'});
	if ($overrides{'status'}) {
		foreach my $name (keys %overrides) {
			$properties{$name} = $overrides{$name};
		}
	}


	# File
	$sqlstmt = "select files.name from files left join file_host on files.file_id = file_host.file_id where file_host.host_id = '$properties{'host_id'}'";
	$properties{'file'} = $dbh->selectrow_array($sqlstmt);
	
	if (@errors) {
		$properties{'errors'} = @errors;
	}
	return %properties;

}

sub datetime() {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	$year += 1900;
	$mon++;
	if ($mon =~ /^\d{1}$/) { $mon = "0".$mon }
	if ($mday =~ /^\d{1}$/) { $mday = "0".$mday }
	if ($hour =~ /^\d{1}$/) { $hour = "0".$hour }
	if ($min =~ /^\d{1}$/) { $min = "0".$min }
	if ($sec =~ /^\d{1}$/) { $sec = "0".$sec }
	return "$year-$mon-$mday $hour:$min:$sec";
}


sub pre_flight_check(@) {
	my $nagios_bin = $_[1];
	my $monarch_home = $_[2];
	my @results;
	my $results = qx($nagios_bin/nagios -v $monarch_home/workspace/nagios.cfg);
	if ($results =~ /Things look okay/i) {
		push @results, "Success:";
	}
	my @res = split(/\n/, $results);
	push (@results,@res);
	return @results;
}

sub commit(@) {
	my $monarch_home = $_[1];
	my @results = ();
	my $res = qx($monarch_home/bin/nagios_reload);
	if ($res =~ /NAGIOS ok|Starting nagios ..done/i) {
		push @results, $res;
		push @results, "Good. Changes accepted.";
	} else {
		push @results, $res;
		push @results, "$res";
	}
	return @results;	
}

sub upload(@) {
	my $upload = $_[1];
	my $filename = $_[2];
	my $buff = 0;
	my $bytes_read = undef;
	my $size = (-s $filename);
	my $file = undef;
	if ($filename =~ /([^\/\\]+)$/) {
		$file = $1;
	}
	$file =~ s/\s+/_/g;
	$file = lc($file);
	my $write_file = "$upload/$file";  
	if (!open(WFD,"> $write_file")) {
	    return qq(Error: Could not open file for writing. $write_file $!);
	} else {
		binmode WFD;	
		while ($bytes_read = read($filename, $buff, 2096)) {
			$size += $bytes_read;
			binmode WFD;
			print WFD $buff;
		}
		close WFD;
		if ((stat $write_file)[7] <= 0) {
			unlink($write_file);
			return qq(Error: Could not upload file: $filename to $write_file $!);
		} else {
			return $write_file;
		}
	}
}

sub process_schema() {
	my $userid = $_[1];
	my $filename = $_[2];
	my $schema = $_[3];
	my @errors = ();
	my %schema = fetch_one('','import_schemas','name',$schema);
	$schema{'host'}--;
	$schema{'alias'}--;
	$schema{'address'}--;
	$schema{'os'}--;
	$schema{'service'}--;
	$schema{'info'}--;
	my ($service, $host, $services, $hosts) = undef;
	my ($os, $address, $hostname, $got_host) = undef;
	my @services = ();
	open (FILE, "< $filename") || push @errors, "error: cannot open $filename $!";
	my @lines = ();
	while (my $line = <FILE>){
		$line =~ s/\"//g;
		push @lines, $line;
	}
	close (FILE);
	push @lines, 'EOF';
	foreach my $line (@lines) {
		my @values = split(/$schema{'field_separator'}/,$line);
		if ($values[$schema{'host'}] eq $hostname ) { 
			my $svc = $values[$schema{'service'}];
			$svc =~ s/^\s+|\s+$//g;
			my $info = $values[$schema{'info'}];
			$info =~ s/^\s+|\s+$//g;
			my %service = (
				service => $svc,
				info => $info
				);
			push @services, { %service };
			$got_host = 1;
		} else {
			if ($got_host || $line eq 'EOF') {
				if (!$os) { $os = "nomatch" }
				my @host_vals = split(/\./, $hostname);
				my $name = shift @host_vals;
				my $alias = $hostname;
				if (!$name) { $name = $address }
				my @host_values = ($name,$userid,'import','incomplete',$alias,$address,$os,'','','');
				my $result = insert_obj('','stage_hosts',\@host_values);
				if ($result =~ /Error/) {
					push @errors, $result;

				} else {				
					for my $i ( 0 .. $#services ) {
						my @service_values = ($services[$i]{'service'},$userid,'import','incomplete',$name,'');
						$result = insert_obj('','stage_host_services',\@service_values);
						if ($result =~ /Error/) { push @errors, $result }
					}
				}
				@services = ();
				$os = undef;
				$address = undef;
				$host = undef;
			}
			$hostname = $values[$schema{'host'}]; 
			$hostname =~ s/^\s+|\s+$//g;
			$address = $values[$schema{'address'}];
			$address =~ s/^\s+|\s+$//g;
			$os = $values[$schema{'os'}];
			$os =~ s/^\s+|\s+$//g;
			if ($values[$schema{'service'}]) {
				my $svc = $values[$schema{'service'}];
				$svc =~ s/^\s+|\s+$//g;
				my $info = $values[$schema{'info'}];
				$info =~ s/^\s+|\s+$//g;
				my %service = (
					service => $svc,
					info => $info
					);
				push @services, { %service };
			}
		}
	}
	return @errors;
}

sub process_nmap(@) {
	my $userid = $_[1];
	my $upload = $_[2];
	my $filename = $_[3];
	$filename = "$upload/$filename";
	my @errors = ();
	my $res = undef;
	my ($tree, $os, $address, $hostname) = undef;
	my ($line, $service, $host, $services, $hosts) = undef;
	my @services = ();
	my $hosts = qq(<?xml version="1.0" ?>\n<templates>\n);
	my $parser = XML::LibXML->new();
	eval {$tree = $parser->parse_file("$filename")};
	push @errors, "Invalid file type: $filename in not an XML Nmap scan." if $@;
	unless (@errors) {
		my $root = $tree->getDocumentElement;
		my @nodes = $root->findnodes( "//host" );
		foreach my $node (@nodes) {
			my @siblings = $node->getChildnodes();
			foreach my $sibling (@siblings) {
				if ($sibling->nodeName() =~ /hostnames$/) {
					if ($sibling->hasChildNodes()) {
						my $child = $sibling->getFirstChild();
						$hostname = $child->getAttribute('name');
					}
				} elsif ($sibling->nodeName() =~ /address$/) {
					$address = $sibling->getAttribute('addr');
				} elsif ($sibling->nodeName() =~ /os$/) {
					my @children = $sibling->getChildnodes();
					foreach my $child (@children) {
						if ($child->nodeName() =~ /osmatch$/) {$os = $child->getAttribute('name');}
					}
				} elsif ($sibling->nodeName() =~ /ports$/) {
					my @children = $sibling->getChildnodes();
					foreach my $child (@children) {
						my ($port, $svc) = undef;
						if ($child->nodeName() =~ /port$/) {
							$port = $child->getAttribute('portid');
							if ($child->hasChildNodes()) {
								my @g_children = $child->getChildnodes();
								if ($g_children[1]) {$svc = $g_children[1]->getAttribute('name');}
							}
							if ($svc) {
								my %service = (
									service => $svc,
									info => $port
									);
								push @services, { %service };
							}
						}
					}			
				}
			}
			if (!$os) { $os = "nomatch" }
			my @host_vals = split(/\./, $hostname);
			my $name = shift @host_vals;
			my $alias = $hostname;
			if (!$name) { $name = $address }
			my @host_values = ($name,$userid,'import','incomplete',$alias,$address,$os,'','','');
			my $result = insert_obj('','stage_hosts',\@host_values);
			if ($result =~ /Error/) {
				push @errors, $result;
			} else {				
				for my $i ( 0 .. $#services ) {
					my @service_values = ($services[$i]{'service'},$userid,$name,'import','incomplete','');
					$result = insert_obj('','stage_host_services',\@service_values);
					if ($result =~ /Error/) { push @errors, $result }
				}
			}
			@services = ();
			$os = undef;
			$address = undef;
			$host = undef;
		}
	}
	return @errors;
}

sub get_service_name(@) {
	my $sid = $_[1];
	my $sqlstmt = "select service_names.name from service_names left join services on ".
		"service_names.servicename_id = services.servicename_id where services.service_id = '$sid'";
	my $name = $dbh->selectrow_array($sqlstmt);
	return $name;
}

sub get_names_in(@) {
	my $id1 = $_[1];
	my $table1 = $_[2];
	my $table2 = $_[3];
	my $id2 = $_[4];
	my $value = $_[5];
	my $where = undef;
	if ($value) { $where = " where $id2 = '$value'" }
	my $sqlstmt = "select * from $table1 where $id1 in ".
		"(select $id1 from $table2$where)";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[1]}
	$sth->finish;	
	return @names;
}

sub get_tree_templates(@) {
	my $id = $_[1];
	my %properties;
	my $sqlstmt = "select * from escalation_templates where template_id in ".
		"(select template_id from escalation_tree_template where tree_id = '$id')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) {
		$properties{$values[0]} = [ @values ];
	}
	$sth->finish;
	return %properties;
}

sub get_tree_template_contactgroup(@) {
	my $tree_id = $_[1];
	my $temp_id = $_[2];
	my %properties;
	my $sqlstmt = "select contactgroups.name from contactgroups where contactgroup_id in ".
		"(select contactgroup_id from tree_template_contactgroup where tree_id = '$tree_id' and template_id = '$temp_id')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_file_name(@) {
	my $table = $_[1];
	my $obj = $_[2];
	my $obj_id = $_[3];
	my $sqlstmt = "select files.name from files left join $table on files.file_id = $table.file_id where $obj = '$obj_id'";
	my ($file) = $dbh->selectrow_array($sqlstmt);
	return $file;
}

sub get_contactgroups(@) {
	my $type = $_[1];
	my $obj = $_[2];
	my $sqlstmt = "select * from contactgroups where contactgroup_id in ".
		"(select contactgroup_id from contactgroup_assign where type = '$type' and object = '$obj')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[1]}
	$sth->finish;	
	return @names;
}

sub get_profile_hostgroup(@) {
	my $pid = $_[1];
	my $sqlstmt = "select name from hostgroups where hostgroup_id in ".
		"(select hostgroup_id from profile_hostgroup where hostprofile_id = '$pid')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_profile_external(@) {
	my $pid = $_[1];
	my $sqlstmt = "select name from externals where external_id in ".
		"(select external_id from external_host_profile where hostprofile_id = '$pid')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_servicename_external(@) {
	my $sid = $_[1];
	my $sqlstmt = "select name from externals where external_id in ".
		"(select external_id from external_service_names where servicename_id = '$sid')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_profile_parent(@) {
	my $pid = $_[1];
	my $sqlstmt = "select name from hosts where host_id in ".
		"(select host_id from profile_parent where hostprofile_id = '$pid')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_parents(@) {
	my $sqlstmt = "select name from hosts where host_id in (select distinct parent_id from host_parent)";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_children() {
	my $pid = $_[1];
	my $sqlstmt = "select hosts.host_id, hosts.name from hosts left join host_parent on hosts.host_id = host_parent.host_id where parent_id = '$pid'"; 
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[1]}
	$sth->finish;	
	return @names;
}

sub get_host_hostgroup(@) {
	my $name = $_[1];
	my $sqlstmt = "select name from hosts where host_id  in (select hostgroup_host.host_id from hostgroup_host ".
		"left join hostgroups on hostgroup_host.hostgroup_id = hostgroups.hostgroup_id where hostgroups.name = '$name')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_hostgroup_host(@) {
	my $name = $_[1];
	my $sqlstmt = "select name from hostgroups where hostgroup_id  in (select hostgroup_host.hostgroup_id from hostgroup_host ".
		"left join hosts on hostgroup_host.host_id = hosts.host_id where hosts.name = '$name')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names;
	while(my @values = $sth->fetchrow_array()) { push @names, $values[0]}
	$sth->finish;	
	return @names;
}

sub get_names(@) {
	my $id = $_[1];
	my $table = $_[2];
	my $members = $_[3];
	my @names;
	foreach my $mem (@{$members}) {
		my %m = fetch_one('',$table,$id,$mem);
		push @names, $m{'name'};
	}
	return @names;
}

sub get_ids(@) {
	my $id = $_[1];
	my $table = $_[2];
	my $members = $_[3];
	my @ids;
	foreach my $mem (@{$members}) {
		my %m = fetch_one('',$table,'name',$mem);
		push @ids, $m{$id};
	}
	return @ids;
}

sub get_tree_detail(@) {
	my $name = $_[1];
	my %ranks = ();
	my %templates = ();
	my %t = fetch_one('','escalation_trees','name',$name);
	my $sqlstmt = "select * from escalation_templates where template_id in ".
		"(select escalation_tree_template.template_id from escalation_tree_template left join escalation_trees on ".
		"escalation_tree_template.tree_id = escalation_trees.tree_id where name = '$name')";

	my $sth = $dbh->prepare($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) {
		$templates{$values[1]}{'name'} = $values[1];
		$templates{$values[1]}{'id'} = $values[0];
		my %data = parse_xml($values[3]);
		foreach my $k (keys %data) { 
			if ($k eq 'first_notification') { 
				$ranks{$data{$k}} = $values[1];
			}
			#$data{$k} =~ s/\*/star/g;
			$templates{$values[1]}{$k} = $data{$k};
		}
	}
	foreach my $template (keys %templates) {
		$sqlstmt = "select name from contactgroups where contactgroup_id in ".
			"(select contactgroup_id from contactgroup_assign where type like '%escalation_templates' and object = '$templates{$template}{'id'}')";
		my $sth = $dbh->prepare($sqlstmt);
		$sth->execute;
		while (my @values = $sth->fetchrow_array()) {
			$templates{$template}{'contactgroups'} .= "$values[0],";
		}
		$sth->finish;
		$sqlstmt = "select name from contactgroups where contactgroup_id in ".
			"(select contactgroup_id from tree_template_contactgroup where tree_id = '$t{'tree_id'}' and template_id = '$templates{$template}{'id'}')";
		my $sth = $dbh->prepare($sqlstmt);
		$sth->execute;
		while (my @values = $sth->fetchrow_array()) {
			$templates{$template}{'contactgroups'} .= "$values[0],";
		}
		$sth->finish;
		chop $templates{$template}{'contactgroups'};
	}
	return \%ranks, \%templates;
}

sub get_dependencies(@) {
	my $sid = $_[1];
	my $sqlstmt = "select * from service_dependency where service_id = '$sid'";
	my %dependencies = ();
	my $sth = $dbh->prepare($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) {
		my @vals = ();
		my %t = fetch_one('','service_dependency_templates','id',$values[4]);
		my %h = fetch_one('','hosts','host_id',$values[3]);
		my @vals = ($t{'name'},$h{'name'});
		$dependencies{$values[0]} = [ @vals ]; 
	}	
	return %dependencies;
}

sub get_host_services(@) {
	my $host_id = $_[1];
	my $sqlstmt = "select service_names.name from service_names where servicename_id in (select servicename_id from services where host_id = '$host_id')";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @services = ();
	while(my @values = $sth->fetchrow_array()) {
		push @services, $values[0];
	}
	$sth->finish;
	@services = sort { $a <=> $b } @services;
	return @services
}

sub get_upload(@) {
	my $upload_dir = $_[1];
	my $type = $_[2];
	my @nmaps = ();
	my @imports = ();
	my @errors = ();
	opendir(DIR, $upload_dir) || push @errors, "error: cannot open $upload_dir to read $!";
	my @files = readdir DIR;
	@files = grep{-T "$upload_dir/$_"} @files;
	foreach my $file (@files) {
		open(FILE, "< $upload_dir/$file") || push @errors, "error: cannot open $upload_dir/$file to read $!";
		$/ = undef;
		my $slurp = <FILE>;
		if ($slurp =~ /nmaprun scanner/) {
			push @nmaps, $file;
		} else {
			push @imports, $file;
		}
		close(FILE);		
	}
	close(DIR);
	if ($type eq 'nmap') {
		return \@errors,\@nmaps;
	} else {
		return \@errors,\@imports;
	}	
}



sub get_hostname_servicename(@) {
	my $table = $_[1];
	my $column = $_[2];
	my $value = $_[3];
	my @names = ();
	my $sqlstmt = undef;
	if ($table eq 'services') {
		$sqlstmt = "select host_id, servicename_id from services where $column = '$value'";
	} elsif ($table eq 'service_dependency') {
		$sqlstmt = "select services.host_id, services.servicename_id from services left join service_dependency on ".
			"services.service_id = service_dependency.service_id where $column = '$value'";
	}
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) {
		my %h = fetch_one('','hosts','host_id',$values[0]);
		my %s = fetch_one('','service_names','servicename_id',$values[1]);
		if ($h{'name'} && $s{'name'}) {
			my %rec = ($h{'name'} => $s{'name'});
			push @names, \%rec;
		}
	}
	$sth->finish;		
	return @names;
}

sub get_contactgroup_object(@) {
	my $value = $_[1];
	my $obj_id = $_[2];
	my %obj_id = %{$obj_id};
	my $sqlstmt = "select type, object from contactgroup_assign where contactgroup_id = '$value'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names = ();
	while(my @values = $sth->fetchrow_array()) {
		my %obj = fetch_one('',$values[0],$obj_id{$values[0]},$values[1]);
		my %rec = ($values[0] => $obj{'name'});
		push @names, \%rec;		
	}
	$sth->finish;		
	return @names;
}

sub get_contact_contactgroup(@) {
	my $value = $_[1];
	my $sqlstmt = "select contacts.name from contacts left join contactgroup_contact on ".
		"contactgroup_contact.contact_id = contacts.contact_id where contactgroup_id = '$value'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names = ();
	while(my @values = $sth->fetchrow_array()) {
		push @names, $values[0];		
	}
	$sth->finish;		
	return @names;
}

sub get_tree_contactgroup(@) {
	my $value = $_[1];
	my $sqlstmt = "select name from escalation_trees left join tree_template_contactgroup on ".
		"tree_template_contactgroup.tree_id = escalation_trees.tree_id where contactgroup_id = '$value'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @names = ();
	while(my @values = $sth->fetchrow_array()) {
		push @names, $values[0];		
	}
	$sth->finish;		
	return @names;
}

sub fetch_scripts(@) {
	my $type = $_[1];
	my %scripts = ();
	my $sqlstmt = "select name, script from extended_$type\_info_templates where script != ''"; 
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) {
		$scripts{$values[0]} = $values[1];		
	}
	$sth->finish;		
	return %scripts;
}

sub delete_file(@) {
	my $dir = $_[1];
	my $file = $_[2];
	my $result = 1;
	unlink("$dir/$file") or $result = "Error: Unable to remove $dir/$file $!";
	return $result;	
}

sub fetch_service_extinfo(@) {
	my $extinfo_id = $_[1];
	my $sqlstmt = "select service_id, servicename_id, host_id from services where serviceextinfo_id = '$extinfo_id'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my %service_host = ();
	while(my @values = $sth->fetchrow_array()) {
		my %sn = fetch_one('','service_names','servicename_id',$values[1]);
		my %h = fetch_one('','hosts','host_id',$values[2]);
		my @vals = ($h{'name'},$sn{'name'});
		$service_host{$values[0]} = [ @vals ];
	}
	$sth->finish;		
	return %service_host;
}

sub get_template_properties(@) {
	my $id = $_[1];
	my %properties = fetch_one('','service_templates','servicetemplate_id',$id);
	my $sqlstmt = "select contactgroups.name from contactgroups left join contactgroup_assign on ".
		"contactgroups.contactgroup_id = contactgroup_assign.contactgroup_id where object = '$id'";
	my @contactgroups = ();
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) {
		push @contactgroups, "$values[0],";
	}
	$sth->finish;		
	if ($properties{'parent_id'}) {
		my $pid = $properties{'parent_id'};
		until (!$pid) {
			my %parent = fetch_one('','service_templates','servicetemplate_id',$pid);
			if ($parent{'parent_id'}) { 
				$pid = $parent{'parent_id'};
			} else {
				$pid = 0;
			}
			foreach my $prop (keys %parent) {
				if (!$properties{$prop}) {
					$properties{$prop} = $parent{$prop};
				}
			}
			my $sqlstmt = "select contactgroups.name from contactgroups left join contactgroup_assign on ".
				"contactgroups.contactgroup_id = contactgroup_assign.contactgroup_id where object = '$id'";
			my $sth = $dbh->prepare ($sqlstmt);
			$sth->execute;
			while(my @values = $sth->fetchrow_array()) {
				push @contactgroups, "$values[0],";
			}
			$sth->finish;		

		}
	}
	@contactgroups = sort @contactgroups;
	$properties{'conatctgroup'} = [ @contactgroups ]; 
	return %properties;
}

sub nagios_defaults() {
	my %nagios = ();
	$nagios{'log_file'} = '/usr/local/nagios/var/nagios.log';
	$nagios{'resource_file'} = '/usr/local/nagios/etc/private/resource.cfg';
	$nagios{'status_file'} = '/usr/local/nagios/var/status.log';
	$nagios{'nagios_user'} = 'nagios';
	$nagios{'nagios_group'} = 'nagios';
	$nagios{'check_external_commands'} = '1';
	$nagios{'command_check_interval'} = '1';
	$nagios{'command_file'} = '/usr/local/nagios/var/rw/nagios.cmd';
	$nagios{'comment_file'} = '/usr/local/nagios/var/nagioscomment.log';
	$nagios{'downtime_file'} = '/usr/local/nagios/var/nagiosdowntime.log';
	$nagios{'lock_file'} = '/var/run/nagios.pid';
	$nagios{'temp_file'} = '/usr/local/nagios/var/nagios.tmp';
	$nagios{'log_rotation_method'} = 'd';
	$nagios{'log_archive_path'} = '/usr/local/nagios/var/archives';
	$nagios{'use_syslog'} = '1';
	$nagios{'log_notifications'} = '1';
	$nagios{'log_service_retries'} = '1';
	$nagios{'log_host_retries'} = '1';
	$nagios{'log_event_handlers'} = '1';
	$nagios{'log_initial_states'} = '0';
	$nagios{'log_external_commands'} = '1';
	$nagios{'log_passive_service_checks'} = '1';
	$nagios{'global_host_event_handler'} = '';
	$nagios{'global_service_event_handler'} = '';
	$nagios{'inter_check_delay_method'} = 's';
	$nagios{'service_interleave_factor'} = 's';
	$nagios{'max_concurrent_checks'} = '100';
	$nagios{'service_reaper_frequency'} = '10';
	$nagios{'sleep_time'} = '1';
	$nagios{'service_check_timeout'} = '60';
	$nagios{'host_check_timeout'} = '30';
	$nagios{'event_handler_timeout'} = '30';
	$nagios{'notification_timeout'} = '30';
	$nagios{'ocsp_timeout'} = '5';
	$nagios{'perfdata_timeout'} = '5';
	$nagios{'retain_state_information'} = '1';
	$nagios{'state_retention_file'} = '/usr/local/nagios/var/nagiosstatus.sav';
	$nagios{'retention_update_interval'} = '60';
	$nagios{'use_retained_program_state'} = '0';
	$nagios{'interval_length'} = '60';
	$nagios{'use_agressive_host_checking'} = '0';
	$nagios{'execute_service_checks'} = '1';
	$nagios{'accept_passive_service_checks'} = '1';
	$nagios{'enable_notifications'} = '1';
	$nagios{'enable_event_handlers'} = '1';
	$nagios{'process_performance_data'} = '0';
	$nagios{'host_perfdata_command'} = 'process-host-perfdata';
	$nagios{'service_perfdata_command'} = 'process-service-perfdata';
	$nagios{'obsess_over_services'} = '0';
	$nagios{'ocsp_command'} = '';
	$nagios{'check_for_orphaned_services'} = '1';
	$nagios{'check_service_freshness'} = '1';
	$nagios{'freshness_check_interval'} = '60';
	$nagios{'aggregate_status_updates'} = '1';
	$nagios{'status_update_interval'} = '15';
	$nagios{'enable_flap_detection'} = '0';
	$nagios{'low_host_flap_threshold'} = '5.0';
	$nagios{'high_host_flap_threshold'} = '20.0';
	$nagios{'low_service_flap_threshold'} = '5.0';
	$nagios{'high_service_flap_threshold'} = '20.0';
	$nagios{'date_format'} = 'us';
	$nagios{'illegal_object_name_chars'} = q(`~!$%^&*|'"<>?,()'=);
	$nagios{'illegal_macro_output_chars'} = q(`~$&|'"<>);
	$nagios{'admin_email'} = 'nagios';
	$nagios{'admin_pager'} = 'pagenagios';
	return %nagios;
}

sub load_nagios_cfg() {
	my @errors = ();
	my $result = delete_all('','setup','type','nagios');
	if ($result =~ /^Error/) { push @errors, $result }
	my %nagios = nagios_defaults();
	my %nagios_dir = fetch_one('','setup','name','nagios_home');
	open(FILE, "< $nagios_dir{'value'}/nagios.cfg") || push @errors, "Error: Unable to open $nagios_dir{'value'}/nagios.cfg $!";
	while (my $line = <FILE>) {
		if ($line =~ /^#|^cfg_file|^cfg_dir/) { next }
		if ($line =~ /(\S+)=(.*)$/) {
			my $directive = $1;
			my $value = $2;
			$nagios{$directive} = $value;
		}
	}
	close (FILE);
	foreach my $name (keys %nagios) {
		my @vals = ($name,'nagios',$nagios{$name});
		my $result = insert_obj('','setup',\@vals);
		if ($result =~ /^Error/) { push @errors, $result }
	}
	return @errors;
}

1;


