#!/usr/bin/perl
# MonArch - Groundwork Monitor Architect
# monarch_load.pl
#
###############################################################################
# Release 0.94c
# 16-May-2005
###############################################################################
#

use lib qq(STRINGTOREPLACE/lib);
use DBI;
use XML::LibXML;
use strict;
use MonarchConf;

my ($database,$user,$passwd) = Conf->get_dbauth();
my $dsn = "DBI:mysql:$database:localhost";
my $dbh = DBI->connect($dsn, $user, $passwd, {'RaiseError' => 1});
my $debug = 0;

sub parse_xml($) {
	my $data = shift;
	my %properties = ();
	if ($data) {
		my $parser = XML::LibXML->new();
		my $doc = $parser->parse_string($data);
		my @nodes = $doc->findnodes( "//prop" );
		foreach my $node (@nodes) {
			if ($node->hasAttributes()) {
				my $property = $node->getAttribute('name');
				my $value = $node->textContent;
				$value =~ s/\s+$|\n//g;
				if ($property =~ /command$/) {
					my $command_line = '';
					if ($value) {
						my @command = split(/!/, $value);
						$properties{$property} = $command[0];
						if ($command[1]) {
							foreach my $c (@command) {
								$command_line .= "$c!";
							}
						}
					}
					$command_line =~ s/!$//;
					$properties{'command_line'} = $command_line;
				} elsif ($property =~ /last_notification$/) {
					my $value = $node->textContent;
					$value =~ s/\s+$|\n//g;
					if ($value == 0) {
						$properties{$property} = '-zero-';
					} else {					
						$properties{$property} = $value;
					}
				} else {
					$properties{$property} = $value;
				}
			}
		}
		return %properties;
	} else {
		$properties{'error'} = "Empty String (parse_xml)";
		return %properties;
	}
}

sub fetch_list_where(@) {
	my $table = $_[1];
	my $list = $_[2];
	my $where = $_[3];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']]%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "select $list from $table where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my @elements = ();
	my @values = ();
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_list_hash_array(@) {
	my $table = $_[1];
	my $where = $_[2];
	my %where = %{$where};
	my %elements = ();
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH|NONE/) {
			$where_clause .= " $name = '$where{$name}' and"; 
		}
	}
	$where_clause =~ s/ and$//;
	if ($where_clause) { $where_clause = " where $where_clause" }
	my $sqlstmt = "select * from $table$where_clause";
	my @values = undef;
	my $sth = $dbh->prepare($sqlstmt);
	if ($sth->execute) {
		while(@values = $sth->fetchrow_array()) {
			$elements{$values[0]} = [ @values ];
		}
		return %elements;
	} else {
		$elements{'error'} = "Error: $@";
		return %elements;
	}
	$sth->finish;
}

sub insert_obj(@) {
	my $table = $_[1];
	my $values =  $_[2];
	my @values = @{$values};
	my $valstr = undef;
	foreach my $val (@values) {
		$val = $dbh->quote($val);
		$valstr .= "$val,";
	}
	chop $valstr;
	my $sqlstmt = "insert into $table values($valstr)";
	my $sth = undef;
	eval{
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute();
	};	
	$sth->finish;
	if ($@) {
		return "Error: $sqlstmt $@";
	} else {
		return 1;
	}
}

sub insert_obj_id(@) {
	my $table = $_[1];
	my $values =  $_[2];
	my $id =  $_[3];
	my @values = @{$values};
	my $valstr = undef;
	foreach my $val (@values) {
		$val = $dbh->quote($val);
		$valstr .= "$val,";
	}
	chop $valstr;
	my $sqlstmt = "insert into $table values($valstr)";
	my $sth = undef;
	eval{
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute();
	};
	$sth->finish;
	if ($@) {
		return "Error: $sqlstmt $@";
	} else {
		my $sqlstmt = "select max($id) as lastrow from $table";
		my $id = $dbh->selectrow_array($sqlstmt);
		return $id;
	}
}


sub fetch_one(@) {
	my $table = $_[1];
	my $name = $_[2];
	my $value = $_[3];
	my $values = undef;
	my %properties = ();
	$value = $dbh->quote($value);
	my $sqlstmt = "select * from $table where $name = $value";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	$values = $sth->fetchrow_hashref();
	foreach my $key (keys %{$values}) {
		if ($key eq 'data') {
			my %data = parse_xml($values->{$key});
			foreach my $k (keys %data) {
				$properties{$k} = $data{$k};
			}
		} else {
			$properties{$key} = $values->{$key};
		}
	}
	$sth->finish;
	return %properties;
}

sub fetch_one_where(@) {
	my $table = $_[1];
	my $where = $_[2];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "select * from $table where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	my $values = $sth->fetchrow_hashref();
	my %properties = ();
	foreach my $key (keys %{$values}) {
		if ($key eq 'data') {
			my %data = parse_xml($values->{$key});
			foreach my $k (keys %data) {
				$properties{$k} = $data{$k};
			}
		} else {
			$properties{$key} = $values->{$key};
		}
	}
	$sth->finish;
	return %properties;
}

sub fetch_list(@) {
	my $table = $_[1];
	my $list = $_[2];
	my @values = ();
	my @elements = ();
	my $sqlstmt = "select $list from $table";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub fetch_distinct_list(@) {
	my $table = $_[1];
	my $list = $_[2];
	my @values = ();
	my @elements = ();
	my $sqlstmt = "select distinct $list from $table";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(@values = $sth->fetchrow_array()) {
		push @elements, $values[0];
	}
	$sth->finish;
	return @elements;
}

sub delete_all(@) {
	my $table = $_[1];
	my $name = $_[2];
	my $obj = $_[3];
	my $sqlstmt = "delete from $table where $name = '$obj'";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		return 1;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}

sub delete_one_where(@) {
	my $table = $_[1];
	my $where = $_[2];
	my %where = %{$where};
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			if ($name eq 'data') {
				my $like = '%[CDATA['.$where{$name}.']]%';
				$where_clause .= " $name like '$like' and"; 
			} else {
				$where_clause .= " $name = '$where{$name}' and"; 
			}
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "delete from $table where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		return 1;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}

sub update_obj_where(@) {
	my $table = $_[1];
	my $values =  $_[2];
	my $where = $_[3];
	my %values = %{$values};
	my %where = %{$where};
	my $valstr = undef;
	foreach my $key (keys %values) {
		unless ($key =~ /HASH/) {
			my $val = $dbh->quote($values{$key});
			$valstr .= "$key = $val,";
		}
	}
	chop $valstr;
	my $where_clause = undef;
	foreach my $name (keys %where) {
		unless ($name =~ /HASH/) {
			$where_clause .= " $name = '$where{$name}' and"; 
		}
	}
	$where_clause =~ s/ and$//;
	my $sqlstmt = "update $table set $valstr where $where_clause";
	my $sth = $dbh->prepare ($sqlstmt);
	if ($sth->execute) {
		return 1;
	} else {
		return "Error: $sqlstmt $@";
	}
	$sth->finish;
}

sub datetime() {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	$year += 1900;
	$mon++;
	if ($mon =~ /^\d{1}$/) { $mon = "0".$mon }
	if ($mday =~ /^\d{1}$/) { $mday = "0".$mday }
	if ($hour =~ /^\d{1}$/) { $hour = "0".$hour }
	if ($min =~ /^\d{1}$/) { $min = "0".$min }
	if ($sec =~ /^\d{1}$/) { $sec = "0".$sec }
	return "$year-$mon-$mday $hour:$min:$sec";
}


sub update_status(@) {
	my $type = $_[1];
	my $message = $_[2];
	$message = $dbh->quote($message);
	my $sqlstmt = "insert into stage_status values ('','$type',$message)";	
	my $sth = $dbh->prepare ($sqlstmt);
	unless ($sth->execute) {
		if ($debug) { print "Error: $sqlstmt $@" }
	}
	$sth->finish;	
}

my $tables = qq(commands
contact_command
contact_templates
contactgroup_assign
contactgroup_contact
contactgroups
contacts
escalation_templates
escalation_tree_template
escalation_trees
extended_host_info_templates
extended_info_coords
extended_service_info_templates
host_dependencies
host_parent
host_templates
hostgroup_host
hostgroups
hosts
service_dependency
service_dependency_templates
service_templates
servicegroup_service
servicegroups
services
stage_escalations
stage_host_hostgroups
stage_host_services
stage_hosts
stage_other
time_periods
tree_template_contactgroup);

#
###########################################################################################################################
# Stage
#

sub stage_load(@) {
	my %nagios_home = fetch_one('','setup','name','nagios_home');
	my %nagios_cgi = fetch_one('','setup','name','cgi_home');
	my $index = 0;
	my @results = ();
	my %template = ();
	my %host_template_reassign = ();
	my %parents = ();
	my %parent_host = ();
	my %dependency = ();
	my %service_dep = ();
	my %file_type = ();
	my %file_object = ('commands' => 'checkcommands.cfg',
		'miscommands' => 'miscommands.cfg',
		'contact_templates' => 'contact_templates.cfg',
		'contactgroups' => 'contactgroups.cfg',
		'contacts' => 'contacts.cfg',
		'escalation_templates' => 'escalation_templates.cfg',
		'escalations' => 'escalations.cfg',
		'extended_host_info_templates' => 'extended_host_info_templates.cfg',
		'extended_host_info' => 'extended_host_info.cfg',
		'extended_service_info_templates' => 'extended_service_info_templates.cfg',
		'extended_service_info' => 'extended_service_info.cfg',
		'host_dependencies' => 'host_dependencies.cfg',
		'host_templates' => 'host_templates.cfg',
		'hostgroups' => 'hostgroups.cfg',
		'service_dependency' => 'service_dependency.cfg',
		'service_dependency_templates' => 'service_dependency_templates.cfg',
		'service_templates' => 'service_templates.cfg',
		'servicegroups' => 'servicegroups.cfg',
		'time_periods' => 'time_periods.cfg',
		'services' => 'services',
		'hosts' => 'hosts.cfg');
	my @stage = ('load_status','','1');
	my $result = insert_obj('','setup',\@stage);
	my $sqlstmt = "truncate table stage_status";
	my $sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;
	my $dt = datetime();
	update_status('','start',$dt);
	my $truncate = 1;
	if ($truncate) {
		my @tables = split(/\n/, $tables);
		foreach my $table (@tables) {
			my $sqlstmt = "truncate table $table";
			my $sth = $dbh->prepare ($sqlstmt);
			eval{$sth->execute()};
			update_status('','error', "Error: $sqlstmt $@") if $@;
		}
	}
	my @tables = ('files','file_host','file_service');
	foreach my $table (@tables) {
		my $sqlstmt = "truncate table $table";
		my $sth = $dbh->prepare ($sqlstmt);
		eval{$sth->execute()};
		update_status('','error', "Error: $sqlstmt $@") if $@;
	}
	update_status('','info', "Reading: $nagios_home{'value'}/nagios.cfg");
	$sqlstmt = "delete from setup where type = 'file'";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;

	my %file_paths = ();
	my @files = ();
	my @dirs = ();
	my $resource_file = undef;
	open(FILE, "< $nagios_home{'value'}/nagios.cfg") or update_status('','error',"error: cannot open $nagios_home{'value'}/nagios.cfg to read $!");
	while (my $line = <FILE>) {
		if ($line =~ /^cfg_file=(\S+)$/) { push @files, $1 }
		if ($line =~ /^cfg_dir=(\S+)$/) { push @dirs, $1 }
		if ($line =~ /^resource_file=(\S+)$/) { $resource_file = $1 }
	}
	close(FILE);
	foreach my $dir (@dirs) {
		opendir(DIR, $dir) || update_status('','error', "error: cannot open $dir to read $!");
		while (my $file = readdir(DIR)) {
			if ($file =~ /(\S+\.cfg$)/) { $file_paths{"$dir/$1"} = $1 }
		}
		close(DIR);
	}
	update_status('','info', "Reading: $nagios_home{'value'}/cgi.cfg");
	open(FILE, "< $nagios_cgi{'value'}/cgi.cfg") or update_status('','error', "error: cannot open $nagios_cgi{'value'}/cgi.cfg to read $!");
	while (my $line = <FILE>) {
		if ($line =~ /^xedtemplate_config_file=(\S+)$/) { push @files, $1 }
	}
	close(FILE);


	foreach my $file (@files) {
		if ($file =~ /.*\/(.*\.cfg)/) { $file_paths{$file} = $1 }
	}
	foreach my $file (keys %file_paths) {
		if ($debug) {
			print "\nfile: $file\n";
			my $x = <STDIN>;
		}

		if ($file =~ /HASH/) { next }
		my $added = 0;
		my $rejected = 0;
		if ($file =~ /nagios.cfg|resource.cfg|minimal.cfg|perfparse.cfg|cgi.cfg/i) { next }
		my @properties = ();
		open (CFG, "< $file") or update_status('','error', "error: cannot open $file $!");
		while (my $line = <CFG>){
			if ($line =~ /#GROUNDWORK#|^#GW/) { next }
			unless ($line =~ /^#/) { $line =~ s/\{//g }
			$line =~ s/\s+$//;
			if ($line =~ /^\s*}/) {
				my @comment = ();
				my %element = ();
				$element{'data'} = "<?xml version=\"1.0\" ?>\n<data>\n";
				foreach my $prop (@properties) {
					if ($prop =~ /\s*define\s+(\S+)/i) { 
						$element{'type'} = $1;
					} elsif ($prop =~ /^\s*register\s+0/i) {
						$element{'type'} .= '_template';
					} elsif ($prop =~ /^\#/) {
						push @comment, "$prop\n";			
					} elsif ($prop =~ /^\s*(\S+)\s+(\S+.*?)(\t|;|$)/) {
						my ($name,$value) = ($1,$2);
						$name =~ s/\s+$//;
						if ($element{$name} eq 'members') {
							$element{$name} .= "$value,";
						} else {
							if ($value eq '0') { $value = "-zero-" }
							$element{$name} = $value;
						}
					}
				}
				if ($element{'type'} eq 'host') {
					$element{'data'} .= " <prop name=\"file_name\"><![CDATA[$file_paths{$file}]]>\n";
					$element{'data'} .= " </prop>\n";
					$element{'name'} .= $element{'host_name'};
					$element{'parent'} .= $element{'host_name'};
					delete $element{'host_name'};
					$file_object{'hosts'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'service') {
					$element{'data'} .= " <prop name=\"file_name\"><![CDATA[$file_paths{$file}]]>\n";
					$element{'data'} .= " </prop>\n";
					if (!$element{'service_description'}) { $element{'service_description'} = $element{'use'} }
					$element{'name'} = $element{'service_description'};
					delete $element{'service_description'};
					if ($element{'hostgroup_name'}) {
						my @h = split(/,/, $element{'hostgroup_name'});
						$element{'parent'} = $h[0];
					} elsif ($element{'host_name'}) {
						my @h = split(/,/, $element{'host_name'});
						$element{'parent'} = $h[0];
					}
					$file_object{'services'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'hostgroupescalation') {
					$element{'name'} = "hostgroupescalation-$index";
					$index++;
				} elsif ($element{'type'} eq 'hostescalation') {
					if ($element{'hostgroup_name'}) {
						$element{'name'} = "hostgroup-$index";
						$element{'parent'} = "hostgroup";
					} elsif ($element{'host_name'}) {
 						$element{'name'} = "host-$index";
						$element{'parent'} = "host";
					}
					$index++;
				} elsif ($element{'type'} eq 'serviceescalation') {
					if ($element{'hostgroup_name'}) {
						$element{'name'} = "hostgroup-$index";
						$element{'parent'} = "hostgroup";
					} elsif ($element{'service_description'} && $element{'service_description'} ne '*') {
 						$element{'name'} = "$element{'service_description'}-$index";
						$element{'type'} = 'serviceescalation-service';
						if ($element{'use'}) {
							$element{'parent'} = $element{'use'};
						} else {
							$element{'parent'} = "no-template-$index";
						}
					} elsif ($element{'host_name'}) {
 						$element{'name'} = "host-$index";
						$element{'parent'} = "host";
					}
					$index++;
				} elsif ($element{'type'} eq 'servicedependency') {
					$element{'name'} = $element{'dependent_service_description'};
					delete $element{'dependent_service_description'};
					if ($element{'dependent_hostgroup_name'}) {
						my @h = split(/,/, $element{'dependent_hostgroup_name'});
						$element{'parent'} = $h[0];
					} elsif ($element{'dependent_host_name'}) {
						my @h = split(/,/, $element{'dependent_host_name'});
						$element{'parent'} = $h[0];
					}
					$file_object{'service_dependency'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'hostdependency') {
					my @h = split(/,/, $element{'dependent_host_name'});
					$element{'name'} = "$h[0]-$index";
					@h = split(/,/, $element{'host_name'});
					$element{'parent'} = $h[0];
					$index++;
				} elsif ($element{'type'} eq 'hostextinfo') {
					my @h = split(/,/, $element{'host_name'});
					if ($h[1]) {
						$element{'name'} = $h[0];
						$element{'parent'} = $element{'type'};
					} else {
						$element{'name'} = $element{'host_name'};
						delete $element{'host_name'};
					}
				} elsif ($element{'type'} eq 'serviceextinfo') {
					$element{'name'} .= $element{'service_description'};
					$element{'parent'} .= $element{'host_name'};
					delete $element{'host_name'};
					delete $element{'service_description'};
				} elsif ($element{'type'} eq 'timeperiod') {
					$element{'name'} .= $element{'timeperiod_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					delete $element{'timeperiod_name'};
					$file_object{'time_periods'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'command') {
					$element{'name'} .= $element{'command_name'};
					$element{'parent'} = $file_paths{$file};
					delete $element{'command_name'};
					if ($element{'command_line'} =~ /\$CONTACTEMAIL\$|\$NOTIFICATIONTYPE\$|\$CONTACTPAGER\$/) {
						$file_object{'miscommands'} = $file_paths{$file};
					} else {
						$file_object{'commands'} = 'checkcommands.cfg';
					}
				} elsif ($element{'type'} eq 'contact') {
					$element{'name'} .= $element{'contact_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					delete $element{'contact_name'};
					$file_object{'contacts'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'contactgroup') {
					$element{'name'} .= $element{'contactgroup_name'};
					delete $element{'contactgroup_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					$element{'members'} =~ s/,$//;
					$file_object{'contactgroups'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'contact_templates') {
					$file_object{'contact_templates'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'hostgroup') {
					$element{'type'} = 'hostgroup-host';
					$element{'name'} .= $element{'hostgroup_name'};
					delete $element{'hostgroup_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					$element{'members'} =~ s/,$//;
				} elsif ($element{'type'} eq 'servicegroup') {
					$element{'type'} = 'servicegroup';
					$element{'name'} .= $element{'servicegroup_name'};
					delete $element{'servicegroup_name'};
					$element{'parent'} .= $element{'alias'};
					delete $element{'alias'};
					$element{'members'} =~ s/,$//;
					$file_object{'servicegroups'} = $file_paths{$file};
				} elsif ($element{'type'} eq 'service_template') {
					if (!$element{'use'}) { $element{'use'} = 'no-parent' }
					$element{'parent'} = $element{'use'};
					delete $element{'use'};
				}
				if ($element{'type'} =~ /template/ && !$element{'name'}) {
					next;
				}
				foreach my $ele (keys %element) {
					if (!$element{$ele} || $ele =~ /HASH|^name$|^type$|^data$|^parent$/) { next }
					$element{'data'} .= " <prop name=\"$ele\"><![CDATA[$element{$ele}]]>\n";
					$element{'data'} .= " </prop>\n";
				}
				$file_type{$file_paths{$file}} = $element{'type'};
				if (!$element{'parent'}) { $element{'parent'} = $element{'name'} }
				$element{'data'} .= "</data>";
				$element{'name'} = $dbh->quote($element{'name'});
				$element{'type'} = $dbh->quote($element{'type'});
				$element{'parent'} = $dbh->quote($element{'parent'});
				$element{'data'} = $dbh->quote($element{'data'});
				my $comment = pop @comment;
				$comment = $dbh->quote($comment);
# insert 
				my $sqlstmt = "insert into stage_other values($element{'name'},$element{'type'},$element{'parent'},$element{'data'},$comment)";
				$sth = $dbh->prepare($sqlstmt);
				eval{$sth->execute()};					
				if ($@) {
					if ($@ =~ /DBD::mysql::st execute failed: Duplicate entry (\S+) for/) {
						update_status('','error', "Duplicate: Object $1 already exists and will be ignored (file: $file).");
						if ($debug) { update_status('','error', "SQL: $sqlstmt $@") }
					} else {
						update_status('','error', "Error: $file $sqlstmt $@");
					}
					$rejected++;
				} else {
					$added++;
				}
				@properties = ();
				push @properties, $line;
			} else {
				push @properties, $line;
			}
		}
		push @results, "File $file: $added objects staged $rejected object rejected.";

	}
	foreach my $file (sort keys %file_paths) {
		my $path = $file;
		$path =~ s/\/$file_paths{$file}//;
		my @values = ('',$file_paths{$file},$path,$file_type{$file_paths{$file}});
		my $file_id = insert_obj_id('','files',\@values,'file_id');
		if ($file_id =~ /Error/) { update_status('','error', $file_id) }
		foreach my $obj (keys %file_object) {
			if ($file_object{$obj} eq $file_paths{$file}) {
				@values = ($obj,'file',$file_id);
				my $result = insert_obj('','setup',\@values);
				if ($result =~ /Error/) { update_status('','error',$result) }
				delete $file_object{$obj};
			}
		}
	}
	foreach my $obj (keys %file_object) {
		my @values = ('',$file_object{$obj},$nagios_home{'value'},$obj);
		my $file_id = insert_obj_id('','files',\@values,'file_id');
		if ($file_id =~ /Error/) { update_status('','error', $file_id) }
		@values = ($obj,'file',$file_id);
		my $result = insert_obj('','setup',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	foreach (@results) { update_status('','info',$_) }
	$dt = datetime();
	update_status('','info',"$dt Stage completed.");

	update_status('','info', "Reading: $resource_file");
	my %w = ('name' => 'resource_home');
	my %u = ('value' => $resource_file);
	$result = update_obj_where('','setup',\%u,\%w);
	if ($result =~ /Error/) { update_status('','error',$result) }
	open(FILE, "< $resource_file") or update_status('','error', "error: cannot open $resource_file to read $!");
	while (my $line = <FILE>) {
		if ($line =~ /\$USER(\d+)\$=(\S+)$/) { 
			if ($1 <= 32) {
				my %w = ('name' => "user$1");
				my %u = ('value' => $2);
				my $result = update_obj_where('','setup',\%u,\%w);
			}
		}
	}
	close(FILE);
	update_status('','end',"$dt Stage completed.")
}

#
###########################################################################################################################
# Process
#

sub process_load() {
	my $dt = datetime();
	update_status('','start',"$dt Processing.");
	my $xml_begin = "<?xml version=\"1.0\" ?>\n<data>\n";
	my $xml_end = "</data>";

# commands
	update_status('','info',"Loading commands");
	my $type = 'command';
	my %where = ('type' => $type);
	my %objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my $cmdtype = 'check';
		if ($data{'command_line'} =~ /\$CONTACTEMAIL\$|\$NOTIFICATIONTYPE\$|\$CONTACTPAGER\$|\$PERFDATA\$/) {
			$cmdtype = 'notify';
		}
		my $data = $xml_begin;
		$data .= " <prop name=\"command_line\"><![CDATA[$data{'command_line'}]]>\n";				
		$data .= " </prop>\n";
		$data .= $xml_end;
		my @values = ('',$objects{$key}[0],$cmdtype,$data,$objects{$key}[4]);
		my $result = insert_obj('','commands',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	my $result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# timeperiods
	update_status('','info',"Loading timeperiods");
	$type = 'timeperiod';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my @values = ('',$objects{$key}[0],$objects{$key}[2],$objects{$key}[3],$objects{$key}[4]);
		my $result = insert_obj('','time_periods',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# contact template
	update_status('','info',"Loading contact templates");
	$type = 'contact_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my %h = fetch_one('','time_periods','name',$data{'host_notification_period'});
		my %s = fetch_one('','time_periods','name',$data{'service_notification_period'});
		delete $data{'host_notification_period'};
		delete $data{'service_notification_period'};
	
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /command/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}
		$data .= $xml_end;				
		my @values = ('',$objects{$key}[0],$h{'timeperiod_id'},$s{'timeperiod_id'},$data,$objects{$key}[4]);
		my $id = insert_obj_id('','contact_templates',\@values,'contacttemplate_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		my @h = split(/,/, $data{'host_notification_commands'});		
		foreach my $h (@h) {
			$h =~ s/^\s+|\s+$//;
			my %c = fetch_one('','commands','name',$h);
			if (!$c{'command_id'}) { next }
			my @vals = ($id,'host',$c{'command_id'});
			my $result = insert_obj('','contact_command',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
		my @s = split(/,/, $data{'service_notification_commands'});		
		foreach my $s (@s) {
			$s =~ s/^\s+|\s+$//;
			my %c = fetch_one('','commands','name',$s);
			if (!$c{'command_id'}) { next }
			my @vals = ($id,'service',$c{'command_id'});
			my $result = insert_obj('','contact_command',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# contacts
	update_status('','info',"Loading contacts");
	$type = 'contact';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	my $index = 1;
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		if (!$data{'use'}) {
			update_status('','error', "Contact $objects{$key}[0] does not use a template.");
			my $props = 0;
			foreach (keys %data) { $props++ }
			if ($data{'email'}) { $props -= 1 }
			if ($data{'pager'}) { $props -= 1 }
			my %hostnotify = fetch_one('','time_periods','name',$data{'host_notification_period'});
			my %servicenotify = fetch_one('','time_periods','name',$data{'service_notification_period'});
			$data{'host_notification_period'} = $hostnotify{'timeperiod_id'};
			$data{'service_notification_period'} = $servicenotify{'timeperiod_id'};
			my @h = split(/,/, $data{'host_notification_commands'});
			my %h_commands = ();
			foreach my $h (@h) {
				$h =~ s/^\s+|\s+$//;
				my %c = fetch_one('','commands','name',$h);
				$h_commands{$h} = $c{'command_id'};
				$props++;
			}
			my @s = split(/,/, $data{'service_notification_commands'});
			my %s_commands = ();
			foreach my $s (@s) {
				$s =~ s/^\s+|\s+$//;
				my %c = fetch_one('','commands','name',$s);
				$s_commands{$s} = $c{'command_id'};
				$props++;
			}
			my %w = ();
			my %templates = fetch_list_hash_array('','contact_templates',\%w);
			foreach my $t (keys %templates) {
				my $gotmatch = 0;
				my %w = ('contacttemplate_id' => $t);
				my @tmplate_commands = fetch_list_where('','contact_command','command_id',\%w);
				foreach my $tc (@tmplate_commands) {
					foreach my $key (keys %h_commands) {
						if ($h_commands{$key} == $tc) { $gotmatch++ }
					}
					foreach my $key (keys %s_commands) {
						if ($s_commands{$key} == $tc) { $gotmatch++ }
					}
				}
				my %tdata = parse_xml($templates{$t}[4]);
				if ($data{'host_notification_period'} eq $templates{$t}[2]) { $gotmatch++ }
				if ($data{'service_notification_period'} eq $templates{$t}[3]) { $gotmatch++ }
				foreach my $d (keys %data) {
					unless ($d =~ /email|pager|period|command/) { 
						if ($data{$d} eq $tdata{$d}) { $gotmatch++ }
					}
				}		
				if ($gotmatch == $props) {
					$data{'use'} = $templates{$t}[1];
					update_status('','action', "Assigned: Contact template $templates{$t}[1] assigned to contact $objects{$key}[0]");
					last;
				}
			}
			if (!$data{'use'}) {
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /email|pager|period|command/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}
				$data .= $xml_end;				
				my $name = "generic-contact-$index";
				my $comment = undef;
				my @values = ('',$name,$data{'host_notification_period'},$data{'service_notification_period'},$data,$comment);
				my $id = insert_obj_id('','contact_templates',\@values,'contacttemplate_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				foreach my $key (keys %h_commands) {
					if (!$h_commands{$key}) { next }
					my @vals = ($id,'host',$h_commands{$key});
					my $result = insert_obj('','contact_command',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				foreach my $key (keys %s_commands) {
					if (!$s_commands{$key}) { next }
					my @vals = ($id,'service',$s_commands{$key});
					my $result = insert_obj('','contact_command',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				update_status('','action', "Added: Contact template $name, assigned to contact $objects{$key}[0]");
				$data{'use'} = $name;
				$index++;
			}
		} 
		my %t = fetch_one('','contact_templates','name',$data{'use'});
		my @values = ('',$objects{$key}[0],$objects{$key}[2],$data{'email'},$data{'pager'},$t{'contacttemplate_id'},'1',$objects{$key}[4]);
		my $result = insert_obj('','contacts',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# contactgroups
	if ($debug) {
		print "\ncontactgroups\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading contactgroups");
	$type = 'contactgroup';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my @values = ('',$objects{$key}[0],$objects{$key}[2],$objects{$key}[4]);
		my $id = insert_obj_id('','contactgroups',\@values,'contactgroup_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		my %data = parse_xml($objects{$key}[3]);
		my @members = split(/,/, $data{'members'});
		foreach my $mem (@members) {
			$mem =~ s/^\s+|\s+$//;
			my %cid = fetch_one('','contacts','name',$mem);
			my @vals = ($id,$cid{'contact_id'});
			my $result = insert_obj('','contactgroup_contact',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# host templates
	if ($debug) {
		print "\nhost templates\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading host templates");
	$type = 'host_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my $props = 0;
		foreach (keys %data) { $props++ }
		my $data = undef;
		if ($data{'parents'}) { 
			$props -= 1;
			$data = $xml_begin;
			$data .= " <prop name=\"parents\"><![CDATA[$data{'parents'}]]>\n";		
			$data .= " </prop>\n";
			$data .= $xml_end;
		}
		if ($data) {
			my @vals = ($objects{$key}[0],'parent_directive',$data{'use'},$data,'');
			my $result = insert_obj('','stage_other',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
			update_status('','error', "Parent directive on host template $objects{$key}[0].");
			update_status('','action', "Parent directive moved: Parents $data{'parents'} moved from host template $objects{$key}[0] to hosts.");
		}
		if ($data{'use'} && $props == 1) {
			my @vals = ($objects{$key}[0],'redundant_template',$data{'use'},$data,'');
			my $result = insert_obj('','stage_other',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
			update_status('','error', "Redundant: Host template $objects{$key}[0] is redundant as it references $data{'use'} but has no properties itself.");
			update_status('','action', "Staged for removal: Host template $objects{$key}[0]. Hosts using $objects{$key}[0] will be referenced to $data{'use'}.");
		} elsif ($data{'use'}) {
			my %w = ('type' => 'host_template','name' => $data{'use'});
			my %parent = fetch_one_where('','stage_other',\%w);
			foreach my $p (keys %parent) {
				if ($p =~ /name|parent|comment/) { next	}
				unless ($data{$p} || $p eq 'parents') {
					$data{$p} = $parent{$p};
				}
			}
			update_status('','note', "Host template $objects{$key}[0] is a host template that references a host template.");
			update_status('','action', "Reference removed: Host template $objects{$key}[0] will no longer reference to $data{'use'}. Please check Manage->Host Templates->$objects{$key}[0] values.");
		}
		my %t = fetch_one('','time_periods','name',$data{'notification_period'});
		my %c = fetch_one('','commands','name',$data{'check_command'});
		my %e = fetch_one('','commands','name',$data{'event_handler'});
		delete $data{'notification_period'};
		delete $data{'check_command'};
		delete $data{'event_handler'};
		$data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /parents|contact_groups|use/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}	
		$data .= $xml_end;
		my @values = ('',$objects{$key}[0],$t{'timeperiod_id'},$c{'command_id'},$e{'command_id'},$data,$objects{$key}[4]);
		my $id = insert_obj_id('','host_templates',\@values,'hosttemplate_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		my @members = split(/,/, $data{'contact_groups'});
		foreach my $mem (@members) {
			$mem =~ s/^\s+|\s+$//;
			my %c = fetch_one('','contactgroups','name',$mem);
			my @vals = ($c{'contactgroup_id'},'host_templates',$id);
			my $result = insert_obj('','contactgroup_assign',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# service templates
	if ($debug) {
		print "\nservice templates\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading service templates");
	$type = 'service_template';
	%where = ('type' => $type,'parent' => 'no-parent');
	%objects = fetch_list_hash_array('','stage_other',\%where);
	my %parents;
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		if ($debug) {
			print "\n----- $objects{$key}[0]\n";
		}

		my %t = fetch_one('','time_periods','name',$data{'check_period'});
		my %n = fetch_one('','time_periods','name',$data{'notification_period'});
		my %c = fetch_one('','commands','name',$data{'check_command'});
		my %e = fetch_one('','commands','name',$data{'event_handler'});
		delete $data{'check_period'};
		delete $data{'notification_period'};
		delete $data{'check_command'};
		delete $data{'event_handler'};
		
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /contact_groups/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}	
		$data .= $xml_end;
		my @values = ('',$objects{$key}[0],$objects{$key}[2],$t{'timeperiod_id'},$n{'timeperiod_id'},$c{'command_id'},$c{'command_id'},$data,$objects{$key}[4]);
		my $id = insert_obj_id('','service_templates',\@values,'servicetemplate_id');
		$parents{$objects{$key}[0]} = $id;
		my @members = split(/,/, $data{'contact_groups'});
		foreach my $mem (@members) {
			$mem =~ s/^\s+|\s+$//;
			my %c = fetch_one('','contactgroups','name',$mem);
			my @vals = ($c{'contactgroup_id'},'service_templates',$id);
			my $result = insert_obj('','contactgroup_assign',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	my $i = 0;
	foreach my $keys (keys %objects) { $i++ }
	my $runaway = $i + 1000;
	until ($i == 0) {
		foreach my $key (keys %objects) {
			if ($runaway == 0) {
				$parents{$objects{$key}[2]} = 0;
			}
			if ($parents{$objects{$key}[2]}) {
				my %data = parse_xml($objects{$key}[3]);
		
				my %t = fetch_one('','time_periods','name',$data{'check_period'});
				my %n = fetch_one('','time_periods','name',$data{'notification_period'});
				my %c = fetch_one('','commands','name',$data{'check_command'});
				my %e = fetch_one('','commands','name',$data{'event_handler'});
				delete $data{'check_period'};
				delete $data{'notification_period'};
				delete $data{'check_command'};
				delete $data{'event_handler'};


				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /contact_groups/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;
				my @values = ('',$objects{$key}[0],$parents{$objects{$key}[2]},$t{'timeperiod_id'},$n{'timeperiod_id'},$c{'command_id'},$e{'command_id'},$data,$objects{$key}[4]);
				my $id = insert_obj_id('','service_templates',\@values,'servicetemplate_id');
				$parents{$objects{$key}[0]} = $id;
				my @members = split(/,/, $data{'contact_groups'});
				foreach my $mem (@members) {
					$mem =~ s/^\s+|\s+$//;
					my %c = fetch_one('','contactgroups','name',$mem);
					my @vals = ($c{'contactgroup_id'},'service_templates',$id);
					my $result = insert_obj('','contactgroup_assign',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				$i--;
				$runaway--;
				delete $objects{$key};
			}
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# hostextinfo templates
	if ($debug) {
		print "\nhostextinfo templates\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading hostextinfo templates");
	$type = 'hostextinfo_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	$index = 1;
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @values = ('',$objects{$key}[0],$objects{$key}[3],'',$objects{$key}[4]);
		my $result = insert_obj('','extended_host_info_templates',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# serviceextinfo templates
	if ($debug) {
		print "\nserviceextinfo templates\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading serviceextinfo templates");
	$type = 'serviceextinfo_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my @values = ('',$objects{$key}[0],$objects{$key}[3],'',$objects{$key}[4]);
		my $result = insert_obj('','extended_service_info_templates',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# hostgroupescalation templates
	if ($debug) {
		print "\nhostgroupescalation templates\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading hostgroupescalation templates");
	$type = 'hostgroupescalation_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgids;
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host|name|type|HASH|comment/i) { next } 
			if ($d =~ /contact_groups/) { 
				my @cgs = split(/,/, $data{'contact_groups'});
				foreach my $c (@cgs) {
					$c =~ s/^\s+|\s+$//;
					my %cg = fetch_one('','contactgroups','name',$c);
					push @cgids, $cg{'contactgroup_id'};
				}
			} else {
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
		}	
		$data .= $xml_end;
		my @values = ('',$objects{$key}[0],'hostgroup',$data,$objects{$key}[4]);
		my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		foreach my $cgid (@cgids) {
			my @vals = ($cgid,'hostgroup_escalation_templates',$id);
			my $result = insert_obj('','contactgroup_assign',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# hostescalation templates
	if ($debug) {
		print "\nhostescalation templates\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading hostescalation templates");
	$type = 'hostescalation_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgids = ();
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host|name|type|HASH|comment/i) { next } 
			if ($d =~ /contact_groups/) { 
				my @cgs = split(/,/, $data{'contact_groups'});
				foreach my $c (@cgs) {
					$c =~ s/^\s+|\s+$//;
					my %cg = fetch_one('','contactgroups','name',$c);
					push @cgids, $cg{'contactgroup_id'};
				}
			} else {
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
		}	
		$data .= $xml_end;
		my @values = ('',$objects{$key}[0],'host',$data,$objects{$key}[4]);
		my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		foreach my $cgid (@cgids) {
			my @vals = ($cgid,'host_escalation_templates',$id);
			my $result = insert_obj('','contactgroup_assign',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	

# serviceescalation templates
	if ($debug) {
		print "\nserviceescalation templates\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading serviceescalation templates");
	$type = 'serviceescalation_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgids;
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host|name|type|HASH|comment/i) { next } 
			if ($d =~ /contact_groups/) { 
				my @cgs = split(/,/, $data{'contact_groups'});
				foreach my $c (@cgs) {
					$c =~ s/^\s+|\s+$//;
					my %cg = fetch_one('','contactgroups','name',$c);
					push @cgids, $cg{'contactgroup_id'};
				}
			} else {
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}
		}	
		$data .= $xml_end;
		my @values = ('',$objects{$key}[0],'service',$data,$objects{$key}[4]);
		my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		foreach my $cgid (@cgids) {
			my @vals = ($cgid,'service_escalation_templates',$id);
			my $result = insert_obj('','contactgroup_assign',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# hosts
	if ($debug) {
		print "\nhosts\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading hosts");
	my %host_parents = ();
	$type = 'host';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	$index = 1;
	foreach my $key (keys %objects) {
		my $status = 1;
		my %host_overrides = ();
		$host_overrides{'data'} = $xml_begin;
		my %data = parse_xml($objects{$key}[3]);
		delete $data{'command_line'};
		my $props = 0;
		my @contactgroups = ();
		my %time_period = ();
		my %check_command = ();
		my %event_handler = ();
		if ($data{'contact_groups'}) {
			@contactgroups = split(/,/, $data{'contact_groups'});
			delete $data{'contact_groups'};
		}
		if ($data{'notification_period'}) { 
			%time_period = fetch_one('','time_periods','name',$data{'notification_period'});
			delete $data{'notification_period'};
			$props++;
		}
		if ($data{'check_command'}) { 
			%check_command = fetch_one('','commands','name',$data{'check_command'});
			delete $data{'check_command'};
			$props++;
		}
		if ($data{'event_handler'}) { 
			%event_handler = fetch_one('','commands','name',$data{'event_handler'});
			delete $data{'event_handler'};
			$props++;
		}
		if (exists($data{'checks_enabled'})) { 
			if ($data{'checks_enabled'} eq '-zero-') { 
				$host_overrides{'data'} .= " <prop name=\"checks_enabled\"><![CDATA[$data{'checks_enabled'}]]>\n";				
				$host_overrides{'data'} .= " </prop>\n";
			}
			$host_overrides{'enabled'} = 1;
			delete $data{'checks_enabled'};
		} 
		foreach (keys %data) { $props++ }
		if ($data{'address'}) { $props-- }
		if ($data{'alias'}) { $props-- }
		if ($data{'parents'}) { $props-- }
		if ($data{'file_name'}) { $props-- }
		if ($data{'use'}) { $props-- }
		if (!$data{'use'}) {
			update_status('','error', "Host $objects{$key}[0] does not use a template");
			my %w = ();
			my %templates = fetch_list_hash_array('','host_templates',\%w);
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[5]);
				my $gotmatch = 0;
				if ($time_period{'timeperiod_id'} eq $templates{$t}[2]) { $gotmatch++ }
				if ($check_command{'command_id'} eq $templates{$t}[3]) { $gotmatch++ }
				if ($event_handler{'command_id'} eq $templates{$t}[4]) { $gotmatch++ }
				foreach my $d (keys %data) {
					if ($d =~ /file_name|address|alias|parents/) { next }
					if ($data{$d} eq $tdata{$d}) { $gotmatch++ }
				}	
				if ($gotmatch >= $props) {
					$data{'use'} = $templates{$t}[0];
					update_status('','action', "Assigned host template $templates{$t}[1] to host $objects{$key}[0].");
					last;
				}
			}
			if (!$data{'use'}) {
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /file_name|address|alias|parents|contact_groups/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;
				my $name = "generic-host-$index";
				my $comment = "";
				my @values = ('',$name,$time_period{'timeperiod_id'},$check_command{'command_id'},$event_handler{'command_id'},$data,$comment);
				my $result = insert_obj('','host_templates',\@values);
				if ($result =~ /Error/) { update_status('','error',$result) }
				$data{'use'} = $name;
				update_status('','action', "Added host template $name and assigned it to host $objects{$key}[0].");
				$index++;
			}
		} elsif ($props > 0) {
			update_status('','note', "Host $objects{$key}[0] has properties suitable for a template.");
			my %w = ();
			my %templates = fetch_list_hash_array('','host_templates',\%w);
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[5]);
				my $gotmatch = 0;
				if ($time_period{'timeperiod_id'} eq $templates{$t}[2]) { $gotmatch++ }
				if ($check_command{'command_id'} eq $templates{$t}[3]) { $gotmatch++ }
				if ($event_handler{'command_id'} eq $templates{$t}[4]) { $gotmatch++ }
				foreach my $d (keys %data) {
					if ($d =~ /command_line|use|error|file_name|address|alias|parents|contact_groups/) { next }
					if ($data{$d} eq $tdata{$d}) { $gotmatch++ }

				}	
				$data{'new'} = 0;
				if ($gotmatch >= $props) {
					$data{'use'} = $templates{$t}[1];
					update_status('','action', "Assigned host template $templates{$t}[1] to host $objects{$key}[0].");
					last;
				} else {
					$data{'new'} = 1;
				}
			}
			
			if ($data{'new'}) {
				delete $data{'new'};
				my %template = fetch_one('','host_templates','name',$data{'use'});
				foreach my $t (keys %template) {
					if ($t =~ /HASH|hosttemplate_id|comment|check_command|event_handler|name|notification_period/) { next }
					unless ($data{$t}) { $data{$t} = $template{$t}	}
				}	
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /command_line|use|error|file_name|address|alias|parents|contact_groups/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;
				my $name = "generic-host-$index";
				my $comment = undef;
				my @values = ('',$name,$time_period{'timeperiod_id'},$check_command{'command_id'},$event_handler{'command_id'},$data,$comment);
				my $result = insert_obj('','host_templates',\@values);
				if ($result =~ /Error/) { update_status('','error',$result) }
				$data{'use'} = $name;
				update_status('','action', "Added host template $name and assigned it to host $objects{$key}[0].");
				$index++;
			}
		}
		my %t = fetch_one('','host_templates','name',$data{'use'});
		my @values = ('',$objects{$key}[0],$data{'alias'},$data{'address'},'n/a',$t{'hosttemplate_id'},'','','','','',$status,$objects{$key}[4]);
		my $id = insert_obj_id('','hosts',\@values,'host_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		if ($host_overrides{'enabled'}) {
			delete $host_overrides{'enabled'};
			$host_overrides{'data'} .= $xml_end;
			@values = ($id,'','','','1',$host_overrides{'data'});
			insert_obj('','host_overrides',\@values);
		}
		$host_parents{$id} = $data{'parents'};
		foreach my $cg (@contactgroups) {
			$cg =~ s/^\s+|\s+$//;
			my %c = fetch_one('','contactgroups','name',$cg);
			my @vals = ($id,'hosts',$c{'contactgroup_id'});
			my $result = insert_obj('','contactgroup_assign',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
		my %f = fetch_one('','files','name',$data{'file_name'});
		my @vals = ($f{'file_id'},$id);
		my $result = insert_obj('','file_host',\@vals);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }


# host parent
	update_status('','info',"Setting host parent relationships");
	foreach my $host (keys %host_parents) {
		my @parents = split(/,/, $host_parents{$host});
		foreach my $p (@parents) {
			$p =~ s/^\s+|\s+$//;
			my %p = fetch_one('','hosts','name',$p);
			my @vals = ($host,$p{'host_id'});
			my $result = insert_obj('','host_parent',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	
# host dependency
	if ($debug) {
		print "\nhost dependency\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading host dependencies");
	$type = 'hostdependency';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @hosts = split(/,/, $data{'dependent_host_name'});
		my @parents = split(/,/, $data{'host_name'});
		my %parent_host = ();
		foreach my $host (@hosts) {
			$host =~ s/^\s+|\s+$//;
			my %h = fetch_one('','hosts','name',$host);
			foreach my $parent (@parents) {
				my %p = fetch_one('','hosts','name',$parent);
				$parent_host{$h{'host_id'}} = $p{'host_id'};
			}
		}		
		my $data = $xml_begin;
		foreach my $d (keys %data) {
			if ($d =~ /host_name/) { next }
			$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
			$data .= " </prop>\n";
		}	
		$data .= $xml_end;
		foreach my $h (keys %parent_host) {
			my @values = ($h,$parent_host{$h},$data,$objects{$key}[4]);
			my $result = insert_obj('','host_dependencies',\@values);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# hostextinfo
	update_status('','info',"Loading hostextinfo");
	$type = 'hostextinfo';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	$index = 1;
	foreach my $key (keys %objects) {
		my @hosts = ();
		my $hoststr = undef;
		my %data = parse_xml($objects{$key}[3]);
		if ($objects{$key}[2] eq 'hostextinfo') {
			@hosts = split(/,/, $data{'host_name'});
		} else {
			push @hosts, $objects{$key}[0];
		}
		if ($data{'notes_url'}) { 
			if ($hosts[0] && $hosts[0] ne '?') { $data{'notes_url'} =~ s/$hosts[0]/\$HOSTNAME\$/g }
			$data{'notes_url'} =~ s/\s/+/g;
		}
		if (!$data{'use'}) {
			my $props = 0;
			foreach (keys %data) { $props++ }
			if ($data{'host_name'}) { $props -= 1 }
			if ($data{'notes_url'}) { $props -= 1 }
			if ($data{'2d_coords'}) { $props -= 1 }
			if ($data{'3d_coords'}) { $props -= 1 }
			my %w = ();
			my %templates = fetch_list_hash_array('','extended_host_info_templates',\%w);
			my %gotmatch = ();
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[2]);
				foreach my $d (keys %data) {
					if ($d =~ /d_coords|host_name/) { next }
					if ($data{$d} eq $tdata{$d}) { $gotmatch{$templates{$t}[0]} += 1 }
				}
			}	
			my $match = undef;
			my $best = 0;
			foreach my $key (keys %gotmatch) {
				if ($gotmatch{$key} > $best) { $match = $key; $best = $gotmatch{$key} }
			}
			if ($gotmatch{$match} >= $props) { $data{'use'} = $match }
			if (!$data{'use'}) {
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /host_name/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;			
				my $name = undef;
				if ($data{'icon_image_alt'}) {
					$name = lc("$data{'icon_image_alt'}-$index");
					$name =~ s/_|\s/-/g;
				} elsif ($data{'icon_image'}) {
					$name = lc("$data{'icon_image'}-$index");
					$name =~ s/_|\s|./-/g;
				} elsif ($data{'vrml_image'}) {
					$name = lc("$data{'vrml_image'}-$index");
					$name =~ s/_|\s|./-/g;
				} elsif ($data{'statusmap_image'}) {
					$name = lc("$data{'statusmap_image'}-$index");
					$name =~ s/_|\s|./-/g;
				} else {
					$name = "generic-hostextinfo-$index";
				}
				my $comment = undef;
				my @values = ('',$name,$data,'',$comment);
				my $id = insert_obj_id('','extended_host_info_templates',\@values,'hostextinfo_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				$data{'use'} = $id;
				$index++;
			}
		} else {
			my %t = fetch_one('','extended_host_info_templates','name',$data{'use'});
			$data{'use'} = $t{'hostextinfo_id'};
			if ($data{'2dcoords'} == $t{'2dcoords'}) { delete $data{'2dcoords'} }
			if ($data{'3dcoords'} == $t{'3dcoords'}) { delete $data{'3dcoords'} }
		}
		foreach my $host (@hosts) {
			my %w = ('name' => $host);
			my %u = ('hostextinfo_id' => $data{'use'});
			my $result = update_obj_where('','hosts',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
			if ($data{'2dcoords'} || $data{'3dcoords'}) {
				my %id = fetch_one('','hosts','name',$host);
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /coords/) {
						$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my @vals = ($id{'host_id'},$data);
				my $result = insert_obj('','extended_info_coords',\@vals);
				if ($result =~ /Error/) { update_status('','error',$result) }
			}
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }


# parent directive
	update_status('','info',"Moving parent directive from templates to hosts");
	$type = 'parent_directive';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @parents = split(/,/, $data{'parents'});	
		my %t = fetch_one('','host_templates','name',$objects{$key}[0]);
		my %w = ('hosttemplate_id' => $t{'hosttemplate_id'});
		my @hosts = fetch_list_where('','hosts','host_id',\%w);
		foreach my $h (@hosts) {
			$h =~ s/^\s+|\s+$//;
			foreach my $p (@parents) {
				$p =~ s/^\s+|\s+$//;
				my %p = fetch_one('','hosts','name',$p);
				my @vals = ($h,$p{'host_id'});
				my $result = insert_obj('','host_parent',\@vals);
				if ($result =~ /Error/) { update_status('','error',$result) }
			}
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# redundant templates
	update_status('','info',"Removing redundant templates");
	$type = 'redundant_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %tnew = fetch_one('','host_templates','name',$objects{$key}[2]);
		my %told = fetch_one('','host_templates','name',$objects{$key}[0]);
		my %w = ('hosttemplate_id' => $told{'hosttemplate_id'});
		my %u = ('hosttemplate_id' => $tnew{'hosttemplate_id'});
		my $result = update_obj_where('','hosts',\%u,\%w);
		if ($result =~ /Error/) { update_status('','error',$result) }
		$result = delete_all('','host_templates','name',$objects{$key}[0]);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# hostgroup templates
	update_status('','info',"Removing hostgroup templates");
	$type = 'hostgroup_templates';
	my %hostgroup_template = ();
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		$hostgroup_template{$objects{$key}[0]} = $data{'contact_groups'};
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# hostgroups
	if ($debug) {
		print "\nhostgroups\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading hostgroups");
	$type = 'hostgroup-host';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @contactgroups = split(/,/, $data{'contact_groups'});
		if ($data{'use'}) {
			my @cguse = split(/,/, $hostgroup_template{$data{'use'}});
			push (@contactgroups,@cguse);
		}
		my @members = split(/,/, $data{'members'});
		my @vals = ('',$objects{$key}[0],$objects{$key}[2],'','','','1',$objects{$key}[4]);
		my $id = insert_obj_id('','hostgroups',\@vals,'hostgroup_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		foreach my $cg (@contactgroups) {
			$cg =~ s/^\s+|\s+$//;
			my %c = fetch_one('','contactgroups','name',$cg);
			my @vals = ($c{'contactgroup_id'},'hostgroups',$id);
			my $result = insert_obj('','contactgroup_assign',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
		foreach my $mem (@members) {
			$mem =~ s/^\s+|\s+$//;
			my %h = fetch_one('','hosts','name',$mem);
			my @vals = ($id,$h{'host_id'});
			my $result = insert_obj('','hostgroup_host',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result)	}
		}	
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# service names
	if ($debug) {
		print "\nservice names\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Creating service names");
	$type = 'service';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	my %sn_use = ();
	my %sn_cmd = ();
	my %sn_cmdline = ();
	my %sn_id = ();
	my %sn_dep = ();
	foreach my $key (sort keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my %use = fetch_one('','service_templates','name',$data{'use'});
		my %cmd = fetch_one('','commands','name',$data{'check_command'});
		$data{'use'} = $use{'servicetemplate_id'};
		$sn_cmd{$objects{$key}[0]} = $cmd{'command_id'};
		$sn_cmdline{$objects{$key}[0]} = $data{'command_line'};
		my $props = 0;
		delete $data{'host_name'};
		delete $data{'file_name'};
		foreach (keys %data) { $props++ }
		if ($data{'use'}) { $props-- }
		if (!$data{'use'}) {
			my @contactgroups;
			my %w = ();
			my %templates = fetch_list_hash_array('','service_templates',\%w);
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[6]);
				my $gotmatch = 0;
				my $bonus = undef;
				my %t = fetch_one('','time_periods','name',$data{'check_period'});
				my %n = fetch_one('','time_periods','name',$data{'notification_period'});
				my %c = fetch_one('','commands','name',$data{'check_command'});
				$data{'check_period'} = $t{'timeperiod_id'};
				$data{'notification_period'} = $n{'timeperiod_id'};
				$data{'check_command'} = $c{'command_id'};
				foreach my $d (keys %data) {
					if ($d =~ /command_line/) { 
						if ($data{$d} eq $tdata{$d}) { $bonus = 1 }
					} elsif ($data{'check_period'} eq $templates{$t}[3]) {
						$gotmatch++;
					} elsif ($data{'notification_period'} eq $templates{$t}[4]) {
						$gotmatch++;
					} elsif ($data{'check_command'} eq $templates{$t}[5]) {
						$gotmatch++;
					} else {
						if ($data{$d} eq $tdata{$d}) { $gotmatch++ }
					}
				}	
				my %w = ('type' => 'service_templates','object' => $templates{$t}[0]);
				my @tcg = fetch_list_where('','contactgroup-assign','contactgroup_id',\%w);
				my @cgs = split(/,/, $data{'contact_groups'});
				foreach my $cg (@cgs) {
					$props++;
					$cg =~ s/^\s+|\s+$//;
					my %cg = fetch_one('','contactgroups','name',$cg);
					push @contactgroups, $cg{'contactgroup_id'};
					foreach (@tcg) { if ($_ eq $cg{'contactgroup_id'}) { $gotmatch++ } }
				}
				if ($gotmatch == $props) {
					if ($bonus) {
						$data{'use'} = $templates{$t}[0];
					} elsif ($tdata{'parent_id'}) {
						$data{'parent_id'} = $tdata{'parent_id'};
					}
				}
				if ($data{'use'}) { last }
			}
			if (!$data{'use'}) {
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /use|contact_groups|^check_period$|check_command|period/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;
				my @values = ('',$objects{$key}[0],$data{'parent_id'},$data{'check_period'},$data{'notification_period'},$data{'check_command'},$data,$objects{$key}[4]);
				my $id = insert_obj_id('','service_templates',\@values,'servicetemplate_id');
				foreach my $c (@contactgroups) {
					my @vals = ($c,'service_templates',$id);
					my $result = insert_obj('','contactgroup_assign',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				$data{'use'} = $id;
				update_status('','error', "Service $objects{$key}[0] does not use a template");
				update_status('','action', "Added service template $objects{$key}[0]. Advise you check properties.");
			}
		} elsif ($props > 2) {
			my @contactgroups;
			my %t = fetch_one('','time_periods','name',$data{'check_period'});
			my %n = fetch_one('','time_periods','name',$data{'notification_period'});
			my %c = fetch_one('','commands','name',$data{'check_command'});
			my %e = fetch_one('','commands','name',$data{'event_handler'});

			$data{'check_period'} = $t{'timeperiod_id'};
			$data{'notification_period'} = $n{'timeperiod_id'};
			$data{'check_command'} = $c{'command_id'};
			$data{'event_handler'} = $e{'command_id'};

			my @cgs = split(/,/, $data{'contact_groups'});
			foreach my $cg (@cgs) {
				$cg =~ s/^\s+|\s+$//;
				my %cg = fetch_one('','contactgroups','name',$cg);
				push @contactgroups, $cg{'contactgroup_id'};
			}
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /use|contact_groups|^check_period$|check_command|period/) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $template_name = "$objects{$key}[0]-$index";
			$index++;
			my @values = ('',$template_name,$data{'use'},$data{'check_period'},$data{'notification_period'},$data{'check_command'},$data{'event_handler'},$data,$objects{$key}[4]);
			my $id = insert_obj_id('','service_templates',\@values,'servicetemplate_id');
			foreach my $c (@contactgroups) {
				my @vals = ($c,'service_templates',$id);
				my $result = insert_obj('','contactgroup_assign',\@vals);
				if ($result =~ /Error/) { update_status('','error',$result) }
			}
			$data{'use'} = $id;
			update_status('','note', "Service $objects{$key}[0] has properties suitable for a template.");
			update_status('','action', "Added service template $template_name.");

		} 
		$sn_use{$objects{$key}[0]} = $data{'use'};
		my %sname = fetch_one('','service_names','name',$objects{$key}[0]);
		my $id = $sname{'servicename_id'};
		if (!$id) {
			my @values = ('',$objects{$key}[0],$objects{$key}[0],$sn_use{$objects{$key}[0]},$sn_cmd{$objects{$key}[0]},$sn_cmdline{$objects{$key}[0]},'','','');
			$id = insert_obj_id('','service_names',\@values,'servicename_id');
			if ($id =~ /Error/) { update_status('','error',$id) }
		} else {
			my %w = ('servicename_id' => $id);
			my %u = ('template' => $sn_use{$objects{$key}[0]},'check_command' =>  $sn_cmd{$objects{$key}[0]},'command_line' => $sn_cmdline{$objects{$key}[0]});
			my $result = update_obj_where('','service_names',\%u,\%w);
		}
		$sn_id{$objects{$key}[0]} = $id;
	}

	if ($debug) {
		print "\nname ===================\n"; 
		my $x = <STDIN>;
	}
# services
	if ($debug) {
		print "\nservices\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading services");
	my %service_groups = ();
	my $sqlstmt = "select * from stage_other where type = 'service'";
	my $sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while(my @values = $sth->fetchrow_array()) {
		my %data = parse_xml($values[3]);
		my %hosts = ();
		if ($data{'host_name'} eq '*') {
			my @hosts = fetch_list('','hosts','host_id');
			foreach (@hosts) { $hosts{$_} = 1;}
		} elsif ($data{'hostgroup_name'}) {
			my @hostgroups = split(/,/, $data{'hostgroup_name'});
			foreach my $hg (@hostgroups) {
				$hg =~ s/^\s+|\s+$//;
				my %hgid = fetch_one('','hostgroups','name',$hg);
				my %w = ('hostgroup_id' => $hgid{'hostgroup_id'});
				my @hosts = fetch_list_where('','hostgroup_host','host_id',\%w);
				foreach (@hosts) { $hosts{$_} = 1 }
			}
		} else {
			my @hostnames = split(/,/, $data{'host_name'});
			foreach my $hn (@hostnames) {
				$hn =~ s/^\s+|\s+$//;
				my %h = fetch_one('','hosts','name',$hn);
				$hosts{$h{'host_id'}} = 1
			}
		}
		foreach my $hostid (keys %hosts) {
			my @vals = ('',$hostid,$sn_id{$values[0]},$sn_use{$values[0]},'','','1',$sn_cmd{$values[0]},$data{'command_line'},$values[4]);
			my $id = insert_obj_id('','services',\@vals,'service_id');
			if ($id =~ /Error/) { update_status('','error',$id) }
			if ($data{'service_groups'}) {	$service_groups{"$hostid-$id"} = $data{'service_groups'}	}	
			my %f = fetch_one('','files','name',$data{'file_name'});
			@vals = ($f{'file_id'},$id);
			my $result = insert_obj('','file_service',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}		
	}
	$sth->finish;
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# servicegroups
	if ($debug) {
		print "\nservice groups\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading servicegroups");
	$type = 'servicegroups';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @members = split(/,/, $data{'members'});
		my @vals = ('',$objects{$key}[0],$objects{$key}[2]);
		my $id = insert_obj_id('','servicegroups',\@vals,'servicegroup_id');
		if ($id =~ /Error/) { update_status('','error',$id) }
		my @services = ();
		my @hosts = ();
		my $pairs = 0;
		my $i = 0;
		foreach my $m (@members) {
			if ($i == 1) {
				push @services, $m;	$i = 0;
			} else {
				push @hosts, $m; $i =1; $pairs++;
			}
		}
		for ($i = 0; $i <= $pairs; $i++) {
			my %h =  fetch_one('','hosts','name',$hosts[$i]);
			my %s =  fetch_one('','services','name',$services[$i]);
			my @vals = ($id,$h{'host_id'},$s{'service_id'});
			my $result = insert_obj('','servicegroup_service',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }		
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }
	foreach my $hs (keys %service_groups) {
		my @sh = split(/,/, $hs);
		my %sg = fetch_one('','servicegroups','name',$service_groups{$hs});
		my @vals = ($sg{'servicegroup_id'},$sh[0],$sh[1]);
		my $result = insert_obj('','servicegroup_service',\@vals);
		if ($result =~ /Error/) { update_status('','error',$result) }		
	}

# service dependency templates
	if ($debug) {
		print "\nservice dependency template\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading service dependency templates");
	$type = 'servicedependency_template';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		if ($data{'service_description'}) {
			my %s = fetch_one('','service_names','name',$data{'service_description'});
			if (!$s{'servicename_id'}) {							
				my @values = ('',$data{'service_description'},$data{'service_description'},'0','0','0','','','');


				my $id = insert_obj_id('','service_names',\@values,'servicename_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				$s{'servicename_id'} = $id;
				$sn_id{$data{'service_description'}} = $id;
				update_status('','error', "Service name $data{'service_description'} does not exist and is required for dependency template $objects{$key}[0].");
				update_status('','action', "Service name $data{'service_description'} added for dependency template $objects{$key}[0]. Advise you check properties.");
			}
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /notification|execution/) {
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}
			}	
			$data .= $xml_end;
			my @vals = ('',$objects{$key}[0],$s{'servicename_id'},$data,$objects{$key}[4]);
			my $id = insert_obj('','service_dependency_templates',\@vals);
			if ($id =~ /Error/) { update_status('','error',$id) }
		} else {
			my $message = ();
			foreach my $name (keys %data) {
				$message .= "$name: $data{$name}<br>";
			}
			update_status('','error',"Unable to create service dependency template: No service description.<br><br>$message");
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

# service dependency
	if ($debug) {
		print "\nservice dependency\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading service dependencies");
	$type = 'servicedependency';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my $props = 0;
		if ($data{'service_description'}) { $props++ }
		if ($data{'execution_failure_criteria'}) { $props++ }
		if ($data{'notification_failure_criteria'}) { $props++ }
		if ($data{'use'}) {
			my %t = fetch_one('','service_dependency_templates','name',$data{'use'});
			$data{'use'} = $t{'id'};
		}
		if (!$data{'use'}) {
			my %w = ();
			my %templates = fetch_list_hash_array('','service_dependency_templates',\%w);
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[3]);
				my $gotmatch = 0;
				my %s = fetch_one('','service_names','servicename_id',$templates{$t}[2]);
				if ($data{'service_description'} eq $s{'name'}) { $gotmatch++ }
				if ($data{'execution_failure_criteria'} eq $tdata{'execution_failure_criteria'}) { $gotmatch++ }
				if ($data{'notification_failure_criteria'} eq $tdata{'notification_failure_criteria'}) { $gotmatch++ }
				if ($gotmatch == $props) {
					$data{'use'} = $templates{$t}[0];
					last;
				}
			}
		}

		if (!$data{'use'} && $props > 0) {
			my %s = fetch_one('','service_names','name',$data{'service_description'});
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /^service_description$|dependent|host_name/) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $data{'service_description'};
			$name =~ s/_/-/g;
			$name = "dep-$name-$index";
			%s = fetch_one('','service_names','name',$data{'service_description'});
			my @vals = ('',$name,$s{'servicename_id'},$data,$objects{$key}[4]);
			my $id = insert_obj_id('','service_dependency_templates',\@vals,'id');
			if ($id =~ /Error/) { update_status('','error',$id) }		
			$data{'use'} = $id;
		}

		my @dhn = split(/,/, $data{'dependent_host_name'});
		my @dependent_hosts = ();
		foreach my $dhn (@dhn) {
			$dhn =~ s/^\s+|\s+$//;
			my %d = fetch_one('','hosts','name',$dhn);
			push @dependent_hosts, $d{'host_id'};
		}
		my @phn = split(/,/, $data{'host_name'});
		my @parent_hosts = ();
		foreach my $phn (@phn) {
			$phn =~ s/^\s+|\s+$//;
			my %h = fetch_one('','hosts','name',$phn);
			push @parent_hosts, $h{'host_id'};
		}
		@dhn = split(/,/, $data{'dependent_hostgroup_name'});
		foreach my $dhn (@dhn) {
			$dhn =~ s/^\s+|\s+$//;
			my %d = fetch_one('','hostgroups','name',$dhn);
			my %w = ('hostgroup_id' => $d{'hostgroup_id'});
			my @hids = fetch_list_where('','hostgroup_host','host_id',\%w);
			push (@dependent_hosts, @hids);
		}
		@phn = split(/,/, $data{'hostgroup_name'});
		foreach my $phn (@phn) {
			$phn =~ s/^\s+|\s+$//;
			my %h = fetch_one('','hostgroups','name',$phn);
			my %w = ('hostgroup_id' => $h{'hostgroup_id'});
			my @hids = fetch_list_where('','hostgroup_host','host_id',\%w);
			push (@parent_hosts, @hids);
		}

		foreach my $dhn (@dependent_hosts) {
			foreach my $phn (@parent_hosts) {
				my $sqlstmt = "select service_id from services where host_id = '$dhn' and servicename_id in "
				. "(select servicename_id from service_names where name = '$objects{$key}[0]')";	
				my $sid = $dbh->selectrow_array($sqlstmt);
				if ($sid) {
					if ($phn eq $dhn) { $sn_dep{$objects{$key}[0]} = $data{'use'}};
					my @vals = ('',$sid,$dhn,$phn,$data{'use'},$objects{$key}[4]);
					my $result = insert_obj('','service_dependency',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }		
				} else {
					update_status('','error', "service $objects{$key}[0] does not exist for dependency");
				}
			}
		}
	}

	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }
# Update Service names
	if ($debug) {
		print "\nupdate service names\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Updating service names with dependency info");
	foreach my $dep (keys %sn_dep) {
		my %u = ('dependency' => $sn_dep{$dep});
		my %w = ('name' => $dep);
		my $result = update_obj_where('','service_names',\%u,\%w);

	}

# Service extended info
	if ($debug) {
		print "\nservice extinfo\n";
		my $x = <STDIN>;
	}
	update_status('','info',"Loading service extended info");
	my %sn_extinfo = ();
	$type = 'serviceextinfo';
	%where = ('type' => $type);
	my %service_desc = ();
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		if ($data{'notes_url'}) { 
			$data{'notes_url'} =~ s/$objects{$key}[2]/\$HOSTNAME\$/g;
			$data{'notes_url'} =~ s/$objects{$key}[0]/\$SERVICENAME\$/g;
			$data{'notes_url'} =~ s/\s/+/g;
		}
		if (!$data{'use'}) {
			my $props = 0;
			foreach (keys %data) { $props++ }
			if ($data{'host_name'}) { $props -= 1 }
			if ($data{'service_description'}) { $props -= 1 }
			my %w = ();
			my %templates = fetch_list_hash_array('','extended_service_info_templates',\%w);
			my %gotmatch = ();
			foreach my $t (keys %templates) {
				my %tdata = parse_xml($templates{$t}[2]);
				foreach my $d (keys %data) {
					if ($data{$d} eq $tdata{$d}) { $gotmatch{$templates{$t}[0]} += 1 }
				}	
			}
			my $match = undef;
			my $best = 0;
			foreach my $key (keys %gotmatch) {
				if ($gotmatch{$key} > $best) { $match = $key; $best = $gotmatch{$key} }
			}
			if ($gotmatch{$match} >= $props) { $data{'use'} = $match	}
			$sn_extinfo{$objects{$key}[0]} = $data{'use'};
			if (!$data{'use'}) {
				my $data = $xml_begin;
				foreach my $d (keys %data) {
					if ($d =~ /host_name/) { next }
					$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
					$data .= " </prop>\n";
				}	
				$data .= $xml_end;
				my $name = undef;
				if (!$service_desc{$objects{$key}[0]}) {
					$name = lc("$objects{$key}[0]");
					$name =~ s/_|\s/-/g;
					$service_desc{$objects{$key}[0]} = 1;
				} elsif ($data{'icon_image_alt'}) {
					$name = lc("$data{'icon_image_alt'}-$index");
					$name =~ s/_|\s/-/g;
				} elsif ($data{'icon_image'}) {
					$name = lc("$data{'icon_image'}-$index");
					$name =~ s/_|\s|./-/g;
				} else {
					$name = "generic-serviceextinfo-$index";
				}

				my $comment = undef;
				my @values = ('',$name,$data,'',$comment);
				my $id = insert_obj_id('','extended_service_info_templates',\@values,'serviceextinfo_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				$data{'use'} = $name;
				$sn_extinfo{$objects{$key}[0]} = $id;
			}
		} else {
			my %t = fetch_one('','extended_service_info_templates','name',$data{'use'});
			$sn_extinfo{$objects{$key}[0]} = $t{'serviceextinfo_id'};
		}
		if ($data{'notes_url'}) {
			my %t = fetch_one('','extended_service_info_templates','name',$data{'use'});
			delete $t{'script'};
			delete $t{'name'};
			delete $t{'comment'};
			delete $t{'serviceextinfo_id'};
			$t{'notes_url'} = $data{'notes_url'};
			my $data = $xml_begin;
			foreach my $d (keys %t) {
				$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my %w = ('name' => $data{'use'});
			my %u = ('data' => $data);
			my $result = update_obj_where('','extended_service_info_templates',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

	foreach my $service (keys %sn_extinfo) {
		my %w = ('name' => $service);
		my %u = ('extinfo' => $sn_extinfo{$service});
		my $result = update_obj_where('','service_names',\%u,\%w);
		if ($result =~ /Error/) { update_status('','error',$result) }
		my %snid = fetch_one('','service_names','name',$service);
		%w = ('servicename_id' => $snid{'servicename_id'});
		%u = ('serviceextinfo_id' => $sn_extinfo{$service});
		$result = update_obj_where('','services',\%u,\%w);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}

########################################################################################################
# hostgroup escalations
############################################################################
# host escalation - hostgroup
	update_status('','info',"Loading hostgroup escalations");
	if ($debug) {
		print "\nLoading hostgroup escalations\n";
		my $x = <STDIN>;
	}

	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;
	$index = 0;
	$type = 'hostgroupescalation';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('','contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objs = split(/,/, $data{'hostgroup_name'});
		my @objids;
		foreach my $obj (@objs) {
			$obj =~ s/^\s+|\s+$//;
			my %o = fetch_one('','hostgroups','name',$obj);
			push @objids, $o{'hostgroup_id'};
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'hostgroup',$data,$objects{$key}[4]);
			my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
			if ($id =~ /Error/) { update_status('','error',$id) }
			$data{'use'} = $id;
		} else {	
			my %w = ('type' => 'hostgroup','name' => $data{'use'});
			my %t = fetch_one_where('','escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('','contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my @values = ('',"$t{'name'}-$index",'hostgroup',$data,$objects{$key}[4]);
				my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'hostgroup_escalation_templates',$id);
					my $result = insert_obj('','contactgroup_assign',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('','escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		my $result = insert_obj('','stage_escalations',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	my @trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hostgroups = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('','hostgroups','hostgroup_id',$hostgroups[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'hostgroup');
		my $tree_id = insert_obj_id('','escalation_trees',\@vals,'tree_id');
		if ($tree_id =~ /Error/) { update_status('','error', $tree_id) }
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			my $result = insert_obj('','escalation_tree_template',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				my $result = insert_obj('','tree_template_contactgroup',\@cgvals);
				if ($result =~ /Error/) { update_status('','error',$result) }
			}
		}
		foreach my $hg (@hostgroups) {
			$hg =~ s/^\s+|\s+$//;
			my %w = ('hostgroup_id' => $hg);
			my %u = ('hostgroup_escalation_id' => $tree_id);
			my $result = update_obj_where('','hostgroups',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }

########################################################################################################
# host escalations
############################################################################
# host escalation - host
	update_status('','info',"Loading host escalations");
	update_status('','info',"--hostgroups");
	if ($debug) {
		print "\nLoading host escalations\n";
		print "\n--hostgroups\n";
		my $x = <STDIN>;
	}
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;
	$type = 'hostescalation';
	%where = ('type' => $type, 'parent' => 'hostgroup');
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('','contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objs = split(/,/, $data{'hostgroup_name'});
		my @objids = ();
		foreach my $obj (@objs) {
			$obj =~ s/^\s+|\s+$//;
			my %o = fetch_one('','hostgroups','name',$obj);
			push @objids, $o{'hostgroup_id'};
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'host',$data,$objects{$key}[4]);
			my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
			if ($id =~ /Error/) { update_status('','error',$id) }
			$data{'use'} = $id;
		} else {	
			my %w = ('type' => 'host','name' => $data{'use'});
			my %t = fetch_one_where('','escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host/) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('','contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my @values = ('',"$t{'name'}-$index",'host',$data,$objects{$key}[4]);
				my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'host_escalation_templates',$id);
					my $result = insert_obj('','contactgroup_assign',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('','escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		my $result = insert_obj('','stage_escalations',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hostgroups = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('','hostgroups','hostgroup_id',$hostgroups[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'hostgroup');
		my $tree_id = insert_obj_id('','escalation_trees',\@vals,'tree_id');
		if ($tree_id =~ /Error/) { update_status('','error',$tree_id) }
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			my $result = insert_obj('','escalation_tree_template',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				my $result = insert_obj('','tree_template_contactgroup',\@cgvals);
				if ($result =~ /Error/) { update_status('','error',$result) }
			}
		}
		foreach my $hg (@hostgroups) {
			$hg =~ s/^\s+|\s+$//;
			my %w = ('hostgroup_id' => $hg);
			my %u = ('host_escalation_id' => $tree_id);
			my $result = update_obj_where('','hostgroups',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }


############################################################################
# host escalation - host
	update_status('','info',"--hosts");
	if ($debug) {
		print "\n --hosts\n";
		my $x = <STDIN>;
	}
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;
	$type = 'hostescalation';
	%where = ('type' => $type, 'parent' => 'host');
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('','contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objids = ();
		if ($data{'host_name'} eq '*') {
			@objids = fetch_list('','hosts','host_id');
		} else {
			my @objs = split(/,/, $data{'host_name'});
			foreach my $obj (@objs) {
				$obj =~ s/^\s+|\s+$//;
				my %o = fetch_one('','hosts','name',$obj);
				push @objids, $o{'host_id'};
			}
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'host',$data,$objects{$key}[4]);
			my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
			if ($id =~ /Error/) { update_status('','error',$id) }
			$data{'use'} = $id;
		} else {	
			my %w = ('type' => 'host','name' => $data{'use'});
			my %t = fetch_one_where('','escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('','contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my @values = ('',"$t{'name'}-$index",'host',$data,$objects{$key}[4]);
				my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'host_escalation_templates',$id);
					my $result = insert_obj('','contactgroup_assign',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('','escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		my $result = insert_obj('','stage_escalations',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hosts = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('','hosts','host_id',$hosts[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'host');
		my $tree_id = insert_obj_id('','escalation_trees',\@vals,'tree_id');
		if ($tree_id =~ /Error/) { update_status('','error', $tree_id) }
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			my $result = insert_obj('','escalation_tree_template',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				my $result = insert_obj('','tree_template_contactgroup',\@cgvals);
				if ($result =~ /Error/) { update_status('','error',$result) }
			}
		}
		foreach my $h (@hosts) {
			$h =~ s/^\s+|\s+$//;
			my %w = ('host_id' => $h);
			my %u = ('host_escalation_id' => $tree_id);
			my $result = update_obj_where('','hosts',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }


########################################################################################################
# service escalations
############################################################################
# service escalation - hostgroup
	update_status('','info',"Loading service escalations");
	update_status('','info',"--hostgroups");
	if ($debug) {
		print "\nLoading service escalations\n";
		print "\n--hostgroups\n";
		my $x = <STDIN>;
	}
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;
	$type = 'serviceescalation';
	%where = ('type' => $type, 'parent' => 'hostgroup');
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids = ();
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('','contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objs = split(/,/, $data{'hostgroup_name'});
		my @objids = ();
		foreach my $obj (@objs) {
			$obj =~ s/^\s+|\s+$//;
			my %o = fetch_one('','hostgroups','name',$obj);
			push @objids, $o{'hostgroup_id'};
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'service',$data,$objects{$key}[4]);
			my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
			if ($id =~ /Error/) { update_status('','error',$id) }
			$data{'use'} = $id;
		} else {	
			my %w = ('type' => 'service','name' => $data{'use'});
			my %t = fetch_one_where('','escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('','contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my @values = ('',"$t{'name'}-$index",'service',$data,$objects{$key}[4]);
				my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'service_escalation_templates',$id);
					my $result = insert_obj('','contactgroup_assign',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('','escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		my $result = insert_obj('','stage_escalations',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hostgroups = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('','hostgroups','hostgroup_id',$hostgroups[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'service');
		my $tree_id = insert_obj_id('','escalation_trees',\@vals,'tree_id');
		if ($tree_id =~ /Error/) { update_status('','error', $tree_id) }
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			my $result = insert_obj('','escalation_tree_template',\@vals);
			if ($result =~ /Error/) { update_status('','error',$result) }
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				my $result = insert_obj('','tree_template_contactgroup',\@cgvals);
				if ($result =~ /Error/) { update_status('','error',$result) }
			}
		}
		foreach my $hg (@hostgroups) {
			$hg =~ s/^\s+|\s+$//;
			my %w = ('hostgroup_id' => $hg);
			my %u = ('service_escalation_id' => $tree_id);
			my $result = update_obj_where('','hostgroups',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }


############################################################################
# service escalation - host
	update_status('','info',"--hosts");
	if ($debug) {
		print "\n--hosts\n";
		my $x = <STDIN>;
	}
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;
	$type = 'serviceescalation';
	%where = ('type' => $type, 'parent' => 'host');
	%objects = fetch_list_hash_array('','stage_other',\%where);
	foreach my $key (keys %objects) {
		my %data = parse_xml($objects{$key}[3]);
		my @cgs = split(/,/, $data{'contact_groups'});
		my @cgids;
		foreach my $c (@cgs) {
			$c =~ s/^\s+|\s+$//;
			my %cg = fetch_one('','contactgroups','name',$c);
			push @cgids, $cg{'contactgroup_id'};
		}
		@cgids = sort { $a <=> $b } @cgids;
		my $cgstr = undef;
		foreach (@cgids) { $cgstr .= "$_," }
		chop $cgstr;
		my @objids;
		if ($data{'host_name'} eq '*') {
			@objids = fetch_list('','hosts','host_id');
		} else {
			my @objs = split(/,/, $data{'host_name'});
			foreach my $obj (@objs) {
				$obj =~ s/^\s+|\s+$//;
				my %o = fetch_one('','hosts','name',$obj);
				push @objids, $o{'host_id'};
			}
		}
		@objids = sort { $a <=> $b } @objids;
		my $objstr = undef;
		foreach (@objids) { $objstr .= "$_," }
		chop $objstr;
		if (!$data{'use'}) {
			my $data = $xml_begin;
			foreach my $d (keys %data) {
				if ($d =~ /contact_groups|host|name|type|HASH|comment/i) { next }
				$data .= " <prop name=\"$d\"><![CDATA[$data{$d}]]>\n";				
				$data .= " </prop>\n";
			}	
			$data .= $xml_end;
			my $name = $objects{$key}[0];
			my @values = ('',"$name",'service',$data,$objects{$key}[4]);
			my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
			if ($id =~ /Error/) { update_status('','error',$id) }
			$data{'use'} = $id;
		} else {	
			my %w = ('type' => 'service','name' => $data{'use'});
			my %t = fetch_one_where('','escalation_templates',\%w);
			$data{'use'} = $t{'template_id'};
			if ($t{'first_notification'} eq $data{'first_notification'}) { delete $data{'first_notification'} }
			if ($t{'last_notification'} eq $data{'last_notification'}) { delete $data{'last_notification'} }
			if ($t{'notification_interval'} eq $data{'notification_interval'}) { delete $data{'notification_interval'} }
			if ($data{'last_notification'} or $data{'first_notification'} or $data{'notification_interval'}) {
				if ($data{'first_notification'}) {$t{'first_notification'} = $data{'first_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				if ($data{'last_notification'}) {$t{'last_notification'} = $data{'last_notification'} }
				my $data = $xml_begin;
				foreach my $d (keys %t) {
					if ($d =~ /host|name|type|HASH|comment/i) { next } 
					if ($d =~ /contact_groups/) { 
						my @cgs = split(/,/, $t{'contact_groups'});
						foreach my $c (@cgs) {
							$c =~ s/^\s+|\s+$//;
							my %cg = fetch_one('','contactgroups','name',$c);
							push @cgids, $cg{'contactgroup_id'};
						}
					} else {
						$data .= " <prop name=\"$d\"><![CDATA[$t{$d}]]>\n";				
						$data .= " </prop>\n";
					}
				}	
				$data .= $xml_end;
				my @values = ('',"$t{'name'}-$index",'service',$data,$objects{$key}[4]);
				my $id = insert_obj_id('','escalation_templates',\@values,'template_id');
				if ($id =~ /Error/) { update_status('','error',$id) }
				foreach my $cgid (@cgids) {
					my @vals = ($cgid,'service_escalation_templates',$id);
					my $result = insert_obj('','contactgroup_assign',\@vals);
					if ($result =~ /Error/) { update_status('','error',$result) }
				}
				$index++;
				$data{'use'} = $id;
			}
		}
		my %w = ('template_id' => $data{'use'});
		my %t = fetch_one_where('','escalation_templates',\%w);
		my @values = ('',$objstr,$data{'use'},$cgstr,$t{'first_notification'},$t{'last_notification'},$t{'notification_interval'});
		my $result = insert_obj('','stage_escalations',\@values);
		if ($result =~ /Error/) { update_status('','error',$result) }
	}
	@trees = ();
	$sqlstmt = "select distinct objects from stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	$sth->execute;
	while (my @values = $sth->fetchrow_array()) { push @trees, $values[0] }
	$sth->finish;
	foreach my $obj (@trees) {
		my @hosts = split(/,/, $obj);
		$sqlstmt = "select * from stage_escalations where objects = '$obj'";
		$sth = $dbh->prepare ($sqlstmt);
		$sth->execute;
		my %n = fetch_one('','hosts','host_id',$hosts[0]);
		my $name = "esctree-$n{'name'}-$index";
		$name =~ s/_/-/g;
		my @vals = ('',$name,$obj,'service');
		my $tree_id = insert_obj_id('','escalation_trees',\@vals,'tree_id');
		if ($tree_id =~ /Error/) { 
			update_status('','error', $tree_id);
			if ($debug) {
				print "\n--escalation_trees $tree_id\n";
				my $x = <STDIN>;
			}
		}
		$index++;
		while (my @values = $sth->fetchrow_array()) {
			@vals = ($tree_id,$values[2]);
			my $result = insert_obj('','escalation_tree_template',\@vals);
			if ($result =~ /Error/) { 
				update_status('','error',"escalation_tree_template $result");
				if ($debug) {
					print "\n--escalation_tree_template $result\n";
					my $x = <STDIN>;
				}
			}
			my @contactgroups = split(/,/, $values[3]);
			foreach my $cg (@contactgroups) {
				my @cgvals = ($tree_id,$values[2],$cg);
				my $result = insert_obj('','tree_template_contactgroup',\@cgvals);
				if ($result =~ /Error/) { 
					update_status('','error',"tree_template_contactgroup $result");
					if ($debug) {
						print "\n--tree_template_contactgroup $result\n";
						my $x = <STDIN>;
					}
				}
			}
		}
		foreach my $h (@hosts) {
			$h =~ s/^\s+|\s+$//;
			my %w = ('host_id' => $h);
			my %u = ('service_escalation_id' => $tree_id);
			my $result = update_obj_where('','hosts',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
############################################################################
# service escalation - service
	update_status('','info',"--services");
	if ($debug) {
		print "\n--services\n";
		my $x = <STDIN>;
	}
	$sqlstmt = "truncate table stage_escalations";
	$sth = $dbh->prepare ($sqlstmt);
	eval{$sth->execute()};
	update_status('','error', "Error: $sqlstmt $@") if $@;
	$type = 'serviceescalation-service';
	%where = ('type' => $type);
	%objects = fetch_list_hash_array('','stage_other',\%where);
	my %hosts = ();
	foreach my $key (keys %objects) {
		my $service_tree = $objects{$key}[0];
		$service_tree =~ s/-\d+$//;
		my $service_name = $service_tree;
		$service_tree =~ s/_/-/g;
		my %esc_tree = fetch_one('','escalation_trees','name',$service_tree);

		my @hosts = ();
		my $tree_id = undef;
		my %data = parse_xml($objects{$key}[3]);
		if ($data{'host_name'} eq '*') {
			@hosts = fetch_list('','hosts','host_id');
		} else {
			my @objs = split(/,/, $data{'host_name'});
			foreach my $obj (@objs) {
				$obj =~ s/^\s+|\s+$//;
				my %o = fetch_one('','hosts','name',$obj);
				push @hosts, $o{'host_id'};
			}
		}
		$hosts{$objects{$key}[2]} = [ @hosts ];
		if ($esc_tree{'tree_id'}) {
			$tree_id = $esc_tree{'tree_id'};
		} else {
			my @vals = ('',$service_tree,"service escalation $service_tree",'service');
			$tree_id = insert_obj_id('','escalation_trees',\@vals,'tree_id');
			if ($tree_id =~ /Error/) { 
				update_status('','error', $tree_id);
				if ($debug) {
					print "\n--escalation_trees $tree_id\n";
					my $x = <STDIN>;
				}
			}
			my $tree_name = $objects{$key}[0];
			$tree_name =~ s/-\d+$//;
			my $sqlstmt = "select * from stage_other where name like '$tree_name%'";
			my $sth = $dbh->prepare($sqlstmt);
			$sth->execute;
			my $template = undef;			
			while(my @values = $sth->fetchrow_array()) { 
				my $got_new = 0;
				unless ($template eq $values[2]) {
					$template = $values[2];
					my %t = fetch_one('','escalation_templates','name',$template);
					my %data = parse_xml($objects{$key}[3]);
					my @cgs = split(/,/, $data{'contact_groups'});
					my @cgids = ();
					@vals = ($tree_id,$t{'template_id'});
					my $result = insert_obj('','escalation_tree_template',\@vals);
					if ($result =~ /Error/) { 
						update_status('','error',"escalation_tree_template $result");
						if ($debug) {
							print "\n--escalation_tree_template $result line 2982\n";
							my $x = <STDIN>;
						}
					}
					foreach my $c (@cgs) {
						$c =~ s/^\s+|\s+$//;
						my %cg = fetch_one('','contactgroups','name',$c);
						my @cgvals = ($tree_id,$t{'template_id'},$cg{'contactgroup_id'});
						my $result = insert_obj('','tree_template_contactgroup',\@cgvals);
						if ($result =~ /Error/) { 
							update_status('','error',"tree_template_contactgroup $result");
							if ($debug) {
								print "\n--tree_template_contactgroup $result\n";
								my $x = <STDIN>;
							}
						}
						push @cgids, $cg{'contactgroup_id'};
					}
					@cgids = sort { $a <=> $b } @cgids;
					my $cgstr = undef;
					foreach (@cgids) { $cgstr .= "$_," }
					chop $cgstr;
				}
			}
			$sth->finish;	
		}
		my %sn = fetch_one('','service_names','name',$service_name);
		foreach my $hid (@hosts) {
			my %w = ('host_id' => $hid,'servicename_id' => $sn{'servicename_id'});
			my %u = ('escalation_id' => $tree_id);
			my $result = update_obj_where('','services',\%u,\%w);
			if ($result =~ /Error/) { update_status('','error',$result) }
		}
	}
	$result = delete_one_where('','stage_other',\%where);
	if ($result =~ /Error/) { update_status('','error',$result) }
	
	$dt = datetime();
	update_status('','completed',"$dt");

}

stage_load();
process_load();


