#!/usr/bin/perl --
# MonArch - Groundwork Monitor Architect
# monarch_update.pl
#
###############################################################################
# Release 0.94c
# 16-May-2005
###############################################################################
#

use DBI;
use strict;
use File::Copy;

my ($dsn, $user,$passwd) = 0;
my $drop_tables = $ARGV[0];
my $database = "monarch";
my $databaseuser = "root";
my $databasepasswd = "";
my $cgibin = "/var/www/cgi-bin";
my $doc_root = "/var/www/html";
my $monarch_home = "/usr/local/groundwork/monarch";
my $web_group = "apache";
my $web_user = "apache";
my $nagios_dir = "/etc/nagios";
my $set_perms = 0;


sub validate_path() {
	if (-e shift) {
		return 1;
	} else {
		return 0;
	}
}

sub validate_user() {
	if (-e shift) {
		return 1;
	} else {
		return 2;
	}
}

sub validate_group() {
	if (-e shift) {
		return 1;
	} else {
		return 2;
	}
}


sub check_prereqs() {
	my @missing_reqs = ();
	my $errors = qx('./check_mods.pl');
	if ($errors) {
		push @missing_reqs, $errors;
		push @missing_reqs, "Missing perl module requirements. See README for list";
		push @missing_reqs, "To check installed modules do:";
		push @missing_reqs, "perl -MFile::Find=find -MFile::Spec::Functions -lwe  'find { wanted => sub { print canonpath $_ if /\.pm\z/ }, no_chdir => 1 }, \@INC'";
	}
	my $ver = qx(mysql -V);
	unless ($ver =~ /Distrib 4.1/i) {
		push @missing_reqs, "$ver";
		push @missing_reqs, "MySQL ver 4.1x required";
	}
	if (@missing_reqs) {
		print "\nPrerequisite check failed (see README.txt). Please correct the following:\n";
		foreach (@missing_reqs) {
			print "\n\t$_\n";
		}
		exit;
	}
}

sub table_update() {
	my $dsn = "DBI:mysql:$database:localhost";
	my $dbh = DBI->connect($dsn, $databaseuser, $databasepasswd, {'RaiseError' => 1});
	my $sqlstmt = "select type from access_list where object = 'nagios_configuration'";
	my $val = $dbh->selectrow_array($sqlstmt);
	unless ($val) {
		$sqlstmt = "select usergroup_id from user_groups where name = 'super_users'";
		my $gid = $dbh->selectrow_array($sqlstmt);

		my @control_list = ('nagios_configuration');
		foreach my $asset (@control_list) {
			my $sqlstmt = "insert into access_list values('$asset','control','$gid','full_control')";
			my $sth = $dbh->prepare($sqlstmt);
			unless ($sth->execute()) {
				print "\n\n $sqlstmt $@\n\n";
				my $x = <STDIN>;
			}
			$sth->finish;
		}
	}

	if ($dbh) {
	   $dbh and $dbh->disconnect();
	}
}

sub distribution() {
	umask 0002;
	if (! -e "$doc_root/monarch") {
		mkdir("$doc_root/monarch", 0770) || print "\n\n$doc_root/monarch $!";
	}
	if (! -e "$doc_root/monarch/download") {
		mkdir("$doc_root/monarch/download", 0770) || print "\n\n$doc_root/monarch/download $!";
	}
	if (! -e "$doc_root/monarch/images") {
		mkdir("$doc_root/monarch/images", 0770) || print "\n\n$doc_root/monarch/images $!";
	}
	if (! -e "$doc_root/monarch/doc") {
		mkdir("$doc_root/monarch/doc", 0770) || print "\n\n$doc_root/monarch/doc $!";
	}
	my $path = '/';
	my @folders = split(/\//, $monarch_home);
	foreach my $folder (@folders) {
		if (! -e "$path$folder") {
			mkdir("$path$folder", 0755) || print "\n\n$path$folder $!";
		}
		$path .= "$folder/";
	}
	if (! -e "$monarch_home/workspace") {
		mkdir("$monarch_home/workspace", 0770) || print "\n\n$monarch_home/monarch/workspace $!";
	}
	if (! -e "$monarch_home/upload") {
		mkdir("$monarch_home/upload", 0770) || print "\n\n$monarch_home/upload $!";
	}
	if (! -e "$monarch_home/backup") {
		mkdir("$monarch_home/backup", 0770) || print "\n\n$monarch_home/backup $!";
	}
	if (! -e "$monarch_home/bin") {
		mkdir("$monarch_home/bin", 0770) || print "\n\n$monarch_home/bin $!";
	}	
	if (! -e "$monarch_home/lib") {
		mkdir("$monarch_home/lib", 0770) || print "\n\n$monarch_home/lib $!";
	}	
	if (! -e "$monarch_home/conf") {
		mkdir("$monarch_home/conf", 0770) || print "\n\n$monarch_home/conf $!";
	}	
	opendir(DIR, './images') || print "\n\n./images $!";
	while (my $file = readdir(DIR)) {
		if ($file =~/^\./) { next }
		copy("./images/$file","$doc_root/monarch/images/$file") || print "\n\n./images/$file $doc_root/monarch/images/$file $!";
	}
	close(DIR);
	opendir(DIR, './doc') || print "\n\n./doc $!";
	while (my $file = readdir(DIR)) {
		if ($file =~/^\./) { next }
		copy("./doc/$file","$doc_root/monarch/doc/$file") || print "\n\n./doc/$file $doc_root/monarch/doc/$file $!";
	}
	close(DIR);
	open(FILE, "< ./monarch.cgi") || print "\n\n./monarch.cgi $!";
	my $out_to_file = undef;
	while (my $line = <FILE>) {
		if ($line =~ /STRINGTOREPLACE/) {
			$line =~ s/STRINGTOREPLACE/$monarch_home/;
		}
		$out_to_file .= $line;
	}
	close(FILE);
	open(FILE, "> $cgibin/monarch.cgi") || print "\n$cgibin/monarch.cgi $!";
	print FILE $out_to_file;
	close(FILE);

	open(FILE, "< ./monarch_load.pl") || print "\n\n./monarch_load.pl $!";
	my $out_to_file = undef;
	while (my $line = <FILE>) {
		if ($line =~ /STRINGTOREPLACE/) {
			$line =~ s/STRINGTOREPLACE/$monarch_home/;
		}
		$out_to_file .= $line;
	}
	close(FILE);
	open(FILE, "> $monarch_home/bin/monarch_load.pl") || print "\n$monarch_home/bin/monarch_load.pl $!";
	print FILE $out_to_file;
	close(FILE);
	
	my @files = ('monarch.js','nicetitle.js','monarch.css');
	foreach my $file (@files) {
		copy("./$file","$doc_root/monarch/$file") || print "\n\n./$file $doc_root/monarch/$file $!";
	}

	open(FILE, "< ./MonarchConf.pm") || print "\n\n./MonarchConf.pm $!";
	my $out_to_file = undef;
	while (my $line = <FILE>) {
		if ($line =~ /DATABASE/) {
			$line =~ s/DATABASE/$database/;
		} elsif ($line =~ /DBUSER/) {
			$line =~ s/DBUSER/$databaseuser/;
		} elsif ($line =~ /DBPASS/) {
			$line =~ s/DBPASS/$databasepasswd/;
		}
		$out_to_file .= $line;
	}
	close(FILE);
	open(FILE, "> $monarch_home/lib/MonarchConf.pm") || print "\n$monarch_home/lib/MonarchConf.pm $!";
	print FILE $out_to_file;
	close(FILE);

	@files = ('MonarchFile.pm','MonarchForms.pm','MonarchStorProc.pm','MonarchExternals.pm','MonarchDoc.pm');
	foreach my $file (@files) {
		copy("./$file","$monarch_home/lib/$file") || print "\n\nnot copied ./$file $monarch_home/lib/$file $!";
	}
	@files = ('nagios_reload');
	foreach my $file (@files) {
		copy("./$file","$monarch_home/bin/$file") || print "\n\nnot copied ./$file $monarch_home/bin/$file $!\n";
	}
	system("chown $web_user:$web_group $cgibin/monarch.cgi");
	system("chmod 750 $cgibin/monarch.cgi");
	system("chown -R $web_user:$web_group $doc_root/monarch");
	system("chown -R $web_user:$web_group $monarch_home");
	system("chown root:$web_group $monarch_home/bin/nagios_reload");
	system("chmod 4750 $monarch_home/bin/nagios_reload");
	system("chmod 750 $monarch_home/bin/monarch_load.pl");
	my $errors = 0;
	if ($set_perms) {
		my @files = ();
		my @dirs = ();
		my $resource_file = undef;
		if (-e "$nagios_dir/nagios.cfg") {
			system("chmod 660 $nagios_dir/nagios.cfg");
			open (FILE, "< $nagios_dir/nagios.cfg") || print "\n$nagios_dir/nagios.cfg $!";
			while (my $line = <FILE>) {
				if ($line =~ /^cfg_file=(\S+)$/) { push @files, $1 }
				if ($line =~ /^cfg_dir=(\S+)$/) { push @dirs, $1 }
				if ($line =~ /^resource_file=(\S+)$/) { $resource_file = $1 }
			}
			close(FILE);
			foreach my $dir (@dirs) {
				opendir(DIR, $dir) || print "\nerror: cannot open $dir to read $!";
				while (my $file = readdir(DIR)) {
					if ($file =~ /(\S+\.cfg$)/) {  push @files, $file }
				}
				close(DIR);
				system("chmod 770 $dir");
			}
		} else {
			print "\n\n\tError: $nagios_dir/nagios.cfg does not exist\n";
			$errors = 1;
		}
		if (-e "$nagios_dir/cgi.cfg") {
			system("chmod 660 $nagios_dir/cgi.cfg");
			open(FILE, "< $nagios_dir/cgi.cfg") or print "error: cannot open $nagios_dir/cgi.cfg to read $!";
			while (my $line = <FILE>) {
				if ($line =~ /^xedtemplate_config_file=(\S+)$/) { push @files, $1 }
			}
			close(FILE);

		} else {
			print "\n\n\tError: $nagios_dir/cgi.cfg does not exist\n";
			$errors = 1;
		}
		unless ($errors) {
			foreach my $file (@files) {
				system("chmod 660 $file");
			}
		}	
		if ($resource_file =~ /(.*)\/.*\.cfg/) {
			system("chmod 770 $1");
		}
		system("chmod 660 $resource_file");
	}
}

sub collect_info() {
	my $validated = 0;
	print "\n\n\tThis script will help you setup Monarch on your system.";
	print "\n\n\tDefaults and guesses will be provided where possible.\n";

	print "\n\n\n\tWhat is the name of the user that will have access to the database? [$databaseuser] :";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $databaseuser = $input }

	print "\n\n\n\tWhat is the password for user $databaseuser to access the database [$databasepasswd] :";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $databasepasswd = $input }

	print "\n\n\n\tWhat is the name of the database you will use for your Monarch installation? [$database] :";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $database = $input }
	
	print "\n\n\n\tEnter web server's user account [$web_user] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $web_user = $input }

	print "\n\n\n\tEnter web server's user group [$web_group] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $web_group = $input }

	unless (-e $doc_root) { 
		if (-e "/var/www/htdocs") {
			$doc_root = "/var/www/htdocs";
		} else {
			$doc_root = "unknown";
		}
	} 
	$validated = 0;
	until ($validated) {
		if ($doc_root) {
			print "\n\n\n\tWhat is your web server's document root [$doc_root] :";
		} else {
			print "\n\n\n\tWhat is the full path to your web server's document root :";
		}
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $doc_root = $input }
		if (-e $doc_root) { 
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid entry $doc_root does not exist.";
		}
	}

	unless (-e $cgibin) { $cgibin = "unknown" }
	$validated = 0;
	until ($validated) {
		print "\n\n\n\tEnter the full path of your cgi-bin directory [$cgibin] : ";
		my $input = <STDIN>;
		chomp $input;
		if ($input) { $cgibin = $input }
		if (-e $cgibin) { 
			$validated = 1;
		} else {
			print "\n\n\n\tError: Invalid entry $doc_root does not exist.";
		}
	}

	print "\n\n\n\tEnter the full installation path for Monarch [$monarch_home] : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input) { $monarch_home = $input }

	print "\n\n\n\tWould you like to set permissions so that Monarch can read your nagios cfg files including resource.cfg?\n\t\t(Note: You will need to manage this manually if your answer is no). yes|no : ";
	my $input = <STDIN>;
	chomp $input;
	if ($input =~ /^yes$/i) { 
		$set_perms = $input; 
	} else {
		$set_perms = 'no'; 
	}
	unless (-e $nagios_dir) { 
		if (-e "/usr/local/nagios/etc") {
			$nagios_dir = "/usr/local/nagios/etc";
		} else {
			$nagios_dir = "unknown";
		}
	} 

	my $nagios_disp = undef;
	unless ($set_perms eq 'no') {	
		$validated = 0;
		until ($validated) {
			print "\n\n\n\tEnter the full path to your nagios.cfg file [$nagios_dir] : ";
			my $input = <STDIN>;
			chomp $input;
			if ($input) { $nagios_dir = $input }
			$nagios_disp = "Nagios cfg dir	= $nagios_dir\n";
			if (-e $nagios_dir) {
				$validated = 1;
			} else {
				print "\n\n\n\tError: Invalid entry $nagios_dir does not exist.";
			}
		}
	}
	if ($database && $databaseuser && $doc_root && $cgibin && $monarch_home) {
		print qq(
	
			Database Name	= $database
			Database User	= $databaseuser
			Database Passwd = $databasepasswd
			DocumentRoot	= $doc_root
			Web user	= $web_user
			Web user group	= $web_group
			cgi-bin		= $cgibin
			Install path	= $monarch_home
			Set permissions	= $set_perms
			$nagios_disp
		Be absolutely certain these values are correct. Otherwise, permissions will be set incorrectly and Monarch will not work.
		Does the information look correct? 'yes' or 'no' : );
		my $response = <STDIN>;
		if ($response =~ /yes/i) {
			distribution();
		} else {
			collect_info();
		}
	} else {
		collect_info();
	}
}


sub save_settings() {
	print qq(
	Update complete!

		Database Name	= $database
		Database User	= $databaseuser
		Database Passwd = $databasepasswd
		DocumentRoot	= $doc_root
		Web user	= $web_user
		Web user group	= $web_group
		cgi-bin		= $cgibin
		install path	= $monarch_home
		nagios cfg dir	= $nagios_dir
		Set permissions	= $set_perms
	
	Would you like to save these settings for updates run from this folder (yes|no)? [yes] :);

	my $input = <STDIN>;
	chomp $input;
	unless ($input = /^no$/i) {
		my $outfile = qq(##########GROUNDWORK#############################################################################################
#GW
#GW\tMonarch update saved settings
#GW
##########GROUNDWORK#############################################################################################

MONARCH_DB=$database
MONARCH_DB_USER=$databaseuser
MONARCH_DB_PASSWORD=$databasepasswd
WEB_USER=$web_user
WEB_GROUP=$web_group
DOCUMENT_ROOT=$doc_root
CGI_BIN=$cgibin
INSTALL_PATH=$monarch_home
NAGIOS_PATH=$nagios_dir
SET_PERMS=$set_perms
);
		open(FILE, "> ./monarch_update.db") || print "\n./monarch_update.db $!";
		print FILE $outfile;
		close(FILE);
	}
}


check_prereqs();
if (-e "./monarch_update.db") {
	open(FILE, "< ./monarch_update.db") || print "\n\n./monarch_update.db $!";
	my $out_to_file = undef;
	while (my $line = <FILE>) {
		if ($line =~ /MONARCH_DB=(.*?)$/) {
			$database = $1;
		} elsif ($line =~ /MONARCH_DB_USER=(.*?)$/) {
			$databaseuser = $1;
		} elsif ($line =~ /MONARCH_DB_PASSWORD=(.*?)$/) {
			$databasepasswd = $1;
		} elsif ($line =~ /DOCUMENT_ROOT=(.*?)$/) {
			$doc_root = $1;
		} elsif ($line =~ /CGI_BIN=(.*?)$/) {
			$cgibin = $1;
		} elsif ($line =~ /INSTALL_PATH=(.*?)$/) {
			$monarch_home = $1;
		} elsif ($line =~ /NAGIOS_PATH=(.*?)$/) {
			$nagios_dir = $1;
		} elsif ($line =~ /SET_PERMS=(.*?)$/) {
			$set_perms = $1;
		}
		$out_to_file .= $line;
	}
	close(FILE);
	print qq(

		Database Name	= $database
		Database User	= $databaseuser
		Database Passwd = $databasepasswd
		DocumentRoot	= $doc_root
		Web user	= $web_user
		Web user group	= $web_group
		cgi-bin		= $cgibin
		install path	= $monarch_home
		nagios dir	= $nagios_dir
		Set permissions	= $set_perms

	Does information look correct? 'yes' or 'no' : );
	my $response = <STDIN>;
	if ($response =~ /yes/i) {
		distribution();
		table_update();
	} else {
		collect_info();
	}
} else {
	collect_info();
}
save_settings();

